/*
 * Version: 1.0
 *
 * The contents of this file are subject to the OpenVPMS License Version
 * 1.0 (the 'License'); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.openvpms.org/license/
 *
 * Software distributed under the License is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Copyright 2021 (C) OpenVPMS Ltd. All Rights Reserved.
 */

package org.openvpms.maven.archetype;

import org.apache.maven.model.Build;
import org.apache.maven.plugin.testing.AbstractMojoTestCase;
import org.apache.maven.plugin.testing.stubs.MavenProjectStub;

import java.io.File;
import java.io.FileInputStream;
import java.util.Properties;

/**
 * Tests the {@link ArchetypeChecksumMojo}.
 *
 * @author Tim Anderson
 */
public class ArchetypeChecksumMojoTestCase extends AbstractMojoTestCase {

    /**
     * Verifies that the mojo can be execute.
     *
     * @throws Exception for any error
     */
    public void testMojoExecution() throws Exception {
        File pluginXml = new File(getBasedir(), "target/test-classes/plugin-config-checksum.xml");
        ArchetypeChecksumMojo mojo = (ArchetypeChecksumMojo) lookupMojo("checksum", pluginXml);
        MavenProjectStub project = new MavenProjectStub();
        project.setBuild(new Build());
        File file = new File("target/tmp/META-INF");
        if (!file.exists()) {
            assertTrue(file.mkdirs());
        }
        project.getBuild().setOutputDirectory("target/tmp");
        mojo.setProject(project);
        mojo.execute();

        Properties properties = new Properties();
        properties.load(new FileInputStream("target/tmp/META-INF/openvpms-archetypes.properties"));
        String checksum = properties.getProperty("checksum.default");
        assertEquals("a09512a", checksum);
    }
}
