/*
 * Version: 1.0
 *
 * The contents of this file are subject to the OpenVPMS License Version
 * 1.0 (the 'License'); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.openvpms.org/license/
 *
 * Software distributed under the License is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Copyright 2021 (C) OpenVPMS Ltd. All Rights Reserved.
 */

package org.openvpms.tool.toolbox.plugin;

import org.openvpms.component.business.dao.im.plugin.PluginDAO;
import org.openvpms.tool.toolbox.AbstractApplicationContextCommand;
import picocli.CommandLine.Command;
import picocli.CommandLine.Parameters;

/**
 * Command to uninstall a plugin, given its key.
 *
 * @author Tim Anderson
 */
@Command(name = "--uninstall", description = "Uninstalls a plugin")
public class UninstallPluginCommand extends AbstractApplicationContextCommand {

    /**
     * The plugin key.
     */
    @Parameters(description = "the plugin key", index = "0", arity = "1")
    private String key;

    /**
     * Runs the command.
     *
     * @return {@code 0} indicates success, non-zero failure
     * @throws Exception for any error
     */
    @Override
    protected int run() throws Exception {
        int result = 1;
        PluginDAO dao = getBean(PluginDAO.class);
        if (dao.remove(key)) {
            System.out.println("Plugin removed. If OpenVPMS is already running it needs to be restarted.");
            result = 0;
        } else {
            System.out.println("Plugin not found");
        }
        return result;
    }
}
