CREATE TABLE IF NOT EXISTS `entity_links` (
  `id`                BIGINT(20)   NOT NULL AUTO_INCREMENT,
  `version`           BIGINT(20)   NOT NULL,
  `linkId`            VARCHAR(36)  NOT NULL,
  `arch_short_name`   VARCHAR(100) NOT NULL,
  `arch_version`      VARCHAR(100) NOT NULL,
  `name`              VARCHAR(255) DEFAULT NULL,
  `description`       VARCHAR(255) DEFAULT NULL,
  `active_start_time` DATETIME DEFAULT NULL,
  `active_end_time`   DATETIME DEFAULT NULL,
  `sequence`          INT(11)      NOT NULL,
  `source_id`         BIGINT(20) DEFAULT NULL,
  `target_id`         BIGINT(20) DEFAULT NULL,
  PRIMARY KEY (`id`),
  KEY `FK3FCDE05DA5B2869` (`source_id`),
  KEY `FK3FCDE05D3EFA2333` (`target_id`),
  CONSTRAINT `FK3FCDE05D3EFA2333` FOREIGN KEY (`target_id`) REFERENCES `entities` (`entity_id`),
  CONSTRAINT `FK3FCDE05DA5B2869` FOREIGN KEY (`source_id`) REFERENCES `entities` (`entity_id`)
    ON DELETE CASCADE
)
  ENGINE =InnoDB
  DEFAULT CHARSET =utf8;

CREATE TABLE IF NOT EXISTS `entity_link_details` (
  `id`    BIGINT(20)    NOT NULL,
  `type`  VARCHAR(255)  NOT NULL,
  `value` VARCHAR(5000) NOT NULL,
  `name`  VARCHAR(255)  NOT NULL,
  PRIMARY KEY (`id`, `name`),
  KEY `FKD2E4DDD9A03946DF` (`id`),
  CONSTRAINT `FKD2E4DDD9A03946DF` FOREIGN KEY (`id`) REFERENCES `entity_links` (`id`)
)
  ENGINE =InnoDB
  DEFAULT CHARSET =utf8;

#
# Add description column to node_descriptors for OBF-229
#
DELIMITER $$
CREATE PROCEDURE alter_node_descriptors()
  BEGIN
    DECLARE _count INT;
    SET _count = (SELECT
                    COUNT(*)
                  FROM INFORMATION_SCHEMA.COLUMNS
                  WHERE TABLE_NAME = 'node_descriptors' AND TABLE_SCHEMA = DATABASE() AND
                        COLUMN_NAME = 'description');
    IF _count = 0
    THEN
      ALTER TABLE node_descriptors
      ADD COLUMN description VARCHAR(255) DEFAULT NULL
      AFTER name;
    END IF;
  END $$
DELIMITER ;

CALL alter_node_descriptors();
DROP PROCEDURE alter_node_descriptors;

#
# Migrate lookup.macroReport report node, for OBF-232
#
UPDATE lookup_details d, lookups l
SET d.type = "object-reference",
  d.value =  concat(substring_index(substring_index(d.value, "<shortName>", -1), "</shortName>", 1), ':',
                    substring_index(substring_index(d.value, "<id>", -1), "</id>", 1), ':',
                    substring_index(substring_index(d.value, "<linkId>", -1), "</linkId>", 1))
WHERE l.lookup_id = d.lookup_id
      AND arch_short_name = "lookup.macroReport"
      AND d.name = "report"
      AND d.type = "org.openvpms.component.business.domain.im.common.IMObjectReference"
      AND d.value LIKE "<org.openvpms.component.business.domain.im.common.IMObjectReference>%";

#
# Update party.organisationPractice for OVPMS-1472 Update to the Customer and Patient Summaries
#
INSERT INTO entity_details (entity_id, name, type, value)
  SELECT
    DISTINCT e.entity_id,
    "showCustomerAccountSummary",
    "boolean",
    "true"
  FROM entities e
  WHERE e.arch_short_name = "party.organisationPractice"
        AND NOT exists(SELECT
                         *
                       FROM entity_details d
                       WHERE d.entity_id = e.entity_id AND d.name = "showCustomerAccountSummary");

#
# Update party.organisationPractice for OVPMS-1531 Screen auto lock
#
INSERT INTO entity_details (entity_id, name, type, value)
  SELECT
    DISTINCT e.entity_id,
    "autoLockScreen",
    "int",
    "0"
  FROM entities e
  WHERE e.arch_short_name = "party.organisationPractice"
        AND NOT exists(SELECT
                         *
                       FROM entity_details d
                       WHERE d.entity_id = e.entity_id AND d.name = "autoLockScreen");


#
# Migrate act.patientClinicalEvent, lookup.appointmentReason for OVPMS-1498
#
UPDATE acts
SET title = reason
WHERE arch_short_name = "act.patientClinicalEvent";

UPDATE acts
SET reason = null
WHERE arch_short_name = "act.patientClinicalEvent" AND title = reason;

UPDATE lookups
SET arch_short_name = "lookup.visitReason"
WHERE arch_short_name = "lookup.appointmentReason";

#
# Migrate party.organisationPractice for OVPMS-1511
#
UPDATE entities e
  JOIN entity_details d
    ON e.entity_id = d.entity_id AND e.arch_short_name = "party.organisationPractice"
       AND d.name = "reminderExportFormat"
SET d.name = "fileExportFormat";

#
#
# Migrate act.patientClinicalProblem for OVPMS-1468
#
DROP TABLE IF EXISTS migrate_problems;
CREATE TABLE migrate_problems (
  act_id  BIGINT PRIMARY KEY,
  problem VARCHAR(255),
  code    VARCHAR(255)
);

INSERT INTO migrate_problems (act_id, problem, code)
  SELECT
    a.act_id,
    d.value,
    upper(d.value)
  FROM acts a JOIN act_details d
      ON a.act_id = d.act_id AND d.name = "problem"
  WHERE a.arch_short_name = "act.patientClinicalProblem" AND d.value IS NOT null AND d.value <> "";

UPDATE migrate_problems
SET code = replace(code, " ", "_");

UPDATE migrate_problems
SET code = replace(code, ".", "_");

UPDATE migrate_problems
SET code = replace(code, "&", "_");

UPDATE migrate_problems
SET code = replace(code, "/", "_");

UPDATE migrate_problems
SET code = replace(code, "\\", "_");

UPDATE migrate_problems
SET code = replace(code, "(", "_");

UPDATE migrate_problems
SET code = replace(code, ")", "_");

UPDATE migrate_problems
SET code = replace(code, "*", "_");

UPDATE migrate_problems
SET code = replace(code, ",", "_");

UPDATE migrate_problems
SET code = replace(code, "-", "_");

UPDATE migrate_problems
SET code = replace(code, "[", "_");

UPDATE migrate_problems
SET code = replace(code, "]", "_");

UPDATE migrate_problems
SET code = replace(code, "__", "_");

INSERT INTO lookups (version, linkId, arch_short_name, active, arch_version, code, name, description, default_lookup)
  SELECT
    0,
    UUID(),
    'lookup.diagnosis',
    1,
    '1.0',
    code,
    problem,
    null,
    0
  FROM (SELECT
          code,
          problem
        FROM migrate_problems
        GROUP BY code) a
  WHERE NOT exists(SELECT
                     *
                   FROM lookups e
                   WHERE e.arch_short_name = 'lookup.diagnosis' AND e.code = a.code);

UPDATE acts a,
  migrate_problems p
SET a.reason = p.code
WHERE a.act_id = p.act_id;

DELETE d
FROM acts a,
  act_details d,
  migrate_problems p
WHERE a.act_id = d.act_id AND a.act_id = p.act_id
      AND d.name = "problem";

DROP TABLE migrate_problems;

#
#
DROP TABLE IF EXISTS file_name_formats;
CREATE TABLE file_name_formats (
  code        VARCHAR(255) PRIMARY KEY,
  name        VARCHAR(255),
  description VARCHAR(255),
  expression  VARCHAR(5000)
);

#
# Add file name formats, for OVPMS-1483
#
INSERT INTO file_name_formats (code, name, description, expression)
  VALUES ("DOC_NAME_DOC_DATE", "Document name - document date", "Valid for customer, patient, and supplier documents only", "concat($file, ' - ', date:format(openvpms:get(.,'startTime'), 'd MMM yyyy'))"),
  ("DOC_NAME_TODAYS_DATE", "Document name - today's date", "Valid for all documents", "concat($file, ' - ', date:format(java.util.Date.new(), 'd MMM yyyy'))"),
  ("DOC_NAME_PATIENT_FULL_NAME_DOC_DATE", "Document name - patient full name - document date", "Valid for patient documents only", "concat($file, ' - ', $patient.name, ' ', $customer.lastName, ' - ', date:format(openvpms:get(.,'startTime'), 'd MMM yyyy'))"),
  ("DOC_NAME_PATIENT_FULL_NAME_TODAYS_DATE", "Document name - patient full name - today's date", "Valid for patient documents only", "concat($file, ' - ', $patient.name, ' ', $customer.lastName, ' - ', date:format(java.util.Date.new(), 'd MMM yyyy'))"),
  ("DOC_NAME_CUSTOMER_FULL_NAME_DOC_DATE", "Document name - customer full name - document date", "Valid for customer documents only", "concat($file, ' - ', party:getPartyFullName($customer), ' - ', date:format(openvpms:get(.,'startTime'), 'd MMM yyyy'))"),
  ("DOC_NAME_CUSTOMER_FULL_NAME_TODAYS_DATE", "Document name - customer full name - today's date", "Valid for customer documents only", "concat($file, ' - ', party:getPartyFullName($customer), ' - ', date:format(java.util.Date.new(), 'd MMM yyyy'))"),
  ("DOC_NAME_SUPPLIER_FULL_NAME_DOC_DATE", "Document name - supplier full name - document date", "Valid for supplier documents only", "concat($file, ' - ', party:getPartyFullName($supplier), ' - ', date:format(openvpms:get(.,'startTime'), 'd MMM yyyy'))"),
  ("DOC_NAME_SUPPLIER_FULL_NAME_TODAYS_DATE", "Document name - supplier full name - today's date", "Valid for supplier documents only", "concat($file, ' - ', party:getPartyFullName($supplier), ' - ', date:format(java.util.Date.new(), 'd MMM yyyy'))");

INSERT INTO lookups (version, linkId, arch_short_name, active, arch_version, code, name, description, default_lookup)
  SELECT
    0,
    UUID(),
    'lookup.fileNameFormat',
    1,
    '1.0',
    code,
    name,
    description,
    0
  FROM (SELECT
          code,
          name,
          description
        FROM file_name_formats a
        WHERE NOT exists(SELECT
                           *
                         FROM lookups e
                         WHERE e.arch_short_name = 'lookup.fileNameFormat' AND e.code = a.code)) l;

INSERT INTO lookup_details (lookup_id, name, type, value)
  SELECT
    l.lookup_id,
    "expression",
    "string",
    a.expression
  FROM file_name_formats a,
    lookups l
  WHERE a.code = l.code AND l.arch_short_name = "lookup.fileNameFormat"
        AND NOT exists(SELECT
                         *
                       FROM lookup_details d
                       WHERE d.lookup_id = l.lookup_id AND d.name = 'expression');

DROP TABLE file_name_formats;

#
# Add VeNom diagnosis codes for OVPMS-1468
#
DROP TABLE IF EXISTS venom_diagnosis;
CREATE TABLE venom_diagnosis (
  code         VARCHAR(255) PRIMARY KEY,
  dictionaryId VARCHAR(255),
  term         VARCHAR(255),
  modelling    VARCHAR(255));
INSERT INTO venom_diagnosis (code, dictionaryId, term, modelling) VALUES
('326-VENOM', '326', 'Abdominal wall rupture', 'Abdominal finding, Traumatic episode finding'),
('328-VENOM', '328', 'Abscess', 'Inflammatory disorder finding'),
('329-VENOM', '329', 'Abscess - brain', 'Inflammatory disorder finding, Nervous system finding, Head and neck finding'),
('330-VENOM', '330', 'Abscess - Cat bite abscess (CBA)', 'Inflammatory disorder finding, Wound finding, Traumatic episode finding'),
('7503-VENOM', '7503', 'Abscess - dog bite abscess', 'Inflammatory disorder finding, Wound finding, Traumatic episode finding'),
('331-VENOM', '331', 'Abscess - hepatic (liver)', 'Inflammatory disorder finding, Hepatobiliary system finding, Abdominal finding'),
('332-VENOM', '332', 'Abscess - neck (cervical)', 'Inflammatory disorder finding, Head and neck finding'),
('333-VENOM', '333', 'Abscess - oral (mouth)', 'Inflammatory disorder finding, Alimentary tract/Digestive system finding, Head and neck finding'),
('334-VENOM', '334', 'Abscess - oral (mouth), dental (tooth)', 'Inflammatory disorder finding, Alimentary tract/Digestive system finding, Head and neck finding'),
('335-VENOM', '335', 'Abscess - pancreatic', 'Inflammatory disorder finding, Alimentary tract/Digestive system finding, Abdominal finding'),
('336-VENOM', '336', 'Abscess - peritoneal', 'Inflammatory disorder finding, Abdominal finding'),
('337-VENOM', '337', 'Abscess - prostatic', 'Inflammatory disorder finding, Urogenital system finding, Reproductive system finding, Pelvic structure finding'),
('15535-VENOM', '15535', 'Abscess - pulmonary (lung)', 'Inflammatory disorder finding, Respiratory system finding, Thoracic finding'),
('338-VENOM', '338', 'Abscess - retrobulbar', 'Inflammatory disorder finding, Ophthalmology/Vision finding, Head and neck finding'),
('339-VENOM', '339', 'Abscess - retropharyngeal', 'Inflammatory disorder finding, Respiratory system finding, Alimentary tract/Digestive system finding, Head and neck finding'),
('340-VENOM', '340', 'Abscess - spinal cord', 'Inflammatory disoder finding, Nervous system finding'),
('341-VENOM', '341', 'Abscess - splenic', 'Inflammatory disorder finding, Haematopoietic system finding, Abdominal finding'),
('342-VENOM', '342', 'Abscess - testicular', 'Inflammatory disorder finding, Reproductive system finding, Urogenital system finding'),
('16477-VENOM', '16477', 'Acanthomatous adamantinoma - mandible', 'Mass lesion/swelling finding, Alimentary tract/Digestive system finding, Soft tissue finding, Head and neck finding'),
('16478-VENOM', '16478', 'Acanthomatous adamantinoma - maxilla', 'Mass lesion/swelling finding, Alimentary tract/Digestive system finding, Soft tissue finding, Head and neck finding'),
('4559-VENOM', '4559', 'Accelerated idioventricular rhythm', 'Cardiovascular system finding, Thoracic finding'),
('343-VENOM', '343', 'Acne', 'Haircoat/skin (+ appendages) finding'),
('344-VENOM', '344', 'Acne - Canine acne', 'Haircoat/skin (+ appendages) finding'),
('345-VENOM', '345', 'Acne - Feline acne', 'Haircoat/skin (+ appendages) finding'),
('346-VENOM', '346', 'Acne - Juvenile acne', 'Haircoat/skin (+ appendages) finding'),
('347-VENOM', '347', 'Acral lick dermatitis (granuloma)', 'Haircoat/skin (+ appendages) finding, Inflammatory disorder finding'),
('2341-VENOM', '2341', 'Acral lick furunculosis', 'Haircoat/skin (+ appendages) finding, Infectious disorder finding'),
('349-VENOM', '349', 'Acral mutilation', 'Haircoat/skin (+ appendages) finding, Nervous system finding, Hereditary disorder finding'),
('7514-VENOM', '7514', 'Acrochordon (Fibroepithelial polyp, Skin tag)', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('350-VENOM', '350', 'Acromegaly', 'Endocrine/Metabolic finding, Nervous system finding, Head and neck finding'),
('351-VENOM', '351', 'Actinic keratosis', 'Haircoat/skin (+ appendages) finding'),
('11729-VENOM', '11729', 'Acute kidney (renal) failure (ARF) – azotaemic (urine output unspecified)', 'Urogenital system finding, Endocrine/Metabolic finding, Abdominal finding'),
('11731-VENOM', '11731', 'Acute kidney (renal) failure (ARF) – azotaemic, anuric', 'Urogenital system finding, Endocrine/Metabolic finding, Abdominal finding'),
('11733-VENOM', '11733', 'Acute kidney (renal) failure (ARF) – azotaemic, oliguric', 'Urogenital system finding, Endocrine/Metabolic finding, Abdominal finding'),
('11735-VENOM', '11735', 'Acute kidney (renal) failure (ARF) – azotaemic, polyuric', 'Urogenital system finding, Endocrine/Metabolic finding, Abdominal finding'),
('11737-VENOM', '11737', 'Acute kidney (renal) injury (AKI) – non-azotaemic', 'Urogenital system finding, Abdominal finding'),
('352-VENOM', '352', 'Acute lung injury (ALI)', 'Respiratory system finding, Thoracic finding'),
('353-VENOM', '353', 'Acute respiratory distress syndrome (ARDS)', 'Respiratory system finding, Thoracic finding'),
('11745-VENOM', '11745', 'Acute-on-chronic kidney (renal) disease', 'Urogenital system finding, Abdominal finding'),
('355-VENOM', '355', 'Addisonian crisis (severe adrenocortical insufficiency)', 'Endocrine/Metabolic finding, Abdominal finding'),
('354-VENOM', '354', 'Addison''s disease (primary hypoadrenocorticism)', 'Endocrine/Metabolic finding, Abdominal finding'),
('16672-VENOM', '16672', 'Adenocarcinoma - adrenocortical', 'Endocrine/Metabolic finding, Abdominal finding, Mass lesion/swelling finding'),
('358-VENOM', '358', 'Adenocarcinoma - apocrine adenocarcinoma of anal sac', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding'),
('359-VENOM', '359', 'Adenocarcinoma - aural (ear)', 'Auditory system finding, Head and neck finding, Mass lesion/swelling finding'),
('16674-VENOM', '16674', 'Adenocarcinoma - caecum', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding'),
('16683-VENOM', '16683', 'Adenocarcinoma - gallbladder', 'Hepatobiliary system finding, Mass lesion/swelling finding, Abdominal finding'),
('362-VENOM', '362', 'Adenocarcinoma - hepatic (liver)', 'Hepatobiliary system finding, Mass lesion/swelling finding, Abdominal finding'),
('16487-VENOM', '16487', 'Adenocarcinoma - large intestinal', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding'),
('16685-VENOM', '16685', 'Adenocarcinoma - mammary', 'Reproductive system finding, Mass lesion/swelling finding'),
('16687-VENOM', '16687', 'Adenocarcinoma - metastatic', 'Mass lesion/swelling finding'),
('16689-VENOM', '16689', 'Adenocarcinoma - nasal (nose)(site unspecified)', 'Mass lesion/swelling finding, Respiratory system finding, Head and neck finding'),
('16694-VENOM', '16694', 'Adenocarcinoma - nasal (paranasal) sinus', 'Mass lesion/swelling finding, Respiratory system finding, Head and neck finding'),
('16692-VENOM', '16692', 'Adenocarcinoma - nasal cavity', 'Mass lesion/swelling finding, Respiratory system finding, Head and neck finding'),
('363-VENOM', '363', 'Adenocarcinoma - oesophageal', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding'),
('16696-VENOM', '16696', 'Adenocarcinoma - pancreatic', 'Mass lesion/swelling finding, Alimentary tract/Digestive system finding, Abdominal finding'),
('11107-VENOM', '11107', 'Adenocarcinoma - perianal gland', 'Haircoat/skin (+ appendages) finding, Perianal finding, Mass lesion/swelling finding'),
('364-VENOM', '364', 'Adenocarcinoma - prostatic', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding, Pelvic structure finding'),
('365-VENOM', '365', 'Adenocarcinoma - pulmonary (lung)', 'Respiratory system finding, Mass lesion/swelling finding, Thoracic finding'),
('366-VENOM', '366', 'Adenocarcinoma - rectal', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Pelvic structure finding'),
('367-VENOM', '367', 'Adenocarcinoma - renal (kidney)', 'Urogenital system finding, Mass lesion/swelling finding, Abdominal finding'),
('16492-VENOM', '16492', 'Adenocarcinoma - salivary gland', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Head and neck finding'),
('16481-VENOM', '16481', 'Adenocarcinoma - sebaceous gland', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('16490-VENOM', '16490', 'Adenocarcinoma - small intestinal', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding'),
('16485-VENOM', '16485', 'Adenocarcinoma - stomach (gastric)', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding'),
('16483-VENOM', '16483', 'Adenocarcinoma - sweat gland', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('16698-VENOM', '16698', 'Adenocarcinoma - tracheal', 'Respiratory system finding, Mass lesion/swelling, Head and neck finding'),
('16700-VENOM', '16700', 'Adenocarcinoma - urethral', 'Mass lesion/swelling finding, Urogenital system finding'),
('16702-VENOM', '16702', 'Adenocarcinoma - urinary bladder', 'Mass lesion/swelling finding, Urogenital system finding'),
('16704-VENOM', '16704', 'Adenocarcinoma - uterine', 'Reproductive system finding, Mass lesion/swelling finding, Abdominal finding'),
('16414-VENOM', '16414', 'Adenocarcinoma (site unspecified)', 'Mass lesion/swelling finding'),
('16724-VENOM', '16724', 'Adenoma - adrenocortical', 'Endocrine/Metabolic finding, Abdominal finding, Mass lesion/swelling finding'),
('370-VENOM', '370', 'Adenoma - apocrine adenoma of anal sac', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding'),
('16726-VENOM', '16726', 'Adenoma - bronchioalveolar', 'Mass lesion/swelling finding, Respiratory system finding, Thoracic finding'),
('16729-VENOM', '16729', 'Adenoma - gallbladder', 'Mass lesion/swelling finding, Hepatobiliary system finding, Abdominal finding'),
('16731-VENOM', '16731', 'Adenoma - hepatic (liver)', 'Mass lesion/swelling finding, Hepatobiliary system finding, Abdominal finding'),
('16734-VENOM', '16734', 'Adenoma - iridociliary', 'Ophthalmology/vision finding, Mass lesion/swelling finding, Head and neck finding'),
('16736-VENOM', '16736', 'Adenoma - large intestine (site unspecified)', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding'),
('16739-VENOM', '16739', 'Adenoma - mammary (type unspecified)', 'Reproductive system finding, Mass lesion/swelling finding'),
('16741-VENOM', '16741', 'Adenoma - mammary, basaloid', 'Reproductive system finding, Mass lesion/swelling finding'),
('16743-VENOM', '16743', 'Adenoma - mammary, complex', 'Reproductive system finding, Mass lesion/swelling finding'),
('16745-VENOM', '16745', 'Adenoma - mammary, simple', 'Reproductive system finding, Mass lesion/swelling finding'),
('16748-VENOM', '16748', 'Adenoma - nasal (nose)(site unspecified)', 'Mass lesion/swelling finding, Respiratory system finding, Head and neck finding'),
('16752-VENOM', '16752', 'Adenoma - nasal (paranasal) sinus', 'Mass lesion/swelling finding, Respiratory system finding, Head and neck finding'),
('16750-VENOM', '16750', 'Adenoma - nasal cavity', 'Mass lesion/swelling finding, Respiratory system finding, Head and neck finding'),
('16754-VENOM', '16754', 'Adenoma - ovarian', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding, Abdominal finding'),
('16756-VENOM', '16756', 'Adenoma - pancreatic', 'Mass lesion/swelling finding, Alimentary tract/Digestive system finding, Abdominal finding'),
('16758-VENOM', '16758', 'Adenoma - parathyroid', 'Endocrine/metabolic finding, Mass lesion/swelling finding, Head and neck finding'),
('11105-VENOM', '11105', 'Adenoma - perianal gland', 'Haircoat/skin (+ appendages) finding, Perianal finding, Mass lesion/swelling finding'),
('16760-VENOM', '16760', 'Adenoma - pituitary (type unspecified)', 'Nervous system finding, Endocrine/Metabolic finding, Mass lesion/swelling finding, Head and neck finding'),
('16762-VENOM', '16762', 'Adenoma - rectum', 'Mass lesion/swelling finding, Alimentary tract/Digestive system finding, Pelvic structure finding'),
('16764-VENOM', '16764', 'Adenoma - renal (kidney)', 'Mass lesion/swelling finding, Urogenital system finding, Abdominal finding'),
('16781-VENOM', '16781', 'Adenoma - sebaceous gland', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('16778-VENOM', '16778', 'Adenoma - small intestine', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding'),
('16779-VENOM', '16779', 'Adenoma - stomach (gastric)', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding'),
('16782-VENOM', '16782', 'Adenoma - sweat gland', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('16784-VENOM', '16784', 'Adenoma - thyroid', 'Endocrine/metabolic finding, Mass lesion/swelling finding, Head and neck finding'),
('16786-VENOM', '16786', 'Adenoma - urinary bladder', 'Mass lesion/swelling finding, Urogenital system finding, Abdominal finding'),
('16788-VENOM', '16788', 'Adenoma - uterine', 'Reproductive system finding, Mass lesion/swelling finding, Abdominal finding'),
('16722-VENOM', '16722', 'Adenoma (site unspecified)', 'Mass lesion/swelling finding'),
('371-VENOM', '371', 'Adverse reaction to blood transfusion', 'Complication of medical care finding, Adverse reaction finding'),
('15175-VENOM', '15175', 'Adverse reaction to drug – anti-parasitic preparation', 'Drug-related disorder finding, Adverse reaction finding'),
('15179-VENOM', '15179', 'Adverse reaction to drug – non-steroidal anti-inflammatory drug (NSAID)(unspecified)', 'Drug-related disorder finding, Adverse reaction finding'),
('15181-VENOM', '15181', 'Adverse reaction to drug – NSAID – carprofen', 'Drug-related disorder finding, Adverse reaction finding'),
('15183-VENOM', '15183', 'Adverse reaction to drug – NSAID – ibuprofen', 'Drug-related disorder finding, Adverse reaction finding'),
('15185-VENOM', '15185', 'Adverse reaction to drug – NSAID – meloxicam', 'Drug-related disorder finding, Adverse reaction finding'),
('15171-VENOM', '15171', 'Adverse reaction to drug (unspecified)', 'Drug-related disorder finding, Adverse reaction finding'),
('374-VENOM', '374', 'Adverse reaction to food', 'Adverse reaction finding'),
('15227-VENOM', '15227', 'Adverse reaction to vaccination', 'Adverse reaction finding'),
('15172-VENOM', '15172', 'Adverse reaction to drug – acetaminophen (paracetamol)', 'Drug-related disorder finding, Adverse reaction finding'),
('15177-VENOM', '15177', 'Adverse reaction to drug – digoxin', 'Drug-related disorder finding, Adverse reaction finding'),
('15187-VENOM', '15187', 'Adverse reaction to drug – phenobarbital', 'Drug-related disorder finding, Adverse reaction finding'),
('15189-VENOM', '15189', 'Adverse reaction to drug – steroid', 'Drug-related disorder finding, Adverse reaction finding'),
('376-VENOM', '376', 'Alopecia areata/pseudopelade', 'Haircoat/skin (+ appendages) finding, Immune system finding'),
('377-VENOM', '377', 'Alopecia X', 'Haircoat/skin (+ appendages) finding, Endocrine/metabolic finding'),
('378-VENOM', '378', 'Alopecia/hair coat abnormality', 'Haircoat/skin (+ appendages) finding'),
('379-VENOM', '379', 'Alopecia/hair coat abnormality - glucocorticoid reaction', 'Haircoat/skin (+ appendages) finding, Adverse reaction finding'),
('380-VENOM', '380', 'Alopecia/hair coat abnormality - injection reaction', 'Haircoat/skin (+ appendages) finding, Complication of medical care finding'),
('381-VENOM', '381', 'Alveolitis, fibrosing', 'Respiratory system finding, Thoracic finding'),
('382-VENOM', '382', 'Ameloblastoma of the jaw', 'Musculoskeletal system finding, Mass lesion/swelling finding, Head and neck finding'),
('383-VENOM', '383', 'Amyloidosis', 'Endocrine/metabolic finding'),
('384-VENOM', '384', 'Amyloidosis - hepatic (liver)', 'Hepatobiliary system finding, Endocrine/metabolic finding, Abdominal finding'),
('385-VENOM', '385', 'Amyloidosis - renal (kidney)', 'Urogenital system finding, Endocrine/metabolic finding, Abdominal finding'),
('386-VENOM', '386', 'Anaemia', 'Haematopoietic system finding'),
('387-VENOM', '387', 'Anaemia - aplastic', 'Haematopoietic system finding'),
('11008-VENOM', '11008', 'Anaemia - haemolytic, immune-mediated (IMHA) - primary (idiopathic)', 'Haematopoietic system finding, Immune system finding'),
('11009-VENOM', '11009', 'Anaemia - haemolytic, immune-mediated (IMHA) - secondary', 'Haematopoietic system finding, Immune system finding'),
('11012-VENOM', '11012', 'Anaemia - haemolytic, non-immune mediated', 'Haematopoietic system finding'),
('388-VENOM', '388', 'Anaemia - non-regenerative', 'Haematopoietic system finding'),
('389-VENOM', '389', 'Anaemia - regenerative', 'Haematopoietic system finding'),
('390-VENOM', '390', 'Anaemia due to blood loss (haemorrhage)', 'Haematopoietic system finding'),
('391-VENOM', '391', 'Anaemia of chronic disease', 'Haematopoietic system finding'),
('392-VENOM', '392', 'Anaemia of chronic renal (kidney) failure', 'Haematopoietic system finding, Urogenital system finding, Abdominal finding'),
('393-VENOM', '393', 'Anal atresia (atresia ani)', 'Alimentary tract/Digestive system finding, Congenital finding'),
('394-VENOM', '394', 'Anal furunculosis', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding'),
('395-VENOM', '395', 'Anal sac abscess', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding'),
('396-VENOM', '396', 'Anal sac disorder', 'Alimentary tract/Digestive system finding'),
('397-VENOM', '397', 'Anal sac impaction', 'Alimentary tract/Digestive system finding'),
('2455-VENOM', '2455', 'Anal sac infection', 'Alimentary tract/Digestive system finding, Infectious disorder finding'),
('2456-VENOM', '2456', 'Anal sacculitis', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding'),
('398-VENOM', '398', 'Anal stricture', 'Alimentary tract/Digestive system finding'),
('399-VENOM', '399', 'Anaphylactoid reaction', 'Immune system finding'),
('400-VENOM', '400', 'Anaphylaxis', 'Immune system finding'),
('401-VENOM', '401', 'Ankylosis', 'Musculoskeletal system finding'),
('402-VENOM', '402', 'Ankylosis - hock', 'Musculoskeletal system finding, Limb finding'),
('403-VENOM', '403', 'Ankylosis - stifle', 'Musculoskeletal system finding, Limb finding'),
('9658-VENOM', '9658', 'Antibiotic-responsive diarrhoea (Small intestinal bacterial overgrowth)', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('404-VENOM', '404', 'Aortic aneurysm', 'Cardiovascular system finding'),
('405-VENOM', '405', 'Aortic valve regurgitation/insufficiency', 'Cardiovascular system finding, Thoracic finding'),
('406-VENOM', '406', 'Aortic valve stenosis', 'Cardiovascular system finding, Thoracic finding'),
('407-VENOM', '407', 'Arachnoid cyst - cerebral (brain)', 'Nervous system finding, Mass lesion/swelling finding'),
('408-VENOM', '408', 'Arachnoid cyst - spinal', 'Nervous system finding, Mass lesion/swelling finding'),
('3625-VENOM', '3625', 'Arteriovenous (AV) fistula/malformation', 'Cardiovascular system finding'),
('412-VENOM', '412', 'Arthritis', 'Musculoskeletal system finding'),
('413-VENOM', '413', 'Arthritis - bacterial', 'Musculoskeletal system finding, Infectious disorder finding'),
('414-VENOM', '414', 'Ascites', 'Abdominal finding, Effusion finding'),
('415-VENOM', '415', 'Aspergillosis', 'Infectious disorder finding'),
('416-VENOM', '416', 'Aspergillus dermatitis', 'Infectious disorder finding, Haircoat/skin (+ appendages) finding'),
('4560-VENOM', '4560', 'Atrial fibrillation', 'Cardiovascular system finding, Thoracic finding'),
('4561-VENOM', '4561', 'Atrial flutter', 'Cardiovascular system finding, Thoracic finding'),
('4562-VENOM', '4562', 'Atrial Premature Complexes (APCs)', 'Cardiovascular system finding, Thoracic finding'),
('15230-VENOM', '15230', 'Atrial standstill', 'Cardiovascular system finding, Thoracic finding'),
('4563-VENOM', '4563', 'Atrioventricular block', 'Cardiovascular system finding, Thoracic finding'),
('418-VENOM', '418', 'Aural (ear) haematoma', 'Auditory system finding, Head and neck finding'),
('419-VENOM', '419', 'Auricular chondritis', 'Auditory system finding, Musculoskeletal system finding, Inflammatory disorder finding, Head and neck finding'),
('420-VENOM', '420', 'Avascular necrosis of the femoral head (Legg-Calvé-Perthes disease)', 'Musculoskeletal system finding, Degenerative disorder finding, Limb finding'),
('11757-VENOM', '11757', 'Azotaemia – post-renal', 'Endocrine/Metabolic finding, Urogenital system finding'),
('11755-VENOM', '11755', 'Azotaemia – pre-renal', 'Endocrine/Metabolic finding, Cardiovascular system finding'),
('11759-VENOM', '11759', 'Azotaemia – renal', 'Endocrine/Metabolic finding, Urogenital system finding, Abdominal finding'),
('11753-VENOM', '11753', 'Azotaemia (unspecified)', 'Endocrine/Metabolic finding'),
('423-VENOM', '423', 'Balanitis (glans penis inflammation)', 'Reproductive system finding, Urogenital system finding, Inflammatory disorder finding'),
('16932-VENOM', '16932', 'Basal cell carcinoma (site unspecified)', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('16923-VENOM', '16923', 'Basal cell tumour (site unspecified)', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('16926-VENOM', '16926', 'Basal cell tumour, cutaneous (skin) - forelimb', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding, Limb finding'),
('16924-VENOM', '16924', 'Basal cell tumour, cutaneous (skin) - head', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding, Head and neck finding'),
('16928-VENOM', '16928', 'Basal cell tumour, cutaneous (skin) - neck', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding, Head and neck finding'),
('16930-VENOM', '16930', 'Basal cell tumour, cutaneous (skin) - other', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('16935-VENOM', '16935', 'Basosquamous carcinoma (site unspecified)', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('425-VENOM', '425', 'Behaviour disorder', 'Nervous system finding, Head and neck finding'),
('426-VENOM', '426', 'Benign prostatic hyperplasia (BPH)', 'Urogenital system finding, Reproductive system finding, Pelvic structure finding'),
('427-VENOM', '427', 'Biliary tract disorder', 'Hepatobiliary system finding, Abdominal finding'),
('428-VENOM', '428', 'Biliary tract disorder - obstructive', 'Hepatobiliary system finding, Abdominal finding'),
('429-VENOM', '429', 'Biliary tract disorder - obstructive, extrahepatic', 'Hepatobiliary system finding, Abdominal finding'),
('9483-VENOM', '9483', 'Biliary tract perforation (not gallbladder)', 'Hepatobiliary system finding, Abdominal finding'),
('431-VENOM', '431', 'Biliary tract rupture (not gallbladder)', 'Hepatobiliary system finding, Abdominal finding'),
('432-VENOM', '432', 'Bite injury', 'Wound finding, Traumatic episode finding'),
('433-VENOM', '433', 'Bite injury - cat bite', 'Wound finding, Traumatic episode finding'),
('434-VENOM', '434', 'Bite injury - dog bite', 'Wound finding, Traumatic episode finding'),
('435-VENOM', '435', 'Black hair follicular dysplasia', 'Haircoat/skin (+ appendages) finding'),
('436-VENOM', '436', 'Blepharitis', 'Ophthalmology/vision finding, Inflammatory disorder finding, Head and neck finding'),
('437-VENOM', '437', 'Blepharoedema', 'Ophthalmology/vision finding, Head and neck finding'),
('438-VENOM', '438', 'Blindness', 'Ophthalmology/vision finding, Head and neck finding'),
('439-VENOM', '439', 'Blindness - bilateral', 'Ophthalmology/vision finding, Head and neck finding'),
('440-VENOM', '440', 'Blindness - central', 'Ophthalmology/vision finding, Head and neck finding'),
('441-VENOM', '441', 'Blindness - peripheral', 'Ophthalmology/vision finding, Head and neck finding'),
('442-VENOM', '442', 'Blindness - unilateral', 'Ophthalmology/vision finding, Head and neck finding'),
('443-VENOM', '443', 'Bone implant complication', 'Complication of medical care finding, Musculoskeletal system finding'),
('444-VENOM', '444', 'Bone implant complication - broken or fatigued', 'Complication of medical care finding, Musculoskeletal system finding'),
('445-VENOM', '445', 'Bone implant complication - infected', 'Complication of medical care finding, Musculoskeletal system finding, Infectious disorder finding'),
('446-VENOM', '446', 'Bone implant complication - loosened', 'Complication of medical care finding, Musculoskeletal system finding'),
('447-VENOM', '447', 'Bone implant complication - migrated', 'Complication of medical care finding, Musculoskeletal system finding'),
('448-VENOM', '448', 'Bone marrow disorder', 'Haematopoietic systen finding'),
('449-VENOM', '449', 'Bordetellosis', 'Infectious disorder finding'),
('450-VENOM', '450', 'Botulism', 'Infectious disorder finding'),
('452-VENOM', '452', 'Brachial plexus disorder', 'Nervous system finding, Limb finding'),
('453-VENOM', '453', 'Brachial plexus injury/avulsion', 'Nervous system finding, Limb finding, Traumatic episode finding'),
('454-VENOM', '454', 'Brachycephalic airway obstruction syndrome (BAOS)', 'Respiratory system finding, Congenital finding, Head and neck finding'),
('7378-VENOM', '7378', 'Bradydysrhythmia - unknown', 'Cardiovascular system finding, Thoracic finding'),
('455-VENOM', '455', 'Brain disorder', 'Nervous system finding, Head and neck finding'),
('456-VENOM', '456', 'Brain haemorrhage', 'Nervous system finding, Cardiovascular system finding, Head and neck finding'),
('457-VENOM', '457', 'Brain injury - traumatic (TBI)', 'Nervous system finding, Traumatic episode finding, Head and neck finding'),
('16350-VENOM', '16350', 'Brain malformation (unspecified)', 'Nervous system finding, Head and neck finding'),
('458-VENOM', '458', 'Brainstem disorder', 'Nervous system finding, Head and neck finding'),
('459-VENOM', '459', 'Brainstem herniation', 'Nervous system finding, Head and neck finding'),
('460-VENOM', '460', 'Brainstem infarction', 'Nervous system finding, Cardiovascular system finding, Head and neck finding'),
('461-VENOM', '461', 'Bronchiectasis', 'Respiratory system finding, Thoracic finding'),
('462-VENOM', '462', 'Bronchitis', 'Respiratory system finding, Inflammatory disorder finding, Thoracic finding'),
('463-VENOM', '463', 'Bronchitis - acute', 'Respiratory system finding, Inflammatory disorder finding, Thoracic finding'),
('464-VENOM', '464', 'Bronchitis - allergic', 'Respiratory system finding, Inflammatory disorder finding, Immune system finding, Thoracic finding'),
('465-VENOM', '465', 'Bulla of lung', 'Respiratory system finding, Thoracic finding'),
('466-VENOM', '466', 'Burn injury', 'Traumatic episode finding, Wound finding'),
('467-VENOM', '467', 'Cachexia', 'Weight/Body condition finding, Nutritional finding'),
('468-VENOM', '468', 'Cachexia - cancer', 'Weight/Body condition finding, Nutritional finding'),
('469-VENOM', '469', 'Cachexia - cardiac', 'Weight/Body condition finding, Nutritional finding'),
('470-VENOM', '470', 'Calcinosis circumscripta', 'Endocrine/Metabolic finding, Haircoat/skin (+ appendages) finding, Soft tissue finding'),
('471-VENOM', '471', 'Calcinosis cutis', 'Endocrine/Metabolic finding, Haircoat/skin (+ appendages) finding'),
('472-VENOM', '472', 'Calicivirus infection', 'Infectious disorder finding'),
('473-VENOM', '473', 'Callus', 'Haircoat/skin (+ appendages) finding'),
('474-VENOM', '474', 'Campylobacteriosis', 'Infectious disorder finding'),
('475-VENOM', '475', 'Candidiasis', 'Infectious disorder finding'),
('7028-VENOM', '7028', 'Canine Chiari-like malformation (CM)', 'Nervous system finding, Head and neck finding'),
('476-VENOM', '476', 'Canine chronic bronchitis/obstructive airway disease', 'Respiratory system finding, Thoracic finding'),
('477-VENOM', '477', 'Canine distemper', 'Infectious disorder finding'),
('478-VENOM', '478', 'Canine haemorrhagic gastroenteritis (HGE)', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding, Abdominal finding'),
('9667-VENOM', '9667', 'Canine parvovirus infection', 'Infectious disorder finding'),
('9600-VENOM', '9600', 'Canine sterile neutrophilic dermatitis', 'Haircoat/skin (+ appendages) finding, Inflammatory disorder finding'),
('16941-VENOM', '16941', 'Carcinoid - hepatic (liver)', 'Neuroendocrine system finding, Hepatobiliary system finding, Mass lesion/swelling finding, Abdominal finding'),
('16943-VENOM', '16943', 'Carcinoid - large intestine', 'Neuroendocrine system finding, Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding'),
('16947-VENOM', '16947', 'Carcinoid - small intestine', 'Neuroendocrine system finding, Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding'),
('16949-VENOM', '16949', 'Carcinoid - stomach (gastric)', 'Neuroendocrine system finding, Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding'),
('16938-VENOM', '16938', 'Carcinoid (site unspecified)', 'Neuroendocrine system finding, Mass lesion/swelling finding'),
('16629-VENOM', '16629', 'Carcinoma - adrenocortical', 'Endocrine/Metabolic finding, Abdominal finding, Mass lesion/swelling finding'),
('16499-VENOM', '16499', 'Carcinoma - biliary tract', 'Mass lesion/swelling finding, Hepatobiliary system finding, Abdominal finding'),
('16503-VENOM', '16503', 'Carcinoma - bronchial/bronchiolar', 'Mass lesion/swelling finding, Respiratory system finding, Thoracic finding'),
('16631-VENOM', '16631', 'Carcinoma - choroid plexus', 'Nervous system finding, Head and neck finding, Mass lesion/swelling finding'),
('16523-VENOM', '16523', 'Carcinoma - colon', 'Mass lesion/swelling finding, Alimentary tract/Digestive system finding, Abdominal finding'),
('16507-VENOM', '16507', 'Carcinoma - hepatocellular', 'Mass lesion/swelling finding, Hepatobiliary system finding, Abdominal finding'),
('16553-VENOM', '16553', 'Carcinoma - laryngeal', 'Mass lesion/swelling finding, Respiratory system finding, Head and neck finding'),
('16557-VENOM', '16557', 'Carcinoma - lung (pulmonary) parenchyma', 'Mass lesion/swelling finding, Respiratory system finding, Thoracic finding'),
('16639-VENOM', '16639', 'Carcinoma - mammary (type unspecified)', 'Reproductive system finding, Mass lesion/swelling finding'),
('16641-VENOM', '16641', 'Carcinoma - mammary, complex', 'Reproductive system finding, Mass lesion/swelling finding'),
('16643-VENOM', '16643', 'Carcinoma - mammary, inflammatory', 'Reproductive system finding, Mass lesion/swelling finding'),
('16645-VENOM', '16645', 'Carcinoma - mammary, simple', 'Reproductive system finding, Mass lesion/swelling finding'),
('16570-VENOM', '16570', 'Carcinoma - nasal cavity', 'Mass lesion/swelling finding, Respiratory system finding, Head and neck finding'),
('16572-VENOM', '16572', 'Carcinoma - nasal planum', 'Mass lesion/swelling finding, Respiratory system finding, Head and neck finding'),
('16574-VENOM', '16574', 'Carcinoma - nasopharynx', 'Mass lesion/swelling finding, Respiratory system finding, Alimentary tract/Digestive system finding, Head and neck finding'),
('16578-VENOM', '16578', 'Carcinoma - oesophageal', 'Mass lesion/swelling finding, Alimentary tract/Digestive system finding'),
('16655-VENOM', '16655', 'Carcinoma - ovarian', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding, Abdominal finding'),
('16582-VENOM', '16582', 'Carcinoma - pancreatic', 'Mass lesion/swelling finding, Alimentary tract/Digestive system finding, Abdominal finding'),
('16588-VENOM', '16588', 'Carcinoma - prostatic', 'Mass lesion/swelling finding, Urogenital system finding, Reproductive system finding, Pelvic structure finding'),
('16529-VENOM', '16529', 'Carcinoma - rectal', 'Mass lesion/swelling finding, Alimentary tract/Digestive system finding, Pelvic structure finding'),
('16592-VENOM', '16592', 'Carcinoma - renal (kidney)', 'Mass lesion/swelling finding, Urogenital system finding, Abdominal finding'),
('16596-VENOM', '16596', 'Carcinoma - salivary gland', 'Mass lesion/swelling finding, Alimentary tract/Digestive system finding, Head and neck finding'),
('16600-VENOM', '16600', 'Carcinoma - skin (cutaneous)', 'Mass lesion/swelling finding, Haircoat/skin (+ appendages) finding'),
('16601-VENOM', '16601', 'Carcinoma - skin (cutaneous), perianal', 'Mass lesion/swelling finding, Haircoat/skin (+ appendages) finding, Perineal finding'),
('16511-VENOM', '16511', 'Carcinoma - small intestinal', 'Mass lesion/swelling finding, Alimentary tract/Digestive system finding, Abdominal finding'),
('16605-VENOM', '16605', 'Carcinoma - stomach (gastric)', 'Mass lesion/swelling finding, Alimentary tract/Digestive system finding, Abdominal finding'),
('16609-VENOM', '16609', 'Carcinoma - stomach (gastric), pylorus', 'Mass lesion/swelling finding, Alimentary tract/Digestive system finding, Abdominal finding'),
('16613-VENOM', '16613', 'Carcinoma - thyroid', 'Mass lesion/swelling finding, Endocrine/metabolic finding, Head and neck finding'),
('16617-VENOM', '16617', 'Carcinoma - tonsil', 'Mass lesion/swelling finding, Haematopoietic system finding, Immune system finding, Respiratory system finding, Alimentary tract/Digestive system finding, Head and neck finding'),
('16621-VENOM', '16621', 'Carcinoma - urethral', 'Mass lesion/swelling finding, Urogenital system finding'),
('16625-VENOM', '16625', 'Carcinoma - urinary bladder', 'Mass lesion/swelling finding, Urogenital system finding, Abdominal finding'),
('16661-VENOM', '16661', 'Carcinoma - uterine', 'Reproductive system finding, Mass lesion/swelling finding, Abdominal finding'),
('16494-VENOM', '16494', 'Carcinoma (site unspecified)', 'Mass lesion/swelling finding'),
('16501-VENOM', '16501', 'Carcinoma in situ - biliary tract', 'Mass lesion/swelling finding, Hepatobiliary system finding, Abdominal finding'),
('16505-VENOM', '16505', 'Carcinoma in situ - bronchial/bronchiolar', 'Mass lesion/swelling finding, Respiratory system finding, Thoracic finding'),
('16525-VENOM', '16525', 'Carcinoma in situ - colon', 'Mass lesion/swelling finding, Alimentary tract/Digestive system finding, Abdominal finding'),
('16509-VENOM', '16509', 'Carcinoma in situ - hepatocellular', 'Mass lesion/swelling finding, Hepatobiliary system finding, Abdominal finding'),
('16555-VENOM', '16555', 'Carcinoma in situ - laryngeal', 'Mass lesion/swelling finding, Respiratory system finding, Head and neck finding'),
('16559-VENOM', '16559', 'Carcinoma in situ - lung (pulmonary) parenchyma', 'Mass lesion/swelling finding, Respiratory system finding, Thoracic finding'),
('16647-VENOM', '16647', 'Carcinoma in situ - mammary (type unspecified)', 'Reproductive system finding, Mass lesion/swelling finding'),
('16649-VENOM', '16649', 'Carcinoma in situ - mammary, complex', 'Reproductive system finding, Mass lesion/swelling finding'),
('16651-VENOM', '16651', 'Carcinoma in situ - mammary, inflammatory', 'Reproductive system finding, Mass lesion/swelling finding'),
('16653-VENOM', '16653', 'Carcinoma in situ - mammary, simple', 'Reproductive system finding, Mass lesion/swelling finding'),
('16571-VENOM', '16571', 'Carcinoma in situ - nasal cavity', 'Mass lesion/swelling finding, Respiratory system finding, Head and neck finding'),
('16573-VENOM', '16573', 'Carcinoma in situ - nasal planum', 'Mass lesion/swelling finding, Respiratory system finding, Head and neck finding'),
('16576-VENOM', '16576', 'Carcinoma in situ - nasopharynx', 'Mass lesion/swelling finding, Respiratory system finding, Alimentary tract/Digestive system finding, Head and neck finding'),
('16580-VENOM', '16580', 'Carcinoma in situ - oesophageal', 'Mass lesion/swelling finding, Alimentary tract/Digestive system finding'),
('16586-VENOM', '16586', 'Carcinoma in situ - pancreatic', 'Mass lesion/swelling finding, Alimentary tract/Digestive system finding, Abdominal finding'),
('16590-VENOM', '16590', 'Carcinoma in situ - prostatic', 'Mass lesion/swelling finding, Urogenital system finding, Reproductive system finding, Pelvic structure finding'),
('16530-VENOM', '16530', 'Carcinoma in situ - rectal', 'Mass lesion/swelling finding, Alimentary tract/Digestive system finding, Pelvic structure finding'),
('16594-VENOM', '16594', 'Carcinoma in situ - renal (kidney)', 'Mass lesion/swelling finding, Urogenital system finding, Abdominal finding'),
('16598-VENOM', '16598', 'Carcinoma in situ - salivary gland', 'Mass lesion/swelling finding, Alimentary tract/Digestive system finding, Head and neck finding'),
('16493-VENOM', '16493', 'Carcinoma in situ - skin (cutaneous)', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('16603-VENOM', '16603', 'Carcinoma in situ - skin (cutaneous), perianal', 'Mass lesion/swelling finding, Haircoat/skin (+ appendages) finding, Perineal finding'),
('16516-VENOM', '16516', 'Carcinoma in situ - small intestinal', 'Mass lesion/swelling finding, Alimentary tract/Digestive system finding, Abdominal finding'),
('16607-VENOM', '16607', 'Carcinoma in situ - stomach (gastric)', 'Mass lesion/swelling finding, Alimentary tract/Digestive system finding, Abdominal finding'),
('16611-VENOM', '16611', 'Carcinoma in situ - stomach (gastric), pylorus', 'Mass lesion/swelling finding, Alimentary tract/Digestive system finding, Abdominal finding'),
('16615-VENOM', '16615', 'Carcinoma in situ - thyroid', 'Mass lesion/swelling finding, Endocrine/metabolic finding, Head and neck finding'),
('16619-VENOM', '16619', 'Carcinoma in situ - tonsil', 'Mass lesion/swelling finding, Haematopoietic system finding, Immune system finding, Respiratory system finding, Alimentary tract/Digestive system finding, Head and neck finding'),
('16623-VENOM', '16623', 'Carcinoma in situ - urethral', 'Mass lesion/swelling finding, Urogenital system finding'),
('16627-VENOM', '16627', 'Carcinoma in situ - urinary bladder', 'Mass lesion/swelling finding, Urogenital system finding, Abdominal finding'),
('16497-VENOM', '16497', 'Carcinoma in situ (site unspecified)', 'Mass lesion/swelling finding'),
('4558-VENOM', '4558', 'Cardiac conduction/impulse generation disorder', 'Cardiovascular system finding, Thoracic finding'),
('504-VENOM', '504', 'Cardiac tamponade', 'Cardiovascular system finding, Thoracic finding'),
('505-VENOM', '505', 'Cardiomegaly', 'Cardiovascular system finding, Thoracic finding'),
('4490-VENOM', '4490', 'Cardiomyopathy - Arrhythmogenic Right Ventricular Cardiomyopathy (ARVC)', 'Cardiovascular system finding, Musculoskeletal system finding, Thoracic finding'),
('506-VENOM', '506', 'Cardiomyopathy - Canine dilated cardiomyopathy', 'Cardiovascular system finding, Musculoskeletal system finding, Thoracic finding'),
('507-VENOM', '507', 'Cardiomyopathy - Canine hypertrophic cardiomyopathy', 'Cardiovascular system finding, Musculoskeletal system finding, Thoracic finding'),
('4129-VENOM', '4129', 'Cardiomyopathy - End stage HCM', 'Cardiovascular system finding, Musculoskeletal system finding, Thoracic finding'),
('508-VENOM', '508', 'Cardiomyopathy - Feline dilated cardiomyopathy', 'Cardiovascular system finding, Musculoskeletal system finding, Thoracic finding'),
('7384-VENOM', '7384', 'Cardiomyopathy - Feline endomyocardial restrictive cardiomyopathy', 'Cardiovascular system finding, Musculoskeletal system finding, Thoracic finding'),
('509-VENOM', '509', 'Cardiomyopathy - Feline hypertrophic cardiomyopathy', 'Cardiovascular system finding, Musculoskeletal system finding, Thoracic finding'),
('510-VENOM', '510', 'Cardiomyopathy - Feline restrictive cardiomyopathy ', 'Cardiovascular system finding, Musculoskeletal system finding, Thoracic finding'),
('511-VENOM', '511', 'Cardiomyopathy - Feline unclassified cardiomyopathy', 'Cardiovascular system finding, Musculoskeletal system finding, Thoracic finding'),
('4133-VENOM', '4133', 'Cardiomyopathy - Focal HCM', 'Cardiovascular system finding, Musculoskeletal system finding, Thoracic finding'),
('4131-VENOM', '4131', 'Cardiomyopathy - Hypertrophic Obstructive Cardiomyopathy (HOCM)', 'Cardiovascular system finding, Musculoskeletal system finding, Thoracic finding'),
('512-VENOM', '512', 'Cardiomyopathy - unclassified or class undetermined', 'Cardiovascular system finding, Musculoskeletal system finding, Thoracic finding'),
('513-VENOM', '513', 'Cardiomyopathy, primary', 'Cardiovascular system finding, Musculoskeletal system finding, Thoracic finding'),
('514-VENOM', '514', 'Cardiomyopathy, secondary', 'Cardiovascular system finding, Musculoskeletal system finding, Thoracic finding'),
('515-VENOM', '515', 'Cardiomyopathy, secondary - hypertensive', 'Cardiovascular system finding, Musculoskeletal system finding, Thoracic finding'),
('516-VENOM', '516', 'Cardiomyopathy, secondary - taurine-deficiency', 'Cardiovascular system finding, Musculoskeletal system finding, Nutritional finding, Thoracic finding'),
('517-VENOM', '517', 'Cardiomyopathy, secondary - thyrotoxic', 'Cardiovascular system finding, Musculoskeletal system finding, Endocrine/metabolic finding, Thoracic finding, Head and neck finding'),
('518-VENOM', '518', 'Cardiopulmonary arrest', 'Cardiovascular system finding, Respiratory system finding, Complication during visit finding'),
('519-VENOM', '519', 'Cat flu', 'Respiratory system finding, Infectious disorder finding, Head and neck finding'),
('520-VENOM', '520', 'Cataplexy', 'Nervous system finding, Head and neck finding'),
('521-VENOM', '521', 'Cataract', 'Ophthalmology/vision finding, Head and neck finding'),
('522-VENOM', '522', 'Cataract - complete', 'Ophthalmology/vision finding, Head and neck finding'),
('523-VENOM', '523', 'Cataract - congenital', 'Ophthalmology/vision finding, Congenital finding, Head and neck finding'),
('524-VENOM', '524', 'Cataract - diabetic', 'Ophthalmology/vision finding, Endocrine/metabolic finding, Alimentary tract/Digestive system finding, Head and neck finding, Abdominal finding'),
('525-VENOM', '525', 'Cataract - incipient (immature)', 'Ophthalmology/vision finding, Head and neck finding'),
('526-VENOM', '526', 'Cataract - incomplete', 'Ophthalmology/vision finding, Head and neck finding'),
('527-VENOM', '527', 'Cataract - Morgagnian', 'Ophthalmology/vision finding, Head and neck finding'),
('528-VENOM', '528', 'Cauda equina syndrome (Cauda equina neuritis, Lumbosacral syndrome)', 'Nervous system finding'),
('529-VENOM', '529', 'Cellulitis', 'Haircoat/skin (+ appendages) finding, Soft tissue finding, Inflammatory disorder finding'),
('16952-VENOM', '16952', 'Cementoma - mandible', 'Alimentary tract/Digestive system finding, Musculoskeletal system finding, Mass lesion/swelling finding, Head and neck finding'),
('16953-VENOM', '16953', 'Cementoma - maxilla', 'Alimentary tract/Digestive system finding, Musculoskeletal system finding, Mass lesion/swelling finding, Head and neck finding'),
('2121-VENOM', '2121', 'Central nervous system (CNS) disorder', 'Nervous system finding'),
('2897-VENOM', '2897', 'Central nervous system (CNS) inflammatory disorder', 'Nervous system finding, Inflammatory disorder finding'),
('327-VENOM', '327', 'Cerebellar abiotrophy', 'Nervous system finding, Degenerative disorder finding, Hereditary disorder finding, Head and neck finding'),
('530-VENOM', '530', 'Cerebellar ataxia', 'Nervous system finding, Head and neck finding'),
('531-VENOM', '531', 'Cerebellar disorder', 'Nervous system finding, Head and neck finding'),
('532-VENOM', '532', 'Cerebellar herniation', 'Nervous system finding, Head and neck finding'),
('533-VENOM', '533', 'Cerebellar infarction', 'Nervous system finding, Cardiovascular system finding, Head and neck finding'),
('534-VENOM', '534', 'Cerebral infarction', 'Nervous system finding, Cardiovascular system finding, Head and neck finding'),
('535-VENOM', '535', 'Cerebrovascular accident (CVA)', 'Nervous system finding, Cardiovascular system finding, Head and neck finding'),
('536-VENOM', '536', 'Cerebrovascular disorder', 'Nervous system finding, Cardiovascular system finding, Head and neck finding'),
('537-VENOM', '537', 'Chalazion (meibomian cyst)', 'Ophthalmology/vision finding, Head and neck finding'),
('538-VENOM', '538', 'Cheilitis (inflammation of lips)', 'Alimentary tract/Digestive system finding, Soft tissue finding, Inflammatory disorder finding, Head and neck finding'),
('539-VENOM', '539', 'Chemical injury to skin (cutaneous)', 'Haircoat/skin (+ appendages) finding, Traumatic episode finding'),
('540-VENOM', '540', 'Chemodectoma', 'Mass lesion/swelling finding, Nervous system finding, Endocrine/metabolic finding'),
('541-VENOM', '541', 'Chlamydiosis', 'Infectious disorder finding'),
('543-VENOM', '543', 'Cholangiohepatitis', 'Hepatobiliary system finding, Inflammatory disorder finding, Abdominal finding'),
('542-VENOM', '542', 'Cholangitis', 'Hepatobiliary system finding, Inflammatory disorder finding, Abdominal finding'),
('544-VENOM', '544', 'Cholangitis/cholangiohepatitis syndrome, feline', 'Hepatobiliary system finding, Inflammatory disorder finding, Abdominal finding'),
('545-VENOM', '545', 'Cholecystitis', 'Hepatobiliary system finding, Inflammatory disorder finding, Abdominal finding'),
('546-VENOM', '546', 'Cholelithiasis', 'Hepatobiliary system finding, Abdominal finding'),
('547-VENOM', '547', 'Cholestasis', 'Hepatobiliary system finding, Abdominal finding'),
('549-VENOM', '549', 'Cholesteatoma - middle ear', 'Mass lesion/swelling finding, Auditory system finding, Head and neck finding'),
('16416-VENOM', '16416', 'Cholesteatoma (site unspecified)', 'Mass lesion/swelling finding'),
('16958-VENOM', '16958', 'Chondroma (site unspecified)', 'Mass lesion/swelling finding, Musculoskeletal system finding, Connective tissue finding'),
('16959-VENOM', '16959', 'Chondrosarcoma - long bone', 'Musculoskeletal system finding, Connective tissue finding, Mass lesion/swelling finding, Limb finding'),
('16961-VENOM', '16961', 'Chondrosarcoma - pelvis', 'Musculoskeletal system finding, Connective tissue finding, Mass lesion/swelling finding, Pelvic structure finding'),
('16963-VENOM', '16963', 'Chondrosarcoma - rib', 'Musculoskeletal system finding, Connective tissue finding, Mass lesion/swelling finding, Thoracic finding'),
('16965-VENOM', '16965', 'Chondrosarcoma - scapula', 'Musculoskeletal system finding, Connective tissue finding, Mass lesion/swelling finding, Limb finding'),
('16975-VENOM', '16975', 'Chondrosarcoma - sinonasal', 'Musculoskeletal system finding, Respiratory system finding, Connective tissue finding, Mass lesion/swelling finding, Head and neck finding'),
('16977-VENOM', '16977', 'Chondrosarcoma - tracheal', 'Musculoskeletal system finding, Respiratory system finding, Connective tissue finding, Mass lesion/swelling finding, Head and neck finding'),
('16418-VENOM', '16418', 'Chondrosarcoma (site unspecified)', 'Mass lesion/swelling finding, Musculoskeletal system finding, Connective tissue finding'),
('551-VENOM', '551', 'Chordae tendinae abnormality - mitral', 'Cardiovascular system finding, Thoracic finding'),
('552-VENOM', '552', 'Chordae tendinae abnormality - tricuspid', 'Cardiovascular system finding, Thoracic finding'),
('553-VENOM', '553', 'Chordae tendinae rupture', 'Cardiovascular system finding, Thoracic finding'),
('554-VENOM', '554', 'Chorioretinitis', 'Ophthalmology/vision finding, Inflammatory disorder finding, Head and neck finding'),
('555-VENOM', '555', 'Chronic airway disease', 'Respiratory system finding, Thoracic finding'),
('4147-VENOM', '4147', 'Chronic degenerative AV valve disease', 'Cardiovascular system finding, Degenerative disorder finding, Thoracic finding'),
('2126-VENOM', '2126', 'Chronic degenerative radiculomyelopathy (CDRM)', 'Nervous system finding, Degenerative disorder finding'),
('11741-VENOM', '11741', 'Chronic kidney (renal) disease (CRF, CKD) – azotaemic', 'Urogenital system finding, Endocrine/metabolic finding, Abdominal finding'),
('11743-VENOM', '11743', 'Chronic kidney (renal) disease (CRF, CKD) – non-azotaemic', 'Urogenital system finding, Abdominal finding'),
('11739-VENOM', '11739', 'Chronic kidney (renal) disease (CRF, CKD) (azotaemia unspecified)', 'Urogenital system finding, Abdominal finding'),
('556-VENOM', '556', 'Chylothorax', 'Effusion finding, Thoracic finding'),
('557-VENOM', '557', 'Chylothorax, malignant', 'Effusion finding, Thoracic finding'),
('559-VENOM', '559', 'Claw injury (traumatic)', 'Haircoat/skin (+ appendages) finding, Traumatic episode finding, Limb finding'),
('3909-VENOM', '3909', 'Claw/nail disorder', 'Haircoat/skin (+ appendages) finding, Limb finding'),
('560-VENOM', '560', 'Cleft palate', 'Alimentary tract/Digestive system finding, Congenital finding, Head and neck finding'),
('561-VENOM', '561', 'Clostridiosis', 'Infectious disorder finding'),
('562-VENOM', '562', 'Coagulopathy (clotting disorder)', 'Haemostatic system finding'),
('563-VENOM', '563', 'Coagulopathy (clotting disorder) - acquired', 'Haemostatic system finding'),
('564-VENOM', '564', 'Coagulopathy (clotting disorder) - inherited', 'Haemostatic system finding, Hereditary disorder finding'),
('565-VENOM', '565', 'Cobalamin (vitamin B12) deficiency', 'Nutritional finding'),
('566-VENOM', '566', 'Coccidiosis', 'Infectious disorder finding'),
('567-VENOM', '567', 'Cognitive dysfunction', 'Nervous system finding, Head and neck finding'),
('568-VENOM', '568', 'Cold agglutinin disease', 'Haematopoietic system finding, Immune system finding'),
('569-VENOM', '569', 'Colitis', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding, Abdominal finding'),
('570-VENOM', '570', 'Colitis - chronic', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding, Abdominal finding'),
('571-VENOM', '571', 'Colitis - eosinophilic', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding, Abdominal finding'),
('572-VENOM', '572', 'Colitis - lymphocytic/plasmacytic', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding, Abdominal finding'),
('573-VENOM', '573', 'Colitis - ulcerative', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding, Abdominal finding'),
('574-VENOM', '574', 'Collie Eye Anomaly', 'Ophthalmology/vision finding, Hereditary disorder finding, Head and neck finding'),
('575-VENOM', '575', 'Colonic stricture', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('576-VENOM', '576', 'Colour dilution alopecia', 'Haircoat/skin (+ appendages) finding'),
('577-VENOM', '577', 'Colour mutant alopecia', 'Haircoat/skin (+ appendages) finding'),
('578-VENOM', '578', 'Compartment syndrome', 'Musculoskeletal system finding'),
('5809-VENOM', '5809', 'Complication of fracture repair', 'Complication of medical care finding'),
('4047-VENOM', '4047', 'Complication of transvenous pacemaker placement', 'Complication of medical care finding'),
('9683-VENOM', '9683', 'CONDITION UNDER INVESTIGATION', 'Clinical finding'),
('579-VENOM', '579', 'Congenital disorder', 'Congenital finding'),
('9574-VENOM', '9574', 'Congenital metabolic disorder', 'Congenital finding, Endocrine/metabolic finding'),
('17978-VENOM', '17978', 'Congenital thoracic (chest) wall anomaly (unspecified)', 'Congenital finding, Thoracic finding'),
('17977-VENOM', '17977', 'Congenital thoracic (chest) wall anomaly: pectus excavatum', 'Congenital finding, Thoracic finding'),
('4542-VENOM', '4542', 'Conjunctivitis', 'Ophthalmology/vision finding, Soft tissue finding, Inflammatory disorder finding, Head and neck finding'),
('580-VENOM', '580', 'Conjunctivitis, infectious', 'Ophthalmology/vision finding, Soft tissue finding, Inflammatory disorder finding, Infectious disorder finding, Head and neck finding'),
('581-VENOM', '581', 'Conjunctivitis, infectious - presumed Feline herpes virus', 'Ophthalmology/vision finding, Soft tissue finding, Inflammatory disorder finding, Infectious disorder finding, Head and neck finding'),
('582-VENOM', '582', 'Conjunctivitis, non-infectious', 'Ophthalmology/vision finding, Soft tissue finding, Inflammatory disorder finding, Head and neck finding'),
('583-VENOM', '583', 'Constipation', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('4540-VENOM', '4540', 'Copper-associated chronic hepatitis', 'Hepatobiliary system finding, Inflammatory disorder finding, Abdominal finding'),
('584-VENOM', '584', 'Copper-associated hepatitis in Bedlington Terriers', 'Hepatobiliary system finding, Inflammatory disorder finding, Hereditary disorder finding, Abdominal finding'),
('585-VENOM', '585', 'Cor pulmonale', 'Respiratory system finding, Cardiovascular system finding, Thoracic finding'),
('586-VENOM', '586', 'Corneal degeneration', 'Ophthalmology/vision finding, Degenerative disorder finding, Head and neck finding'),
('5777-VENOM', '5777', 'Corneal disorder', 'Ophthalmology/vision finding, Head and neck finding'),
('587-VENOM', '587', 'Corneal dystrophy', 'Ophthalmology/vision finding, Head and neck finding'),
('588-VENOM', '588', 'Corneal dystrophy - endothelial', 'Ophthalmology/vision finding, Head and neck finding'),
('589-VENOM', '589', 'Corneal dystrophy - epithelial', 'Ophthalmology/vision finding, Head and neck finding'),
('590-VENOM', '590', 'Corneal dystrophy - stromal', 'Ophthalmology/vision finding, Head and neck finding'),
('591-VENOM', '591', 'Corneal laceration', 'Ophthalmology/vision finding, Wound finding, Traumatic episode finding, Head and neck finding'),
('592-VENOM', '592', 'Corneal melanosis', 'Ophthalmology/vision finding, Head and neck finding'),
('593-VENOM', '593', 'Corneal perforation', 'Ophthalmology/vision finding, Head and neck finding'),
('594-VENOM', '594', 'Corneal scar', 'Ophthalmology/vision finding, Head and neck finding'),
('595-VENOM', '595', 'Corneal sequestrum', 'Ophthalmology/vision finding, Head and neck finding'),
('596-VENOM', '596', 'Corneal stromal abscess', 'Ophthalmology/vision finding, Inflammatory disorder finding, Head and neck finding'),
('597-VENOM', '597', 'Corneal ulcer', 'Ophthalmology/vision finding, Inflammatory disorder finding, Head and neck finding'),
('598-VENOM', '598', 'Corneal ulcer - infective', 'Ophthalmology/vision finding, Inflammatory disorder finding, Infectious disorder finding, Head and neck finding'),
('599-VENOM', '599', 'Corneal ulcer - infective, bacterial', 'Ophthalmology/vision finding, Inflammatory disorder finding, Infectious disorder finding, Head and neck finding'),
('600-VENOM', '600', 'Corneal ulcer - infective, viral', 'Ophthalmology/vision finding, Inflammatory disorder finding, Infectious disorder finding, Head and neck finding'),
('602-VENOM', '602', 'Cornification disorder', 'Haircoat/skin (+ appendages) finding'),
('603-VENOM', '603', 'Coronary artery abnormality', 'Cardiovascular system finding, Thoracic finding'),
('604-VENOM', '604', 'Coronavirus infection', 'Infectious disorder finding'),
('605-VENOM', '605', 'Cowpoxvirus infection', 'Infectious disorder finding'),
('606-VENOM', '606', 'Cranial neuropathy', 'Nervous system finding, Head and neck finding'),
('607-VENOM', '607', 'Craniomandibular osteopathy', 'Musculoskeletal system finding, Head and neck finding'),
('608-VENOM', '608', 'Cricopharyngeal achalasia (cricopharyngeal dysphagia)', 'Alimentary tract/Digestive system finding, Respiratory system finding, Musculoskeletal system finding, Head and neck finding'),
('609-VENOM', '609', 'Cruciate disease', 'Musculoskeletal system finding, Connective tissue finding, Limb finding'),
('610-VENOM', '610', 'Cruciate ligament instability - caudal', 'Musculoskeletal system finding, Connective tissue finding, Limb finding'),
('611-VENOM', '611', 'Cruciate ligament instability - cranial', 'Musculoskeletal system finding, Connective tissue finding, Limb finding'),
('612-VENOM', '612', 'Cruciate ligament instability, partial - caudal', 'Musculoskeletal system finding, Connective tissue finding, Limb finding'),
('613-VENOM', '613', 'Cruciate ligament instability, partial - cranial', 'Musculoskeletal system finding, Connective tissue finding, Limb finding'),
('614-VENOM', '614', 'Cruciate ligament rupture - caudal', 'Musculoskeletal system finding, Connective tissue finding, Limb finding'),
('616-VENOM', '616', 'Cruciate ligament rupture - caudal and cranial', 'Musculoskeletal system finding, Connective tissue finding, Limb finding'),
('615-VENOM', '615', 'Cruciate ligament rupture - cranial', 'Musculoskeletal system finding, Connective tissue finding, Limb finding'),
('617-VENOM', '617', 'Cryptococcosis', 'Infectious disorder finding'),
('618-VENOM', '618', 'Cryptorchidism - bilateral', 'Urogenital system finding, Reproductive system finding, Congenital disorder finding'),
('619-VENOM', '619', 'Cryptorchidism - unilateral', 'Urogenital system finding, Reproductive system finding, Congenital disorder finding'),
('620-VENOM', '620', 'Cryptosporidiosis', 'Infectious disorder finding'),
('4518-VENOM', '4518', 'Cutaneous (skin) bacterial overgrowth', 'Haircoat/skin (+ appendages) finding, Infectious disorder finding'),
('621-VENOM', '621', 'Cutaneous horn', 'Haircoat/skin (+ appendages) finding'),
('622-VENOM', '622', 'Cutaneous lupus erythematosus', 'Haircoat/skin (+ appendages) finding, Immune system finding'),
('623-VENOM', '623', 'Cyclic flank alopecia', 'Haircoat/skin (+ appendages) finding'),
('16982-VENOM', '16982', 'Cystadenocarcinoma - biliary', 'Hepatobiliary system finding, Mass lesion/swelling finding, Abdominal finding'),
('16984-VENOM', '16984', 'Cystadenocarcinoma - ovarian', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding, Abdominal finding'),
('16986-VENOM', '16986', 'Cystadenocarcinoma - renal (kidney)', 'Urogenital system finding, Mass lesion/swelling finding, Abdominal finding'),
('16988-VENOM', '16988', 'Cystadenocarcinoma - salivary gland', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Head and neck finding'),
('16979-VENOM', '16979', 'Cystadenocarcinoma (site unspecified)', 'Mass lesion/swelling finding'),
('16993-VENOM', '16993', 'Cystadenoma - biliary', 'Hepatobiliary system finding, Mass lesion/swelling finding, Abdominal finding'),
('16995-VENOM', '16995', 'Cystadenoma - ovarian', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding, Abdominal finding'),
('16997-VENOM', '16997', 'Cystadenoma - salivary gland', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Head and neck finding'),
('16990-VENOM', '16990', 'Cystadenoma (site unspecified)', 'Mass lesion/swelling finding'),
('624-VENOM', '624', 'Cystitis', 'Urogenital system finding, Inflammatory disorder finding, Abdominal finding'),
('625-VENOM', '625', 'Cystitis - bacterial', 'Urogenital system finding, Inflammatory disorder finding, Infectious disorder finding, Abdominal finding'),
('626-VENOM', '626', 'Cystitis - chronic', 'Urogenital system finding, Inflammatory disorder finding, Abdominal finding'),
('627-VENOM', '627', 'Cystitis - polypoid', 'Urogenital system finding, Inflammatory disorder finding, Abdominal finding'),
('628-VENOM', '628', 'Deafness - complete', 'Auditory system finding, Head and neck finding'),
('629-VENOM', '629', 'Deafness - partial', 'Auditory system finding, Head and neck finding'),
('23-VENOM', '23', 'Dental (tooth) disorder', 'Alimentary tract/Digestive system finding, Head and neck finding'),
('631-VENOM', '631', 'Dental (tooth) structure injury', 'Alimentary tract/Digestive system finding, Head and neck finding'),
('632-VENOM', '632', 'Dermatitis', 'Haircoat/skin (+ appendages) finding, Inflammatory disorder finding'),
('633-VENOM', '633', 'Dermatitis - chronic', 'Haircoat/skin (+ appendages) finding, Inflammatory disorder finding'),
('634-VENOM', '634', 'Dermatitis - contact dermatitis', 'Haircoat/skin (+ appendages) finding, Inflammatory disorder finding'),
('635-VENOM', '635', 'Dermatitis - contagious viral pustular dermatitis', 'Haircoat/skin (+ appendages) finding, Inflammatory disorder finding, Infectious disorder finding'),
('636-VENOM', '636', 'Dermatitis - lip-fold', 'Haircoat/skin (+ appendages) finding, Inflammatory disorder finding'),
('637-VENOM', '637', 'Dermatitis - perforating dermatitis', 'Haircoat/skin (+ appendages) finding, Inflammatory disorder finding'),
('638-VENOM', '638', 'Dermatitis - photodermatitis', 'Haircoat/skin (+ appendages) finding, Inflammatory disorder finding'),
('639-VENOM', '639', 'Dermatitis - pyotraumatic (acute moist)', 'Haircoat/skin (+ appendages) finding, Inflammatory disorder finding, Infectious disorder finding'),
('640-VENOM', '640', 'Dermatitis - seborrhoeic', 'Haircoat/skin (+ appendages) finding, Inflammatory disorder finding'),
('641-VENOM', '641', 'Dermatitis - solar dermatitis', 'Haircoat/skin (+ appendages) finding, Inflammatory disorder finding'),
('642-VENOM', '642', 'Dermatitis - thymoma-associated exfoliative dermatitis', 'Haircoat/skin (+ appendages) finding, Inflammatory disorder finding, Immune system finding, Endocrine/metabolic finding, Thoracic finding'),
('643-VENOM', '643', 'Dermatitis and/or panniculitis, (pyo)granulomatous - bacterial', 'Haircoat/skin (+ appendages) finding, Inflammatory disorder finding, Infectious disorder finding'),
('644-VENOM', '644', 'Dermatitis and/or panniculitis, (pyo)granulomatous - fungal', 'Haircoat/skin (+ appendages) finding, Inflammatory disorder finding, Infectious disorder finding'),
('645-VENOM', '645', 'Dermatitis and/or panniculitis, (pyo)granulomatous - sterile', 'Haircoat/skin (+ appendages) finding, Inflammatory disorder finding, Immune system finding'),
('647-VENOM', '647', 'Dermatophilosis', 'Haircoat/skin (+ appendages) finding, Infectious disorder finding'),
('648-VENOM', '648', 'Dermatophytosis (Ringworm)', 'Haircoat/skin (+ appendages) finding, Infectious disorder finding'),
('649-VENOM', '649', 'Dermoid cyst', 'Mass lesion/swelling finding'),
('651-VENOM', '651', 'Dermoid cyst - cutaneous (skin)', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('650-VENOM', '650', 'Dermoid cyst - nasal (nose)', 'Respiratory system finding, Mass lesion/swelling finding, Head and neck finding'),
('652-VENOM', '652', 'Dermoid cyst - spinal', 'Nervous system finding, Mass lesion/swelling finding'),
('653-VENOM', '653', 'Dermoid sinus', 'Haircoat/skin (+ appendages) finding, Hereditary disorder finding, Developmental disorder finding, Congenital finding'),
('655-VENOM', '655', 'Diabetes insipidus', 'Endocrine/Metabolic finding'),
('4590-VENOM', '4590', 'Diabetes insipidus - central', 'Endocrine/Metabolic finding, Nervous system finding, Head and neck finding'),
('657-VENOM', '657', 'Diabetes insipidus - nephrogenic', 'Endocrine/Metabolic finding, Urogenital system finding, Abdominal finding'),
('658-VENOM', '658', 'Diabetes mellitus', 'Endocrine/Metabolic finding, Alimentary tract/Digestive system finding, Abdominal finding'),
('660-VENOM', '660', 'Diabetes mellitus - hyperosmolar non-ketotic syndrome', 'Endocrine/Metabolic finding, Alimentary tract/Digestive system finding, Abdominal finding'),
('661-VENOM', '661', 'Diabetes mellitus - stable', 'Endocrine/Metabolic finding, Alimentary tract/Digestive system finding, Abdominal finding'),
('662-VENOM', '662', 'Diabetes mellitus - unstable', 'Endocrine/Metabolic finding, Alimentary tract/Digestive system finding, Abdominal finding'),
('659-VENOM', '659', 'Diabetes mellitus resolved', 'Endocrine/Metabolic finding, Alimentary tract/Digestive system finding, Abdominal finding'),
('654-VENOM', '654', 'Diabetic ketoacidosis (DKA)', 'Endocrine/Metabolic finding, Alimentary tract/Digestive system finding, Abdominal finding'),
('324-VENOM', '324', 'DIAGNOSIS NOT MADE (No diagnosis)', 'Clinical finding'),
('15534-VENOM', '15534', 'DIAGNOSIS NOT MADE (Not presented for problem investigation)', 'Clinical finding'),
('325-VENOM', '325', 'DIAGNOSIS UNCHANGED from last visit', 'Clinical finding'),
('663-VENOM', '663', 'Diaphragmatic rupture', 'Musculoskeletal system finding, Thoracic finding, Abdominal finding'),
('664-VENOM', '664', 'Diaphragmatic rupture - traumatic', 'Musculoskeletal system finding, Traumatic episode finding, Thoracic finding, Abdominal finding'),
('665-VENOM', '665', 'Discoid lupus erythematosus (DLE)', 'Haircoat/skin (+ appendages) finding, Immune system finding'),
('666-VENOM', '666', 'Discospondylitis', 'Musculoskeletal system finding, Infectious disorder finding, Inflammatory disorder finding, Spine (vertebral column) finding'),
('667-VENOM', '667', 'Disseminated intravascular coagulation (DIC)', 'Haemostatic system finding'),
('668-VENOM', '668', 'Distichiasis', 'Ophthalmology/vision finding, Head and neck finding'),
('11725-VENOM', '11725', 'Dressing-related complication (unspecified)', 'Episode of care finding'),
('669-VENOM', '669', 'Dysautonomia', 'Nervous system finding'),
('670-VENOM', '670', 'Dysphagia', 'Alimentary tract/Digestive system finding'),
('671-VENOM', '671', 'Dysphagia - oesophageal', 'Alimentary tract/Digestive system finding'),
('672-VENOM', '672', 'Dysphagia - pharyngeal', 'Alimentary tract/Digestive system finding, Respiratory system finding, Head and neck finding'),
('673-VENOM', '673', 'Dystocia', 'Reproductive system finding'),
('3913-VENOM', '3913', 'Dystocia - fetal maldisposition', 'Reproductive system finding'),
('3914-VENOM', '3914', 'Dystocia - fetopelvic disproportion', 'Reproductive system finding'),
('3915-VENOM', '3915', 'Dystocia - uterine inertia', 'Reproductive system finding, Abdominal finding'),
('674-VENOM', '674', 'Ear (aural) disease', 'Auditory system finding, Head and neck finding'),
('675-VENOM', '675', 'Ear (aural) disease, chronic', 'Auditory system finding, Head and neck finding'),
('676-VENOM', '676', 'Ear (aural) infection', 'Auditory system finding, Infectious disorder finding, Head and neck finding'),
('677-VENOM', '677', 'Ear (aural) infection - bacterial', 'Auditory system finding, Infectious disorder finding, Head and neck finding'),
('678-VENOM', '678', 'Ear (aural) infection - fungal', 'Auditory system finding, Infectious disorder finding, Head and neck finding'),
('679-VENOM', '679', 'Ear margin dermatosis', 'Haircoat/skin (+ appendages) finding, Auditory system finding, Head and neck finding'),
('680-VENOM', '680', 'Ectropion', 'Ophthalmology/vision finding, Head and neck finding'),
('681-VENOM', '681', 'Eczema', 'Haircoat/skin (+ appendages) finding, Inflammatory disorder finding'),
('682-VENOM', '682', 'Effluvium, anagen or telogen', 'Haircoat/skin (+ appendages) finding'),
('683-VENOM', '683', 'Ehlers-Danlos syndrome', 'Haircoat/skin (+ appendages) finding, Musculoskeletal system finding, Hereditary disorder finding, Developmental disorder finding, Congenital disorder finding'),
('684-VENOM', '684', 'Elbow dysplasia (canine)', 'Musculoskeletal system finding, Hereditary disorder finding, Limb finding'),
('685-VENOM', '685', 'Elbow dysplasia (canine) - elbow incongruity', 'Musculoskeletal system finding, Hereditary disorder finding, Limb finding'),
('686-VENOM', '686', 'Elbow dysplasia (canine) - medial coronoid process disease, fragment identified', 'Musculoskeletal system finding, Hereditary disorder finding, Limb finding'),
('687-VENOM', '687', 'Elbow dysplasia (canine) - medial coronoid process disease, no fragment identified', 'Musculoskeletal system finding, Hereditary disorder finding, Limb finding'),
('688-VENOM', '688', 'Elbow dysplasia (canine) - osteochondritis dissecans (OCD)', 'Musculoskeletal system finding, Hereditary disorder finding, Limb finding'),
('689-VENOM', '689', 'Elbow dysplasia (canine) - ununited anconeal process (UAP)', 'Musculoskeletal system finding, Hereditary disorder finding, Limb finding'),
('690-VENOM', '690', 'Elongated soft palate', 'Alimentary/Digestive system finding, Respiratory system finding, Congenital finding, Head and neck finding'),
('691-VENOM', '691', 'Emaciation', 'Nutritional finding'),
('7821-VENOM', '7821', 'Embolus - air', 'Cardiovascular system finding'),
('3962-VENOM', '3962', 'Embolus - cement', 'Cardiovascular system finding'),
('7816-VENOM', '7816', 'Embolus - fat', 'Cardiovascular system finding'),
('8312-VENOM', '8312', 'Embolus - material unknown', 'Cardiovascular system finding'),
('8304-VENOM', '8304', 'Embolus - other', 'Cardiovascular system finding'),
('8296-VENOM', '8296', 'Embolus - Total Hip Replacement-associated', 'Cardiovascular system finding, Complication of medical care finding'),
('692-VENOM', '692', 'Empyema - other', 'Inflammatory disorder finding'),
('693-VENOM', '693', 'Empyema - pyothorax', 'Inflammatory disorder finding, Effusion finding, Thoracic finding'),
('694-VENOM', '694', 'Encephalitis', 'Nervous system finding, Inflammatory disorder finding, Head and neck finding'),
('3803-VENOM', '3803', 'Encephalitozoon cuniculi infection', ''),
('695-VENOM', '695', 'Encephalopathy', 'Nervous system finding, Head and neck finding'),
('696-VENOM', '696', 'Encephalopathy - hypertensive', 'Nervous system finding, Cardiovascular system finding, Head and neck finding'),
('697-VENOM', '697', 'Encephalopathy - uraemic', 'Nervous system finding, Endocrine/Metabolic finding, Head and neck finding'),
('698-VENOM', '698', 'Endocardiosis', 'Cardiovascular system finding, Thoracic finding'),
('699-VENOM', '699', 'Endocarditis', 'Cardiovascular system finding, Inflammatory disorder finding, Thoracic finding'),
('700-VENOM', '700', 'Endocarditis - bacterial', 'Cardiovascular system finding, Inflammatory disorder finding, Infectious disorder finding, Thoracic finding'),
('701-VENOM', '701', 'Endometrial cystic hyperplasia', 'Reproductive system finding, Abdominal finding'),
('702-VENOM', '702', 'Endometritis', 'Reproductive system finding, Inflammatory disorder finding, Abdominal finding'),
('703-VENOM', '703', 'Endophthalmitis', 'Ophthalmology/vision finding, Inflammatory disorder finding, Head and neck finding'),
('704-VENOM', '704', 'Enteritis', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding, Abdominal finding'),
('705-VENOM', '705', 'Enteritis - lymphocytic/plasmacytic', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding, Abdominal finding'),
('706-VENOM', '706', 'Enteropathy', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('707-VENOM', '707', 'Enthesopathy', 'Musculoskeletal system finding'),
('708-VENOM', '708', 'Entropion', 'Ophthalmology/vision finding, Head and neck finding'),
('5875-VENOM', '5875', 'Envenomation', '#NAME?'),
('709-VENOM', '709', 'Enzymopathy', 'Endocrine/Metabolic finding'),
('9645-VENOM', '9645', 'Eosinophilic bronchopneumopathy (Pulmonary infiltrates with eosinophilia, PIE)', 'Respiratory system finding, Inflammatory disorder finding, Thoracic finding'),
('710-VENOM', '710', 'Eosinophilic granuloma', 'Haircoat/skin (+ appendages) finding, Immune system finding, Alimentary tract/Digestive system finding, Head and neck finding'),
('711-VENOM', '711', 'Eosinophilic plaque', 'Haircoat/skin (+ appendages) finding, Immune system finding'),
('712-VENOM', '712', 'Ependymoma of brain', 'Nervous system finding, Mass lesion/swelling finding, Head and neck finding'),
('4610-VENOM', '4610', 'Ependymoma of spinal cord', 'Nervous system finding, Mass lesion/swelling finding'),
('713-VENOM', '713', 'Epidermoid cyst', 'Mass lesion/swelling finding'),
('714-VENOM', '714', 'Epidermoid cyst - spinal', 'Nervous system finding, Mass lesion/swelling finding'),
('715-VENOM', '715', 'Epilepsy', 'Nervous system finding, Head and neck finding'),
('716-VENOM', '716', 'Epilepsy - Cursive (running) epilepsy', 'Nervous system finding, Head and neck finding'),
('717-VENOM', '717', 'Epilepsy - focal', 'Nervous system finding, Head and neck finding'),
('718-VENOM', '718', 'Epilepsy - idiopathic generalised', 'Nervous system finding, Head and neck finding'),
('719-VENOM', '719', 'Epiphyseal disorder', 'Musculoskeletal system finding'),
('720-VENOM', '720', 'Epiphysitis/physitis', 'Musculoskeletal system finding, Inflammatory disorder finding'),
('721-VENOM', '721', 'Episcleritis', 'Ophthalmology/vision finding, Inflammatory disorder finding, Head and neck finding'),
('722-VENOM', '722', 'Episcleritis - nodular', 'Ophthalmology/vision finding, Inflammatory disorder finding, Head and neck finding'),
('17003-VENOM', '17003', 'Epithelioma - intracutaneous cornifying', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('17005-VENOM', '17005', 'Epithelioma - meibomian', 'Ophthalmology/vision finding, Mass lesion/swelling finding, Head and neck finding'),
('17007-VENOM', '17007', 'Epithelioma - perianal gland', 'Haircoat/skin (+ appendages) finding, Perineal finding, Mass lesion/swelling finding'),
('17009-VENOM', '17009', 'Epithelioma (site unspecified)', 'Mass lesion/swelling finding'),
('17012-VENOM', '17012', 'Epulis - giant cell, mandible', 'Mass lesion/swelling finding, Alimentary tract/Digestive system finding, Soft tissue finding, Head and neck finding'),
('17014-VENOM', '17014', 'Epulis - giant cell, maxilla', 'Mass lesion/swelling finding, Alimentary tract/Digestive system finding, Soft tissue finding, Head and neck finding'),
('16801-VENOM', '16801', 'Epulis - ossifying epulis, mandible', 'Mass lesion/swelling finding, Alimentary tract/Digestive system finding, Soft tissue finding, Head and neck finding'),
('16802-VENOM', '16802', 'Epulis - ossifying epulis, maxilla', 'Mass lesion/swelling finding, Alimentary tract/Digestive system finding, Soft tissue finding, Head and neck finding'),
('16805-VENOM', '16805', 'Epulis - periodontal fibromatous epulis (periodontal ossifying fibroma), mandible', 'Mass lesion/swelling finding, Alimentary tract/Digestive system finding, Soft tissue finding, Head and neck finding'),
('16806-VENOM', '16806', 'Epulis - periodontal fibromatous epulis (periodontal ossifying fibroma), maxilla', 'Mass lesion/swelling finding, Alimentary tract/Digestive system finding, Soft tissue finding, Head and neck finding'),
('16797-VENOM', '16797', 'Epulis (type unspecified) - mandible', 'Mass lesion/swelling finding, Alimentary tract/Digestive system finding, Soft tissue finding, Head and neck finding'),
('16798-VENOM', '16798', 'Epulis (type unspecified) - maxilla', 'Mass lesion/swelling finding, Alimentary tract/Digestive system finding, Soft tissue finding, Head and neck finding'),
('727-VENOM', '727', 'Erythema multiforme', 'Haircoat/skin (+ appendages) disorder finding, Immune system finding'),
('728-VENOM', '728', 'Erythema multiforme - herpesvirus-associated', 'Haircoat/skin (+ appendages) disorder finding, Immune system finding, Infectious disorder finding'),
('729-VENOM', '729', 'Erythroid hypoplasia', 'Haematopoietic system finding'),
('730-VENOM', '730', 'Escherichia coli infection', 'Infectious disorder finding'),
('731-VENOM', '731', 'Evans syndrome', 'Haematopoietic system finding, Immune system finding, Haemostatic system finding'),
('2908-VENOM', '2908', 'Exercise-induced collapse in Labrador Retrievers', 'Musculoskeletal system finding, Hereditary disorder finding'),
('732-VENOM', '732', 'Exocrine pancreatic insufficiency (EPI)', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('733-VENOM', '733', 'Eye proptosed', 'Ophthalmology/vision finding, Head and neck finding'),
('734-VENOM', '734', 'Eyelid coloboma/agenesis', 'Ophthalmology/vision finding, Congenital finding, Head and neck finding'),
('735-VENOM', '735', 'Eyelid laceration', 'Ophthalmology/vision finding, Wound finding, Traumatic episode finding, Head and neck finding'),
('736-VENOM', '736', 'Facial neuropathy/nerve paralysis', 'Nervous system finding, Head and neck finding'),
('737-VENOM', '737', 'Faecal incontinence', 'Alimentary tract/Digestive system finding'),
('738-VENOM', '738', 'Familial nephropathy', 'Urogenital system finding, Inflammatory disorder finding, Hereditary disorder finding, Abdominal finding'),
('739-VENOM', '739', 'Fanconi''s syndrome', 'Urogenital system finding, Hereditary disorder finding, Abdominal finding'),
('740-VENOM', '740', 'Feline asthma (Feline allergic airway disease, FAAD)', 'Respiratory system finding, Thoracic finding'),
('741-VENOM', '741', 'Feline calicivirus infection', 'Infectious disorder finding, Respiratory system finding, Head and neck finding'),
('742-VENOM', '742', 'Feline central retinal degeneration (FCRD)', 'Ophthalmology/vision finding, Degenerative disorder finding, Head and neck finding'),
('743-VENOM', '743', 'Feline dysautonomia', 'Nervous system finding'),
('744-VENOM', '744', 'Feline eosinophilic granuloma complex', 'Haircoat/skin (+ appendages) finding, Immune system finding'),
('745-VENOM', '745', 'Feline hypereosinophilic syndrome', 'Haematopoietic system finding'),
('746-VENOM', '746', 'Feline idiopathic cystitis', 'Urogenital system finding, Inflammatory disorder finding, Abdominal finding'),
('747-VENOM', '747', 'Feline immunodeficiency virus (FIV) infection', 'Infectious disorder finding'),
('748-VENOM', '748', 'Feline infectious peritonitis (FIP)', 'Infectious disorder finding, Inflammatory disorder finding'),
('750-VENOM', '750', 'Feline leukaemia virus (FeLV) giant cell dermatitis', 'Infectious disorder finding, Haircoat/skin (+ appendages) finding, Inflammatory disorder finding'),
('749-VENOM', '749', 'Feline leukaemia virus (FeLV) infection', 'Infectious disorder finding'),
('751-VENOM', '751', 'Feline lower urinary tract disease (FLUTD)', 'Urogenital system finding'),
('9669-VENOM', '9669', 'Feline parvovirus infection (Feline panleucopenia)', 'Infectious disorder finding'),
('752-VENOM', '752', 'Feline ulcerative dermatitis', 'Haircoat/skin (+ appendages) finding, Inflammatory disorder finding'),
('753-VENOM', '753', 'Feline urological syndrome (FUS)', 'Urogenital system finding'),
('754-VENOM', '754', 'Feline viral respiratory disease complex', 'Infectious disorder finding, Respiratory system finding, Head and neck finding'),
('755-VENOM', '755', 'Feline viral rhinotracheitis', 'Infectious disorder finding, Respiratory system finding, Head and neck finding'),
('756-VENOM', '756', 'Fetal death/abortion', 'Reproductive system finding'),
('17016-VENOM', '17016', 'Fibroadenoma - mammary', 'Reproductive system finding, Connective tissue finding, Mass lesion/swelling finding'),
('757-VENOM', '757', 'Fibrocartilagenous embolism (FCE) of spinal cord', 'Nervous system finding'),
('17030-VENOM', '17030', 'Fibroma - ossifying', 'Connective tissue finding, Mass lesion/swelling finding'),
('17026-VENOM', '17026', 'Fibroma - urinary bladder', 'Urogenital system finding, Connective tissue finding, Mass lesion/swelling finding, Abdominal finding'),
('17024-VENOM', '17024', 'Fibroma (site unspecified)', 'Connective tissue finding, Mass lesion/swelling finding'),
('17028-VENOM', '17028', 'Fibroma (type unspecified)', 'Connective tissue finding, Mass lesion/swelling finding'),
('758-VENOM', '758', 'Fibropruritic nodule', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('17032-VENOM', '17032', 'Fibrosarcoma - bone', 'Musculoskeletal system finding, Connective tissue finding, Mass lesion/swelling finding'),
('17034-VENOM', '17034', 'Fibrosarcoma - cutaneous (skin)', 'Haircoat/skin (+ appendages) finding, Connective tissue finding, Mass lesion/swelling finding'),
('17040-VENOM', '17040', 'Fibrosarcoma - epididymal', 'Urogenital system finding, Reproductive system finding, Connective tissue finding, Mass lesion/swelling finding'),
('17042-VENOM', '17042', 'Fibrosarcoma - Feline injection site', 'Complication of medical care finding, Connective tissue finding, Soft tissue finding, Mass lesion/swelling finding'),
('17044-VENOM', '17044', 'Fibrosarcoma - hepatic (liver)', 'Hepatobiliary system finding, Connective tissue finding, Mass lesion/swelling finding, Abdominal finding'),
('17046-VENOM', '17046', 'Fibrosarcoma - intestinal', 'Alimentary tract/Digestive system finding, Connective tissue finding, Mass lesion/swelling finding, Abdominal finding'),
('17048-VENOM', '17048', 'Fibrosarcoma - mammary', 'Reproductive system finding, Connective tissue finding, Mass lesion/swelling finding'),
('17050-VENOM', '17050', 'Fibrosarcoma - nasal cavity', 'Respiratory system finding, Connective tissue finding, Mass lesion/swelling finding, Head and neck finding'),
('17052-VENOM', '17052', 'Fibrosarcoma - oral, mandible', 'Alimentary tract/Digestive system finding, Connective tissue finding, Mass lesion/swelling finding, Head and neck finding'),
('17055-VENOM', '17055', 'Fibrosarcoma - oral, maxilla', 'Alimentary tract/Digestive system finding, Connective tissue finding, Mass lesion/swelling finding, Head and neck finding'),
('17057-VENOM', '17057', 'Fibrosarcoma - oropharyngeal', 'Respiratory system finding, Alimentary tract/Digestive system finding, Connective tissue finding, Mass lesion/swelling finding, Head and neck finding'),
('17059-VENOM', '17059', 'Fibrosarcoma - renal (kidney)', 'Urogenital system finding, Connective tissue finding, Mass lesion/swelling finding, Abdominal finding'),
('17039-VENOM', '17039', 'Fibrosarcoma - soft tissue', 'Connective tissue finding, Soft tissue finding, Mass lesion/swelling finding'),
('17036-VENOM', '17036', 'Fibrosarcoma - subcutaneous', 'Haircoat/skin (+ appendages) finding, Connective tissue finding, Mass lesion/swelling finding'),
('17061-VENOM', '17061', 'Fibrosarcoma - urinary bladder', 'Urogenital system finding, Connective tissue finding, Mass lesion/swelling finding, Abdominal finding'),
('16420-VENOM', '16420', 'Fibrosarcoma (site unspecified)', 'Connective tissue finding, Mass lesion/swelling finding'),
('4564-VENOM', '4564', 'First degree AV block', 'Cardiovascular system finding, Thoracic finding'),
('760-VENOM', '760', 'Flail chest', 'Musculoskeletal system finding, Traumatic episode finding, Thoracic finding'),
('761-VENOM', '761', 'Fly bites', 'Haircoat/skin (+ appendages) finding, Infectious disorder finding'),
('762-VENOM', '762', 'Follicular cornification disorder', 'Haircoat/skin (+ appendages) finding'),
('763-VENOM', '763', 'Follicular dysplasia', 'Haircoat/skin (+ appendages) finding'),
('764-VENOM', '764', 'Folliculitis and furunculosis - of muzzle', 'Haircoat/skin (+ appendages) finding, Inflammatory disorder finding, Infectious disorder finding, Head and neck finding'),
('765-VENOM', '765', 'Folliculitis and furunculosis - pyotraumatic', 'Haircoat/skin (+ appendages) finding, Inflammatory disorder finding, Infectious disorder finding'),
('767-VENOM', '767', 'Food intolerance', 'Adverse reaction finding'),
('768-VENOM', '768', 'Footpad hyperkeratosis', 'Haircoat/skin (+ appendages) finding, Limb finding'),
('769-VENOM', '769', 'Foreign body', 'Foreign body finding'),
('770-VENOM', '770', 'Foreign body - aural (ear)', 'Foreign body finding, Auditory system finding, Head and neck finding'),
('771-VENOM', '771', 'Foreign body - bronchial', 'Foreign body finding, Respiratory system finding, Thoracic finding'),
('772-VENOM', '772', 'Foreign body - cutaneous (skin)', 'Foreign body finding, Haircoat/skin (+ appendages) finding'),
('773-VENOM', '773', 'Foreign body - gastric (stomach)', 'Foreign body finding, Alimentary tract/Digestive system finding, Abdominal finding'),
('774-VENOM', '774', 'Foreign body - intestinal, large', 'Foreign body finding, Alimentary tract/Digestive system finding, Abdominal finding'),
('775-VENOM', '775', 'Foreign body - intestinal, small', 'Foreign body finding, Alimentary tract/Digestive system finding, Abdominal finding'),
('776-VENOM', '776', 'Foreign body - intra-ocular', 'Foreign body finding, Ophthalmology/vision finding, Head and neck finding'),
('778-VENOM', '778', 'Foreign body - intra-ocular, anterior chamber', 'Foreign body finding, Ophthalmology/vision finding, Head and neck finding'),
('777-VENOM', '777', 'Foreign body - intra-ocular, corneal', 'Foreign body finding, Ophthalmology/vision finding, Head and neck finding'),
('779-VENOM', '779', 'Foreign body - limb', 'Foreign body finding, Limb finding'),
('780-VENOM', '780', 'Foreign body - nasal (nose)', 'Foreign body finding, Respiratory system finding, Head and neck finding'),
('781-VENOM', '781', 'Foreign body - nasal sinus', 'Foreign body finding, Respiratory system finding, Head and neck finding'),
('782-VENOM', '782', 'Foreign body - oesophageal', 'Foreign body finding, Alimentary tract/Digestive system finding'),
('783-VENOM', '783', 'Foreign body - oral (mouth)', 'Foreign body finding, Alimentary tract/Digestive system finding, Head and neck finding'),
('784-VENOM', '784', 'Foreign body - orbital', 'Foreign body finding, Ophthalmology/vision finding, Head and neck finding'),
('785-VENOM', '785', 'Foreign body - peritoneal', 'Foreign body finding, Abdominal finding'),
('786-VENOM', '786', 'Foreign body - pharyngeal', 'Foreign body finding, Alimentary tract/Digestive system finding, Respiratory system finding'),
('787-VENOM', '787', 'Foreign body - pulmonary (lung)', 'Foreign body finding, Respiratory system finding, Thoracic finding'),
('788-VENOM', '788', 'Foreign body - retrobulbar', 'Foreign body finding, Ophthalmology/vision finding, Head and neck finding'),
('789-VENOM', '789', 'Foreign body - retroperitoneal', 'Foreign body finding, Abdominal finding'),
('790-VENOM', '790', 'Foreign body - third eyelid/nictitating membrane', 'Foreign body finding, Ophthalmology/vision finding, Soft tissue finding, Head and neck finding'),
('8543-VENOM', '8543', 'Foreign body - tracheal', 'Foreign body finding, Respiratory system finding'),
('796-VENOM', '796', 'Foreign body left post-operatively - clinical object', 'Foreign body finding, Complication of medical care finding'),
('797-VENOM', '797', 'Foreign body left post-operatively - non-clinical object', 'Foreign body finding, Complication of medical care finding'),
('792-VENOM', '792', 'Foreign body, air gun pellet', 'Foreign body finding'),
('791-VENOM', '791', 'Foreign body, alimentary tract - linear', 'Foreign body finding, Alimentary tract/Digestive system finding'),
('793-VENOM', '793', 'Foreign body, bone', 'Foreign body finding'),
('794-VENOM', '794', 'Foreign body, fishing hook', 'Foreign body finding'),
('795-VENOM', '795', 'Foreign body, grass seed', 'Foreign body finding'),
('798-VENOM', '798', 'Fracture', 'Musculoskeletal system finding'),
('814-VENOM', '814', 'Fracture healed', 'Musculoskeletal system finding'),
('815-VENOM', '815', 'Fracture healing', 'Musculoskeletal system finding'),
('816-VENOM', '816', 'Fracture healing disorder - delayed union/non-union', 'Musculoskeletal system finding'),
('817-VENOM', '817', 'Fracture healing disorder - malunion', 'Musculoskeletal system finding'),
('5845-VENOM', '5845', 'Fracture of os penis', 'Musculoskeletal system finding, Urogenital system finding, Reproductive system finding'),
('818-VENOM', '818', 'Fracture of pelvic limb', 'Musculoskeletal system finding, Limb finding'),
('819-VENOM', '819', 'Fracture of pelvic limb - calcaneus', 'Musculoskeletal system finding, Limb finding'),
('820-VENOM', '820', 'Fracture of pelvic limb - femur', 'Musculoskeletal system finding, Limb finding'),
('821-VENOM', '821', 'Fracture of pelvic limb - fibula', 'Musculoskeletal system finding, Limb finding'),
('822-VENOM', '822', 'Fracture of pelvic limb - metatarsal(s)', 'Musculoskeletal system finding, Limb finding'),
('823-VENOM', '823', 'Fracture of pelvic limb - patella', 'Musculoskeletal system finding, Limb finding'),
('824-VENOM', '824', 'Fracture of pelvic limb - paw, multiple fractures', 'Musculoskeletal system finding, Limb finding'),
('825-VENOM', '825', 'Fracture of pelvic limb - phalanx', 'Musculoskeletal system finding, Limb finding'),
('826-VENOM', '826', 'Fracture of pelvic limb - talus', 'Musculoskeletal system finding, Limb finding'),
('827-VENOM', '827', 'Fracture of pelvic limb - tarsal bone, central', 'Musculoskeletal system finding, Limb finding'),
('828-VENOM', '828', 'Fracture of pelvic limb - tarsal bone, distal', 'Musculoskeletal system finding, Limb finding'),
('829-VENOM', '829', 'Fracture of pelvic limb - tibia', 'Musculoskeletal system finding, Limb finding'),
('830-VENOM', '830', 'Fracture of pelvic limb - tibia AND fibula', 'Musculoskeletal system finding, Limb finding'),
('831-VENOM', '831', 'Fracture of pelvis', 'Musculoskeletal system finding, Pelvic structure finding'),
('832-VENOM', '832', 'Fracture of pelvis - acetabulum', 'Musculoskeletal system finding, Pelvic structure finding'),
('833-VENOM', '833', 'Fracture of pelvis - ilium', 'Musculoskeletal system finding, Pelvic structure finding'),
('834-VENOM', '834', 'Fracture of pelvis - ischium', 'Musculoskeletal system finding, Pelvic structure finding'),
('835-VENOM', '835', 'Fracture of pelvis - multiple fractures', 'Musculoskeletal system finding, Pelvic structure finding'),
('836-VENOM', '836', 'Fracture of pelvis - pubis', 'Musculoskeletal system finding, Pelvic structure finding'),
('837-VENOM', '837', 'Fracture of pelvis with sacroiliac luxation/subluxation', 'Musculoskeletal system finding, Pelvic structure finding'),
('838-VENOM', '838', 'Fracture of rib(s)', 'Musculoskeletal system finding, Thoracic finding'),
('839-VENOM', '839', 'Fracture of skull', 'Musculoskeletal system finding, Head and neck finding'),
('840-VENOM', '840', 'Fracture of skull - frontal bone', 'Musculoskeletal system finding, Head and neck finding'),
('841-VENOM', '841', 'Fracture of skull - frontal sinus', 'Musculoskeletal system finding, Respiratory system finding, Head and neck finding'),
('842-VENOM', '842', 'Fracture of skull - involving temporomandibular joint (TMJ)', 'Musculoskeletal system finding, Head and neck finding'),
('843-VENOM', '843', 'Fracture of skull - mandible', 'Musculoskeletal system finding, Head and neck finding'),
('844-VENOM', '844', 'Fracture of skull - maxilla', 'Musculoskeletal system finding, Head and neck finding'),
('845-VENOM', '845', 'Fracture of skull - multiple fractures', 'Musculoskeletal system finding, Head and neck finding'),
('846-VENOM', '846', 'Fracture of skull - nasal bone', 'Musculoskeletal system finding, Respiratory system finding, Head and neck finding'),
('847-VENOM', '847', 'Fracture of skull - orbit', 'Musculoskeletal system finding, Ophthalmology/vision finding, Head and neck finding'),
('848-VENOM', '848', 'Fracture of skull - zygomatic bone', 'Musculoskeletal system finding, Head and neck finding'),
('849-VENOM', '849', 'Fracture of sternum', 'Musculoskeletal system finding, Thoracic finding'),
('850-VENOM', '850', 'Fracture of thoracic limb', 'Musculoskeletal system finding, Limb finding'),
('851-VENOM', '851', 'Fracture of thoracic limb - carpal bone, accessory', 'Musculoskeletal system finding, Limb finding'),
('852-VENOM', '852', 'Fracture of thoracic limb - carpal bone, distal', 'Musculoskeletal system finding, Limb finding'),
('853-VENOM', '853', 'Fracture of thoracic limb - carpal bone, radial', 'Musculoskeletal system finding, Limb finding'),
('854-VENOM', '854', 'Fracture of thoracic limb - carpal bone, ulnar', 'Musculoskeletal system finding, Limb finding'),
('855-VENOM', '855', 'Fracture of thoracic limb - carpal bones, multiple', 'Musculoskeletal system finding, Limb finding'),
('856-VENOM', '856', 'Fracture of thoracic limb - clavicle', 'Musculoskeletal system finding, Limb finding'),
('857-VENOM', '857', 'Fracture of thoracic limb - humerus', 'Musculoskeletal system finding, Limb finding'),
('858-VENOM', '858', 'Fracture of thoracic limb - humerus, intercondylar Y/T', 'Musculoskeletal system finding, Limb finding'),
('859-VENOM', '859', 'Fracture of thoracic limb - humerus, lateral condyle', 'Musculoskeletal system finding, Limb finding'),
('860-VENOM', '860', 'Fracture of thoracic limb - humerus, medial condyle', 'Musculoskeletal system finding, Limb finding'),
('861-VENOM', '861', 'Fracture of thoracic limb - metacarpal(s)', 'Musculoskeletal system finding, Limb finding'),
('862-VENOM', '862', 'Fracture of thoracic limb - paw, multiple fractures', 'Musculoskeletal system finding, Limb finding'),
('863-VENOM', '863', 'Fracture of thoracic limb - phalanx', 'Musculoskeletal system finding, Limb finding'),
('864-VENOM', '864', 'Fracture of thoracic limb - radius', 'Musculoskeletal system finding, Limb finding'),
('865-VENOM', '865', 'Fracture of thoracic limb - radius AND ulna', 'Musculoskeletal system finding, Limb finding'),
('866-VENOM', '866', 'Fracture of thoracic limb - scapula', 'Musculoskeletal system finding, Limb finding'),
('867-VENOM', '867', 'Fracture of thoracic limb - ulna', 'Musculoskeletal system finding, Limb finding'),
('868-VENOM', '868', 'Fracture of thoracic limb - ulna, Monteggia''s fracture', 'Musculoskeletal system finding, Limb finding'),
('869-VENOM', '869', 'Fracture of thoracic limb - ulna, olecranon', 'Musculoskeletal system finding, Limb finding'),
('870-VENOM', '870', 'Fracture of vertebral column', 'Musculoskeletal system finding, Spine (vertebral column) finding'),
('871-VENOM', '871', 'Fracture of vertebral column - cervical', 'Musculoskeletal system finding, Spine (vertebral column) finding, Head and neck finding'),
('877-VENOM', '877', 'Fracture of vertebral column - coccygeal', 'Musculoskeletal system finding, Spine (vertebral column) finding, Tail finding'),
('872-VENOM', '872', 'Fracture of vertebral column - first cervical vertebra (C1)', 'Musculoskeletal system finding, Spine (vertebral column) finding, Head and neck finding'),
('874-VENOM', '874', 'Fracture of vertebral column - lumbar', 'Musculoskeletal system finding, Spine (vertebral column) finding, Abdominal finding'),
('876-VENOM', '876', 'Fracture of vertebral column - sacrum', 'Musculoskeletal system finding, Spine (vertebral column) finding, Pelvic structure finding'),
('873-VENOM', '873', 'Fracture of vertebral column - second cervical vertebra (C2)', 'Musculoskeletal system finding, Spine (vertebral column) finding, Head and neck finding'),
('875-VENOM', '875', 'Fracture of vertebral column - thoracic', 'Musculoskeletal system finding, Spine (vertebral column) finding, Thoracic finding'),
('799-VENOM', '799', 'Fracture, closed', 'Musculoskeletal system finding'),
('800-VENOM', '800', 'Fracture, comminuted', 'Musculoskeletal system finding'),
('801-VENOM', '801', 'Fracture, greenstick', 'Musculoskeletal system finding'),
('802-VENOM', '802', 'Fracture, intra-articular', 'Musculoskeletal system finding'),
('803-VENOM', '803', 'Fracture, oblique', 'Musculoskeletal system finding'),
('804-VENOM', '804', 'Fracture, open', 'Musculoskeletal system finding'),
('805-VENOM', '805', 'Fracture, pathological', 'Musculoskeletal system finding'),
('806-VENOM', '806', 'Fracture, Salter-Harris Type I', 'Musculoskeletal system finding'),
('807-VENOM', '807', 'Fracture, Salter-Harris Type II', 'Musculoskeletal system finding'),
('808-VENOM', '808', 'Fracture, Salter-Harris Type III', 'Musculoskeletal system finding'),
('809-VENOM', '809', 'Fracture, Salter-Harris Type IV', 'Musculoskeletal system finding'),
('810-VENOM', '810', 'Fracture, Salter-Harris Type V', 'Musculoskeletal system finding'),
('811-VENOM', '811', 'Fracture, spiral', 'Musculoskeletal system finding'),
('813-VENOM', '813', 'Fracture, transverse', 'Musculoskeletal system finding'),
('812-VENOM', '812', 'Fracture. stress', 'Musculoskeletal system finding'),
('878-VENOM', '878', 'Furunculosis', 'Haircoat/skin (+ appendages) finding, Inflammatory disorder finding'),
('879-VENOM', '879', 'Galactosialidosis', 'Endocrine/metabolic finding, Nervous system finding, Congenital finding, Hereditary disorder finding'),
('880-VENOM', '880', 'Gallbladder mucocele', 'Hepatobiliary system finding, Abdominal finding'),
('881-VENOM', '881', 'Gallbladder perforation', 'Hepatobiliary system finding, Abdominal finding'),
('882-VENOM', '882', 'Gallbladder rupture', 'Hepatobiliary system finding, Abdominal finding'),
('883-VENOM', '883', 'Gastric (stomach) dilation', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('885-VENOM', '885', 'Gastric (stomach) outflow (pyloric) obstruction', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('886-VENOM', '886', 'Gastric (stomach) perforation', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('887-VENOM', '887', 'Gastric (stomach) rupture', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('888-VENOM', '888', 'Gastric (stomach) torsion - chronic', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('889-VENOM', '889', 'Gastric (stomach) ulceration', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('884-VENOM', '884', 'Gastric dilation-volvulus syndrome (GDV)', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('17063-VENOM', '17063', 'Gastrinoma', 'Alimentary tract/Digestive system finding, Endocrine/Metabolic finding, Mass lesion/swelling finding'),
('890-VENOM', '890', 'Gastritis', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding, Abdominal finding'),
('891-VENOM', '891', 'Gastritis - acute', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding, Abdominal finding'),
('892-VENOM', '892', 'Gastritis - chronic', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding, Abdominal finding'),
('893-VENOM', '893', 'Gastritis - hypertrophic', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding, Abdominal finding'),
('894-VENOM', '894', 'Gastritis - lymphocytic', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding, Abdominal finding'),
('895-VENOM', '895', 'Gastroenteritis', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding, Abdominal finding'),
('896-VENOM', '896', 'Gastroenteritis - acute', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding, Abdominal finding'),
('897-VENOM', '897', 'Gastroesophageal reflux', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('7457-VENOM', '7457', 'Gastrointestinal disorder, parasitic', 'Alimentary tract/Digestive system finding, Infectious disorder finding, Abdominal finding'),
('898-VENOM', '898', 'Gastrointestinal haemorrhage', 'Alimentary tract/Digestive system finding, Cardiovascular system finding, Abdominal finding'),
('7459-VENOM', '7459', 'Gastrointestinal helminth infestation (worms)', 'Alimentary tract/Digestive system finding, Infectious disorder finding, Abdominal finding'),
('17070-VENOM', '17070', 'Gastrointestinal Stromal Tumour (GIST) - large intestine, colon', 'Alimentary tract/Digestive system finding, Connective tissue finding, Mass lesion/swelling finding, Abdominal finding'),
('17072-VENOM', '17072', 'Gastrointestinal Stromal Tumour (GIST) - large intestine, rectum', 'Alimentary tract/Digestive system finding, Connective tissue finding, Mass lesion/swelling finding, Abdominal finding, Pelvic structure finding'),
('17074-VENOM', '17074', 'Gastrointestinal Stromal Tumour (GIST) - small intestine', 'Alimentary tract/Digestive system finding, Connective tissue finding, Mass lesion/swelling finding, Abdominal finding'),
('17076-VENOM', '17076', 'Gastrointestinal Stromal Tumour (GIST) - stomach (gastric)', 'Alimentary tract/Digestive system finding, Connective tissue finding, Mass lesion/swelling finding, Abdominal finding'),
('17065-VENOM', '17065', 'Gastrointestinal Stromal Tumour (GIST)(site unspecified)', 'Alimentary tract/Digestive system finding, Connective tissue finding, Mass lesion/swelling finding, Abdominal finding'),
('899-VENOM', '899', 'Giardiasis', 'Infectious disorder finding'),
('900-VENOM', '900', 'Gingivitis/gingival (gum) disease', 'Alimentary tract/Digestive system finding, Head and neck finding'),
('901-VENOM', '901', 'Glaucoma', 'Ophthalmology/vision finding, Head and neck finding'),
('902-VENOM', '902', 'Glaucoma - primary', 'Ophthalmology/vision finding, Head and neck finding'),
('903-VENOM', '903', 'Glaucoma - secondary', 'Ophthalmology/vision finding, Head and neck finding'),
('16811-VENOM', '16811', 'Glioma, brain - astrocytoma', 'Nervous system finding, Mass lesion/swelling finding, Head and neck finding'),
('16814-VENOM', '16814', 'Glioma, brain - oligoastrocytoma', 'Nervous system finding, Mass lesion/swelling finding, Head and neck finding'),
('16817-VENOM', '16817', 'Glioma, brain - oligodendroglioma', 'Nervous system finding, Mass lesion/swelling finding, Head and neck finding'),
('16809-VENOM', '16809', 'Glioma, brain (type unspecified)', 'Nervous system finding, Mass lesion/swelling finding, Head and neck finding'),
('16822-VENOM', '16822', 'Glioma, spinal cord - astrocytoma', 'Nervous system finding, Mass lesion/swelling finding'),
('16824-VENOM', '16824', 'Glioma, spinal cord - oligoastrocytoma', 'Nervous system finding, Mass lesion/swelling finding'),
('16826-VENOM', '16826', 'Glioma, spinal cord - oligodendroglioma', 'Nervous system finding, Mass lesion/swelling finding'),
('16820-VENOM', '16820', 'Glioma, spinal cord (type unspecified)', 'Nervous system finding, Mass lesion/swelling finding'),
('907-VENOM', '907', 'Glomerulonephritis', 'Urogenital system finding, Inflammatory disorder finding, Abdominal finding'),
('17078-VENOM', '17078', 'Glucagonoma', 'Alimentary tract/Digestive system finding, Endocrine/Metabolic finding, Neuroendocrine system finding, Abdominal finding'),
('8370-VENOM', '8370', 'Glucosuria', 'Urogenital system finding'),
('17083-VENOM', '17083', 'Granular cell myoblastoma - central nervous system (CNS)', 'Nervous system finding, Mass lesion/swelling finding'),
('17085-VENOM', '17085', 'Granular cell myoblastoma - oropharyngeal', 'Respiratory system finding, Alimentary tract/Digestive system finding, Nervous system finding, Mass lesion/swelling finding, Head and neck finding'),
('17081-VENOM', '17081', 'Granular cell myoblastoma (site unspecified)', 'Nervous system finding, Mass lesion/swelling finding'),
('17090-VENOM', '17090', 'Granuloma - renal (kidney)', 'Urogenital system finding, Immune system finding, Inflammatory disorder finding, Mass lesion/swelling finding, Abdominal finding '),
('17087-VENOM', '17087', 'Granuloma (site unspecified)', 'Immune system finding, Inflammatory disorder finding, Mass lesion/swelling finding'),
('908-VENOM', '908', 'Granulomatous meningoencephalitis (GME)', 'Nervous system finding, Inflammatory disorder finding, Head and neck finding'),
('909-VENOM', '909', 'Granulomatous meningoencephalomyelitis', 'Nervous system finding, Inflammatory disorder finding, Head and neck finding'),
('17092-VENOM', '17092', 'Granulosa cell tumour (ovary)', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding, Abdominal finding'),
('17095-VENOM', '17095', 'Haemangioma - bone', 'Musculoskeletal system finding, Cardiovascular system finding, Mass lesion/swelling finding'),
('17097-VENOM', '17097', 'Haemangioma - cutaneous (skin)', 'Haircoat/skin (+ appendages) finding, Cardiovascular system finding, Mass lesion/swelling finding'),
('17099-VENOM', '17099', 'Haemangioma - oropharyngeal', 'Respiratory system finding, Alimentary tract/Digestive system finding, Cardiovascular system finding, Mass lesion/swelling finding, Head and neck finding'),
('911-VENOM', '911', 'Haemangioma - splenic', 'Haematopoietic system finding, Cardiovascular system finding, Mass lesion/swelling finding, Abdominal finding'),
('16422-VENOM', '16422', 'Haemangioma (site unspecified)', 'Cardiovascular system finding, Mass lesion/swelling finding'),
('912-VENOM', '912', 'Haemangiopericytoma', 'Cardiovascular system finding, Mass lesion/swelling finding'),
('17101-VENOM', '17101', 'Haemangiosarcoma - bone', 'Musculoskeletal system finding, Cardiovascular system finding, Connective tissue finding, Mass lesion/swelling finding'),
('17103-VENOM', '17103', 'Haemangiosarcoma - cardiac (heart)', 'Cardiovascular system finding, Connective tissue finding, Thoracic finding, Mass lesion/swelling finding'),
('915-VENOM', '915', 'Haemangiosarcoma - cutaneous (skin)', 'Cardiovascular system finding, Connective tissue finding, Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('5843-VENOM', '5843', 'Haemangiosarcoma - hepatic (liver)', 'Cardiovascular system finding, Connective tissue finding, Hepatobiliary system finding, Mass lesion/swelling finding, Abdominal finding'),
('17105-VENOM', '17105', 'Haemangiosarcoma - metastatic/disseminated', 'Cardiovascular system finding, Connective tissue finding, Mass lesion/swelling finding'),
('17107-VENOM', '17107', 'Haemangiosarcoma - retroperitoneal', 'Cardiovascular system finding, Connective tissue finding, Abdominal finding, Mass lesion/swelling finding'),
('916-VENOM', '916', 'Haemangiosarcoma - splenic', 'Cardiovascular system finding, Connective tissue finding, Haematopoietic system finding, Mass lesion/swelling finding, Abdominal finding'),
('17109-VENOM', '17109', 'Haemangiosarcoma - subcutaneous', 'Cardiovascular system finding, Connective tissue finding, Soft tissue finding, Mass lesion/swelling finding'),
('16424-VENOM', '16424', 'Haemangiosarcoma (site unspecified)', 'Cardiovascular system finding, Connective tissue finding, Mass lesion/swelling finding'),
('8368-VENOM', '8368', 'Haematoma - other', 'Cardiovascular system finding'),
('917-VENOM', '917', 'Haematuria', 'Urogenital system finding, Cardiovascular system finding'),
('918-VENOM', '918', 'Haematuria - non-traumatic', 'Urogenital system finding, Cardiovascular system finding'),
('919-VENOM', '919', 'Haematuria - traumatic', 'Urogenital system finding, Cardiovascular system finding, Traumatic episode finding'),
('920-VENOM', '920', 'Haemopericardium', 'Cardiovascular system finding, Effusion finding, Thoracic finding'),
('921-VENOM', '921', 'Haemoperitoneum (haemoabdomen)', 'Cardiovascular system finding, Effusion finding, Abdominal finding'),
('922-VENOM', '922', 'Haemoperitoneum (haemoabdomen) - non-traumatic', 'Cardiovascular system finding, Effusion finding, Abdominal finding'),
('923-VENOM', '923', 'Haemoperitoneum (haemoabdomen) - traumatic', 'Cardiovascular system finding, Effusion finding, Traumatic episode finding, Abdominal finding'),
('18052-VENOM', '18052', 'Haemophagocytic syndrome - idiopathic', 'Haematopoietic system finding'),
('18053-VENOM', '18053', 'Haemophagocytic syndrome - underlying disease identified', 'Haematopoietic system finding'),
('924-VENOM', '924', 'Haemophilia', 'Haemostatic system finding, Hereditary disorder finding'),
('18050-VENOM', '18050', 'Haemorrhagic pneumonia - infectious agent unknown', 'Infectious disorder finding, Respiratory system finding, Thoracic finding'),
('18042-VENOM', '18042', 'Haemorrhagic pneumonia (infectious agent unspecified)', 'Infectious disorder finding, Respiratory system finding, Thoracic finding'),
('18046-VENOM', '18046', 'Haemorrhagic pneumonia due to E. coli', 'Infectious disorder finding, Respiratory system finding, Thoracic finding'),
('18048-VENOM', '18048', 'Haemorrhagic pneumonia due to leptospirosis', 'Infectious disorder finding, Respiratory system finding, Thoracic finding'),
('18044-VENOM', '18044', 'Haemorrhagic pneumonia due to S. equi subsp. zooepidemicus', 'Infectious disorder finding, Respiratory system finding, Thoracic finding'),
('7429-VENOM', '7429', 'Haemorrhagic Viral Disease (HVD) of rabbits', ''),
('925-VENOM', '925', 'Haemothorax', 'Cardiovascular system finding, Effusion finding, Thoracic finding'),
('926-VENOM', '926', 'Hair shaft abnormality', 'Haircoat/skin (+ appendages) finding'),
('17111-VENOM', '17111', 'Hamartoma', 'Mass lesion/swelling finding'),
('927-VENOM', '927', 'Heart (cardiac) anomaly', 'Cardiovascular system finding, Thoracic finding'),
('928-VENOM', '928', 'Heart (cardiac) anomaly, congenital', 'Cardiovascular system finding, Congenital finding, Thoracic finding'),
('929-VENOM', '929', 'Heart (cardiac) anomaly, congenital - Atrial septal defect (ASD)', 'Cardiovascular system finding, Congenital finding, Thoracic finding'),
('930-VENOM', '930', 'Heart (cardiac) anomaly, congenital - Atrioventricular septal defect (AVSD)', 'Cardiovascular system finding, Congenital finding, Thoracic finding'),
('931-VENOM', '931', 'Heart (cardiac) anomaly, congenital - Cor triatriatum dexter', 'Cardiovascular system finding, Congenital finding, Thoracic finding'),
('932-VENOM', '932', 'Heart (cardiac) anomaly, congenital - Cor triatriatum sinister', 'Cardiovascular system finding, Congenital finding, Thoracic finding'),
('934-VENOM', '934', 'Heart (cardiac) anomaly, congenital - Double outlet right ventricle', 'Cardiovascular system finding, Congenital finding, Thoracic finding'),
('933-VENOM', '933', 'Heart (cardiac) anomaly, congenital - Double-chambered right ventricle', 'Cardiovascular system finding, Congenital finding, Thoracic finding'),
('7386-VENOM', '7386', 'Heart (cardiac) anomaly, congenital - Patent/Persistence of Foramen Ovale (PFO)', 'Cardiovascular system finding, Congenital finding, Thoracic finding'),
('935-VENOM', '935', 'Heart (cardiac) anomaly, congenital - Tetralogy of Fallot', 'Cardiovascular system finding, Congenital finding, Thoracic finding'),
('936-VENOM', '936', 'Heart (cardiac) anomaly, congenital - Ventricular septal defect (VSD)', 'Cardiovascular system finding, Congenital finding, Thoracic finding'),
('937-VENOM', '937', 'Heart (cardiac) disease', 'Cardiovascular system finding, Thoracic finding'),
('3436-VENOM', '3436', 'Heart (cardiac) disease - acute', 'Cardiovascular system finding, Thoracic finding'),
('4543-VENOM', '4543', 'Heart (cardiac) disease - chronic', 'Cardiovascular system finding, Thoracic finding'),
('938-VENOM', '938', 'Heart (cardiac) disease, congenital', 'Cardiovascular system finding, Congenital finding, Thoracic finding'),
('939-VENOM', '939', 'Heart (cardiac) failure', 'Cardiovascular system finding, Thoracic finding'),
('940-VENOM', '940', 'Heart (cardiac) failure - acute left-sided', 'Cardiovascular system finding, Thoracic finding'),
('941-VENOM', '941', 'Heart (cardiac) failure - acute right-sided', 'Cardiovascular system finding, Thoracic finding'),
('942-VENOM', '942', 'Heart (cardiac) failure, congestive', 'Cardiovascular system finding, Thoracic finding'),
('943-VENOM', '943', 'Heart (cardiac) failure, congestive, left-sided - acute', 'Cardiovascular system finding, Thoracic finding'),
('944-VENOM', '944', 'Heart (cardiac) failure, congestive, left-sided - chronic', 'Cardiovascular system finding, Thoracic finding'),
('945-VENOM', '945', 'Heart (cardiac) failure, congestive, right-sided - acute', 'Cardiovascular system finding, Thoracic finding'),
('946-VENOM', '946', 'Heart (cardiac) failure, congestive, right-sided - chronic', 'Cardiovascular system finding, Thoracic finding'),
('947-VENOM', '947', 'Heart (cardiac) murmur', 'Cardiovascular system finding, Thoracic finding'),
('948-VENOM', '948', 'Heart (cardiac) murmur - functional', 'Cardiovascular system finding, Thoracic finding'),
('949-VENOM', '949', 'Heart (cardiac) murmur - innocent', 'Cardiovascular system finding, Thoracic finding'),
('950-VENOM', '950', 'Heart (cardiac) valve anomaly, congenital', 'Cardiovascular system finding, Thoracic finding'),
('951-VENOM', '951', 'Heart (cardiac) valve disorder', 'Cardiovascular system finding, Thoracic finding'),
('952-VENOM', '952', 'Heat stroke', 'Body temperature finding'),
('953-VENOM', '953', 'Helicobacter infection', 'Alimentary tract/Digestive system finding, Infectious disorder finding, Abdominal finding'),
('954-VENOM', '954', 'Helicobacter-like organism infection', 'Alimentary tract/Digestive system finding, Infectious disorder finding, Abdominal finding'),
('955-VENOM', '955', 'Hemaralopia (day blindness)', 'Ophthalmology/vision finding, Head and neck finding'),
('956-VENOM', '956', 'Hepatic (liver) cirrhosis', 'Hepatobiliary system finding, Abdominal finding'),
('957-VENOM', '957', 'Hepatic (liver) encephalopathy (HE)', 'Hepatobiliary system finding, Nervous system finding, Endocrine/metabolic finding, Abdominal finding, Head and neck finding'),
('958-VENOM', '958', 'Hepatic (liver) failure - acute', 'Hepatobiliary system finding, Abdominal finding'),
('959-VENOM', '959', 'Hepatic (liver) failure - chronic', 'Hepatobiliary system finding, Abdominal finding'),
('960-VENOM', '960', 'Hepatic (liver) lipidosis', 'Hepatobiliary system finding, Abdominal finding'),
('961-VENOM', '961', 'Hepatic (liver) necrosis', 'Hepatobiliary system finding, Abdominal finding'),
('962-VENOM', '962', 'Hepatic (liver) nodular hyperplasia', 'Hepatobiliary system finding, Abdominal finding'),
('963-VENOM', '963', 'Hepatitis', 'Hepatobiliary system finding, Inflammatory disorder finding, Abdominal finding'),
('964-VENOM', '964', 'Hepatitis - acute', 'Hepatobiliary system finding, Inflammatory disorder finding, Abdominal finding'),
('965-VENOM', '965', 'Hepatitis - chronic', 'Hepatobiliary system finding, Inflammatory disorder finding, Abdominal finding'),
('9896-VENOM', '9896', 'Hepatocutaneous syndrome (Metabolic epidermal necrosis, Supf necrolytic dermatitis)', 'Endocrine/Metabolic finding, Haircoat/skin (+ appendages) finding, Hepatobiliary system finding, Abdominal finding'),
('966-VENOM', '966', 'Hepatoma', 'Hepatobiliary system finding, Mass lesion/swelling finding, Abdominal finding'),
('967-VENOM', '967', 'Hepatopathy (liver disorder)', 'Hepatobiliary system finding, Abdominal finding'),
('968-VENOM', '968', 'Hepatotoxicity (liver toxicity)', 'Hepatobiliary system finding, Abdominal finding'),
('969-VENOM', '969', 'Hereditary ataxia in Terriers', 'Nervous system finding, Hereditary disorder finding'),
('9575-VENOM', '9575', 'Hereditary metabolic disorder', 'Hereditary disorder finding, Endocrine/Metabolic finding'),
('970-VENOM', '970', 'Hermaphroditism', 'Urogenital system finding, Reproductive system finding'),
('971-VENOM', '971', 'Hernia', 'Musculoskeletal system finding'),
('972-VENOM', '972', 'Hernia - abdominal', 'Musculoskeletal system finding, Abdominal finding'),
('973-VENOM', '973', 'Hernia - diaphragmatic', 'Musculoskeletal system finding, Thoracic finding, Abdominal finding'),
('974-VENOM', '974', 'Hernia - diaphragmatic, congenital', 'Musculoskeletal system finding, Congenital finding, Thoracic finding, Abdominal finding'),
('975-VENOM', '975', 'Hernia - hiatal', 'Musculoskeletal system finding, Alimentary tract/Digestive system finding, Thoracic finding, Abdominal finding'),
('976-VENOM', '976', 'Hernia - inguinal', 'Musculoskeletal system finding, Abdominal finding, Pelvic structure finding'),
('977-VENOM', '977', 'Hernia - perineal', 'Musculoskeletal system finding, Pelvic structure finding'),
('978-VENOM', '978', 'Hernia - peritoneopericardial', 'Musculoskeletal system finding, Cardiovascular system finding, Congenital finding, Thoracic finding, Abdominal finding'),
('979-VENOM', '979', 'Hernia - umbilical', 'Musculoskeletal system finding, Abdominal finding'),
('980-VENOM', '980', 'Hernia, acquired', 'Musculoskeletal system finding'),
('981-VENOM', '981', 'Hernia, congenital', 'Musculoskeletal system finding, Congenital finding'),
('982-VENOM', '982', 'Herpesvirus infection', 'Infectious disorder finding'),
('983-VENOM', '983', 'Hip dysplasia, developmental', 'Musculoskeletal system finding, Hereditary disorder finding, Developmental disorder finding, Limb finding'),
('984-VENOM', '984', 'Hip dysplasia, developmental - bilateral', 'Musculoskeletal system finding, Hereditary disorder finding, Developmental disorder finding, Limb finding'),
('985-VENOM', '985', 'Hip dysplasia, developmental - unilateral', 'Musculoskeletal system finding, Hereditary disorder finding, Developmental disorder finding, Limb finding'),
('16791-VENOM', '16791', 'Histiocytic sarcoma - disseminated (malignant histiocytosis)', 'Haematopoietic system finding, Connective tissue finding, Mass lesion/swelling finding'),
('16792-VENOM', '16792', 'Histiocytic sarcoma - localised (focal)', 'Haematopoietic system finding, Connective tissue finding, Mass lesion/swelling finding'),
('16793-VENOM', '16793', 'Histiocytic sarcoma - localised, metastatic', 'Haematopoietic system finding, Connective tissue finding, Mass lesion/swelling finding'),
('16828-VENOM', '16828', 'Histiocytoma - canine cutaneous (skin)', 'Haematopoietic system finding, Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('988-VENOM', '988', 'Histiocytosis - reactive histiocytosis, cutaneous (skin)', 'Haematopoietic system finding, Haircoat/skin (+ appendages) finding'),
('989-VENOM', '989', 'Histiocytosis - reactive histiocytosis, systemic', 'Haematopoietic system finding'),
('990-VENOM', '990', 'Hookworm dermatitis', 'Infectious disorder finding, Haircoat/skin (+ appendages) finding, Inflammatory disorder finding'),
('991-VENOM', '991', 'Horner''s syndrome', 'Nervous system finding'),
('992-VENOM', '992', 'Hydrocephalus', 'Nervous system finding, Head and neck finding'),
('993-VENOM', '993', 'Hydrocephalus - internal', 'Nervous system finding, Head and neck finding'),
('994-VENOM', '994', 'Hydrocephalus - post-traumatic', 'Nervous system finding, Traumatic episode finding, Head and neck finding'),
('995-VENOM', '995', 'Hydromyelia', 'Nervous system finding, Congenital finding'),
('996-VENOM', '996', 'Hydronephrosis', 'Urogenital system finding, Abdominal finding'),
('997-VENOM', '997', 'Hydroureter', 'Urogenital system finding, Abdominal finding'),
('998-VENOM', '998', 'Hygroma', 'Mass lesion/swelling finding'),
('999-VENOM', '999', 'Hymenoptera stings (bee, wasp)', 'Intoxication (poisoning)/toxin exposure finding, Wound finding, Traumatic episode finding'),
('9602-VENOM', '9602', 'Hyperadrenocorticism - adrenal-dependent ("Cushing''s")', 'Endocrine/Metabolic finding, Abdominal finding'),
('9605-VENOM', '9605', 'Hyperadrenocorticism - atypical (sex hormone overproduction)', 'Endocrine/Metabolic finding, Abdominal finding'),
('9604-VENOM', '9604', 'Hyperadrenocorticism - iatrogenic ("Cushing''s")', 'Endocrine/Metabolic finding, Abdominal finding'),
('9603-VENOM', '9603', 'Hyperadrenocorticism - pituitary-dependent ("Cushing''s")', 'Endocrine/Metabolic finding, Nervous system finding, Abdominal finding, Head and neck finding'),
('9601-VENOM', '9601', 'Hyperadrenocorticism ("Cushing''s")', 'Endocrine/Metabolic finding, Abdominal finding'),
('1000-VENOM', '1000', 'Hyperandrogenism', 'Endocrine/Metabolic finding, Urogenital system finding, Reproductive system finding'),
('1002-VENOM', '1002', 'Hypercalcaemia', 'Endocrine/Metabolic finding'),
('1003-VENOM', '1003', 'Hypercalcaemia - idiopathic', 'Endocrine/Metabolic finding'),
('1004-VENOM', '1004', 'Hypercalcaemia of malignancy (paraneoplastic)', 'Endocrine/Metabolic finding, Mass lesion/swelling finding'),
('1005-VENOM', '1005', 'Hypercalcaemic nephropathy', 'Endocrine/Metabolic finding, Urogenital system finding, Abdominal finding'),
('15591-VENOM', '15591', 'Hyperchloraemia', 'Endocrine/Metabolic finding'),
('1006-VENOM', '1006', 'Hypercholesterolaemia', 'Endocrine/Metabolic finding'),
('1007-VENOM', '1007', 'Hypercoagulable state', 'Haemostatic system finding'),
('1011-VENOM', '1011', 'Hyperglycaemia', 'Endocrine/Metabolic finding'),
('1012-VENOM', '1012', 'Hyperinsulinism', 'Endocrine/Metabolic finding, Alimentary tract/Digestive system finding, Abdominal finding'),
('1013-VENOM', '1013', 'Hyperkalaemia', 'Endocrine/Metabolic finding'),
('1014-VENOM', '1014', 'Hyperlipidaemia', 'Endocrine/Metabolic finding'),
('1015-VENOM', '1015', 'Hypermagnesaemia', 'Endocrine/Metabolic finding'),
('1016-VENOM', '1016', 'Hypernatraemia', 'Endocrine/Metabolic finding'),
('1017-VENOM', '1017', 'Hyperoestrogenism', 'Endocrine/Metabolic finding, Urogenital system finding, Reproductive system finding'),
('1018-VENOM', '1018', 'Hyperparathyroidism', 'Endocrine/Metabolic finding, Head and neck finding'),
('1019-VENOM', '1019', 'Hyperparathyroidism, primary', 'Endocrine/Metabolic finding, Head and neck finding'),
('1020-VENOM', '1020', 'Hyperparathyroidism, secondary - nutritional', 'Endocrine/Metabolic finding, Nutritional finding, Head and neck finding'),
('1021-VENOM', '1021', 'Hyperparathyroidism, secondary - renal', 'Endocrine/Metabolic finding, Urogenital system finding, Head and neck finding, Abdominal finding'),
('1023-VENOM', '1023', 'Hypersensitivity (allergic) skin disorder', 'Immune system finding, Haircoat/skin (+ appendages) finding'),
('1024-VENOM', '1024', 'Hypersensitivity (allergic) skin disorder - angioedema', 'Immune system finding, Haircoat/skin (+ appendages) finding'),
('1025-VENOM', '1025', 'Hypersensitivity (allergic) skin disorder - atopic dermatitis', 'Immune system finding, Haircoat/skin (+ appendages) finding, Hereditary disorder finding'),
('1026-VENOM', '1026', 'Hypersensitivity (allergic) skin disorder - contact hypersensitivity', 'Immune system finding, Haircoat/skin (+ appendages) finding'),
('1027-VENOM', '1027', 'Hypersensitivity (allergic) skin disorder - eosinophilic furunculosis of the face', 'Immune system finding, Haircoat/skin (+ appendages) finding, Inflammatory disorder finding, Head and neck finding'),
('1028-VENOM', '1028', 'Hypersensitivity (allergic) skin disorder - flea bite hypersensitivity', 'Immune system finding, Haircoat/skin (+ appendages) finding, Infectious disorder finding'),
('1029-VENOM', '1029', 'Hypersensitivity (allergic) skin disorder - food hypersensitivity', 'Immune system finding, Haircoat/skin (+ appendages) finding, Adverse reaction finding'),
('1030-VENOM', '1030', 'Hypersensitivity (allergic) skin disorder - indolent ulcer', 'Immune system finding, Haircoat/skin (+ appendages) finding'),
('1031-VENOM', '1031', 'Hypersensitivity (allergic) skin disorder - mast cell-rich papular dermatitis', 'Immune system finding, Haircoat/skin (+ appendages) finding, Inflammatory disorder finding, Haematopoietic system finding'),
('1032-VENOM', '1032', 'Hypersensitivity (allergic) skin disorder - mosquito bite hypersensitivity', 'Immune system finding, Haircoat/skin (+ appendages) finding, Infectious disorder finding'),
('1033-VENOM', '1033', 'Hypersensitivity (allergic) skin disorder - urticaria', 'Immune system finding, Haircoat/skin (+ appendages) finding'),
('1022-VENOM', '1022', 'Hypersensitivity disorder (allergy)', 'Immune system finding'),
('1034-VENOM', '1034', 'Hypertension - pulmonary', 'Respiratory system finding, Cardiovascular system finding, Thoracic finding'),
('1035-VENOM', '1035', 'Hypertension secondary to renal (kidney) disease', 'Cardiovascular system finding, Urogenital system finding, Abdominal finding'),
('1036-VENOM', '1036', 'Hypertensive disorder', 'Cardiovascular system finding'),
('1037-VENOM', '1037', 'Hyperthyroidism', 'Endocrine/Metabolic finding, Head and neck finding'),
('1038-VENOM', '1038', 'Hypertrophic osteopathy (HOA) (Marie''s disease)', 'Musculoskeletal system finding, Limb finding'),
('1039-VENOM', '1039', 'Hypertrophic osteosis of the calvarium', 'Musculoskeletal system finding, Head and neck finding'),
('1040-VENOM', '1040', 'Hyperviscosity syndrome', 'Clinical finding'),
('1041-VENOM', '1041', 'Hypoadrenocorticism (adrenocortical hypofunction)', 'Endocrine/Metabolic finding, Abdominal finding'),
('1042-VENOM', '1042', 'Hypoadrenocorticism (adrenocortical hypofunction) - iatrogenic', 'Endocrine/Metabolic finding, Abdominal finding'),
('1043-VENOM', '1043', 'Hypocalcaemia', 'Endocrine/Metabolic finding'),
('1044-VENOM', '1044', 'Hypocalcaemia - puerperal', 'Endocrine/Metabolic finding'),
('15589-VENOM', '15589', 'Hypochloraemia', 'Endocrine/Metabolic finding'),
('1045-VENOM', '1045', 'Hypoglycaemia', 'Endocrine/Metabolic finding'),
('1046-VENOM', '1046', 'Hypokalaemia', 'Endocrine/Metabolic finding'),
('1047-VENOM', '1047', 'Hypomagnesaemia', 'Endocrine/Metabolic finding'),
('1048-VENOM', '1048', 'Hyponatraemia', 'Endocrine/Metabolic finding'),
('1049-VENOM', '1049', 'Hypoparathyroidism', 'Endocrine/Metabolic finding, Head and neck finding'),
('1050-VENOM', '1050', 'Hypoparathyroidism - post-surgical', 'Endocrine/Metabolic finding, Head and neck finding, Complication of medical care finding'),
('1051-VENOM', '1051', 'Hypopituitarism', 'Endocrine/Metabolic finding, Nervous system finding, Head and neck finding'),
('1052-VENOM', '1052', 'Hypothyroidism', 'Endocrine/Metabolic finding, Head and neck finding'),
('1053-VENOM', '1053', 'Hypovolaemic shock', 'Cardiovascular system finding'),
('1054-VENOM', '1054', 'Ichthyosis', 'Haircoat/skin (+ appendages) finding'),
('1055-VENOM', '1055', 'Idiopathic facial dermatitis', 'Haircoat/skin (+ appendages) finding, Head and neck finding'),
('16407-VENOM', '16407', 'Idiopathic head tremor (head bob)', 'Nervous system finding, Head and neck finding'),
('1056-VENOM', '1056', 'Idiopathic lymphocytic mural folliculitis', 'Haircoat/skin (+ appendages) finding, Immune system finding'),
('7820-VENOM', '7820', 'Idiopathic renal haemorrhage', 'Urogenital system finding, Cardiovascular system finding, Abdominal finding'),
('1057-VENOM', '1057', 'Idiopathic seborrhoea', 'Haircoat/skin (+ appendages) finding'),
('1060-VENOM', '1060', 'Incomplete ossification of the humeral condyle (IOHC)', 'Musculoskeletal system finding, Developmental disorder finding, Limb finding'),
('1061-VENOM', '1061', 'Infectious canine tracheobronchitis (Kennel Cough)', 'Respiratory system finding, Infectious disorder finding, Inflammatory disorder finding, Head and neck finding, Thoracic finding'),
('1062-VENOM', '1062', 'Infectious myxomatosis of rabbits', ''),
('1063-VENOM', '1063', 'Inflammatory bowel disease (IBD)', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding, Abdominal finding'),
('16920-VENOM', '16920', 'Infundibular keratinising acanthoma', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('1064-VENOM', '1064', 'Insect bite(s)', 'Wound finding, Traumatic episode finding'),
('1065-VENOM', '1065', 'Insulin overdose', 'Complication of medical care finding, Drug-related disorder finding'),
('1066-VENOM', '1066', 'Insulinoma', 'Alimentary tract/Digestive system finding, Endocrine/Metabolic finding, Neuroendocrine system finding, Abdominal finding'),
('1067-VENOM', '1067', 'Interdigital cyst (dogs)', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding, Limb finding'),
('2128-VENOM', '2128', 'Interdigital dermatitis', 'Haircoat/skin (+ appendages) finding, Inflammatory disorder finding, Limb finding'),
('1068-VENOM', '1068', 'Interdigital pyogranulomas', 'Haircoat/skin (+ appendages) finding, Inflammatory disorder finding, Limb finding'),
('4022-VENOM', '4022', 'Intermittent claudication', 'Cardiovascular system finding. Limb finding'),
('1069-VENOM', '1069', 'Intersexuality', 'Urogenital system finding, Reproductive system finding'),
('17112-VENOM', '17112', 'Interstitial (Leydig) cell testicular tumour', 'Urogenital system finding, Reproductive system finding, Endocrine/Metabolic finding, Mass lesion/swelling finding'),
('11761-VENOM', '11761', 'Interstitial nephritis', 'Urogenital system finding, Inflammatory disorder finding, Abdominal finding'),
('1070-VENOM', '1070', 'Intertrigo', 'Haircoat/skin (+ appendages) finding, Infectious disorder finding'),
('2634-VENOM', '2634', 'Intertrigo - tail fold', 'Haircoat/skin (+ appendages) finding, Infectious disorder finding, Tail finding'),
('1073-VENOM', '1073', 'Intervertebral disc degeneration', 'Musculoskeletal system finding, Degenerative disorder finding, Spine (vertebral column) finding'),
('1074-VENOM', '1074', 'Intervertebral disc disorder', 'Musculoskeletal system finding, Spine (vertebral column) finding'),
('1075-VENOM', '1075', 'Intervertebral disc extrusion/herniation/prolapse', 'Musculoskeletal system finding, Spine (vertebral column) finding'),
('1076-VENOM', '1076', 'Intervertebral disc extrusion/herniation/prolapse - cervical', 'Musculoskeletal system finding, Spine (vertebral column) finding, Head and neck finding'),
('1077-VENOM', '1077', 'Intervertebral disc extrusion/herniation/prolapse - lumbar', 'Musculoskeletal system finding, Spine (vertebral column) finding, Abdominal finding'),
('1078-VENOM', '1078', 'Intervertebral disc extrusion/herniation/prolapse - Schmorl''s node', 'Musculoskeletal system finding, Spine (vertebral column) finding'),
('1079-VENOM', '1079', 'Intervertebral disc extrusion/herniation/prolapse - thoracic', 'Musculoskeletal system finding, Spine (vertebral column) finding, Thoracic finding'),
('1080-VENOM', '1080', 'Intervertebral disc rupture', 'Musculoskeletal system finding, Spine (vertebral column) finding'),
('1071-VENOM', '1071', 'Intervertebral discitis', 'Musculoskeletal system finding, Inflammatory disorder finding, Spine (vertebral column) finding'),
('1072-VENOM', '1072', 'Intervertebral discitis - infective', 'Musculoskeletal system finding, Inflammatory disorder finding, Infective disorder finding, Spine (vertebral column) finding'),
('1081-VENOM', '1081', 'Intestinal obstruction', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('1082-VENOM', '1082', 'Intestinal obstruction - large intestine', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('1083-VENOM', '1083', 'Intestinal obstruction - small intestine', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('1084-VENOM', '1084', 'Intestinal perforation', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('1085-VENOM', '1085', 'Intestinal perforation - large intestine', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('1086-VENOM', '1086', 'Intestinal perforation - small intestine', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('6916-VENOM', '6916', 'Intestinal ulceration', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding, Abdominal finding'),
('6917-VENOM', '6917', 'Intestinal ulceration - large intestine', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding, Abdominal finding'),
('1087-VENOM', '1087', 'Intestinal ulceration - small intestine', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding, Abdominal finding'),
('1090-VENOM', '1090', 'Intoxication (poisoning) - caustic agent', 'Intoxication (poisoning)/toxin exposure finding'),
('1091-VENOM', '1091', 'Intoxication (poisoning) - detergent', 'Intoxication (poisoning)/toxin exposure finding'),
('1092-VENOM', '1092', 'Intoxication (poisoning) - ethylene glycol', 'Intoxication (poisoning)/toxin exposure finding'),
('1093-VENOM', '1093', 'Intoxication (poisoning) - insecticide', 'Intoxication (poisoning)/toxin exposure finding'),
('4715-VENOM', '4715', 'Intoxication (poisoning) - metaldehyde', 'Intoxication (poisoning)/toxin exposure finding'),
('1094-VENOM', '1094', 'Intoxication (poisoning) - paraquat', 'Intoxication (poisoning)/toxin exposure finding'),
('1096-VENOM', '1096', 'Intoxication (poisoning) - rodenticide', 'Intoxication (poisoning)/toxin exposure finding'),
('1097-VENOM', '1097', 'Intoxication (poisoning) - theobromine/chocolate', 'Intoxication (poisoning)/toxin exposure finding'),
('5769-VENOM', '5769', 'Intoxication (poisoning) - vitamin D', 'Intoxication (poisoning)/toxin exposure finding'),
('15211-VENOM', '15211', 'Intoxication (poisoning)(unspecified)', 'Intoxication (poisoning)/toxin exposure finding'),
('15203-VENOM', '15203', 'Intoxication (poisoning), drug – accidental non-therapeutic drug overdose', 'Intoxication (poisoning)/toxin exposure finding, Drug-related disorder finding'),
('1099-VENOM', '1099', 'Intoxication (poisoning), drug - acetaminophen (paracetamol)', 'Intoxication (poisoning)/toxin exposure finding, Drug-related disorder finding'),
('15192-VENOM', '15192', 'Intoxication (poisoning), drug – anti-parasitic preparation', 'Intoxication (poisoning)/toxin exposure finding, Drug-related disorder finding'),
('15193-VENOM', '15193', 'Intoxication (poisoning), drug – digoxin', 'Intoxication (poisoning)/toxin exposure finding, Drug-related disorder finding'),
('15194-VENOM', '15194', 'Intoxication (poisoning), drug – non-steroidal anti-inflammatory drug (NSAID)(unspecified)', 'Intoxication (poisoning)/toxin exposure finding, Drug-related disorder finding'),
('15197-VENOM', '15197', 'Intoxication (poisoning), drug – NSAID – carprofen', 'Intoxication (poisoning)/toxin exposure finding, Drug-related disorder finding'),
('15199-VENOM', '15199', 'Intoxication (poisoning), drug – NSAID – ibuprofen', 'Intoxication (poisoning)/toxin exposure finding, Drug-related disorder finding'),
('15200-VENOM', '15200', 'Intoxication (poisoning), drug – NSAID – meloxicam', 'Intoxication (poisoning)/toxin exposure finding, Drug-related disorder finding'),
('1102-VENOM', '1102', 'Intoxication (poisoning), drug - phenobarbital', 'Intoxication (poisoning)/toxin exposure finding, Drug-related disorder finding, Complication of medical care finding'),
('15202-VENOM', '15202', 'Intoxication (poisoning), drug – accidental therapeutic drug overdose', 'Intoxication (poisoning)/toxin exposure finding, Drug-related disorder finding, Complication of medical care finding'),
('15207-VENOM', '15207', 'Intoxication (poisoning), drug – non-accidental drug overdose', 'Intoxication (poisoning)/toxin exposure finding, Drug-related disorder finding'),
('15190-VENOM', '15190', 'Intoxication (poisoning), drug (unspecified)', 'Intoxication (poisoning)/toxin exposure finding, Drug-related disorder finding'),
('15223-VENOM', '15223', 'Intoxication (poisoning), heavy metal – lead', 'Intoxication (poisoning)/toxin exposure finding'),
('15225-VENOM', '15225', 'Intoxication (poisoning), heavy metal – zinc', 'Intoxication (poisoning)/toxin exposure finding'),
('15220-VENOM', '15220', 'Intoxication (poisoning), heavy metal (unspecified)', 'Intoxication (poisoning)/toxin exposure finding'),
('15216-VENOM', '15216', 'Intoxication (poisoning), plant – Allium genus – onion', 'Intoxication (poisoning)/toxin exposure finding'),
('15213-VENOM', '15213', 'Intoxication (poisoning), plant – Allium genus (unspecified)', 'Intoxication (poisoning)/toxin exposure finding'),
('15218-VENOM', '15218', 'Intoxication (poisoning), plant – grape/raisin', 'Intoxication (poisoning)/toxin exposure finding'),
('15219-VENOM', '15219', 'Intoxication (poisoning), plant – lily', 'Intoxication (poisoning)/toxin exposure finding'),
('15209-VENOM', '15209', 'Intoxication (poisoning), plant (unspecified)', 'Intoxication (poisoning)/toxin exposure finding'),
('1103-VENOM', '1103', 'Intussusception, gastroesophageal', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('1104-VENOM', '1104', 'Intussusception, intestinal', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('1105-VENOM', '1105', 'Intussusception, intestinal - ileocaecal', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('1106-VENOM', '1106', 'Intussusception, intestinal - ileocolic', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('1107-VENOM', '1107', 'Intussusception, intestinal - large intestine', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('1108-VENOM', '1108', 'Intussusception, intestinal - small intestine', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('11769-VENOM', '11769', 'IRIS azotaemia Stage I (<125 µmol/L dogs, <140 µmol/L cats)', 'Urogenital system finding, Endocrine/metabolic finding, Abdominal finding'),
('11771-VENOM', '11771', 'IRIS azotaemia Stage II (125-180 µmol/L dogs, 140-250 µmol/L cats)', 'Urogenital system finding, Endocrine/metabolic finding, Abdominal finding'),
('11773-VENOM', '11773', 'IRIS azotaemia Stage III (180-400 µmol/L dogs, 250-440 µmol/L cats)', 'Urogenital system finding, Endocrine/metabolic finding, Abdominal finding'),
('11775-VENOM', '11775', 'IRIS azotaemia Stage IV (>440 µmol/L dogs & cats)', 'Urogenital system finding, Endocrine/metabolic finding, Abdominal finding'),
('11793-VENOM', '11793', 'IRIS blood pressure – end organ damage (cardiac) – hypertension', 'Urogenital system finding, Cardiovascular system finding, Abdominal finding'),
('11791-VENOM', '11791', 'IRIS blood pressure – end organ damage (ocular) – hypertension', 'Urogenital system finding, Cardiovascular system finding, Abdominal finding'),
('11795-VENOM', '11795', 'IRIS blood pressure – end organ damage (other) – hypertension', 'Urogenital system finding, Cardiovascular system finding, Abdominal finding'),
('11785-VENOM', '11785', 'IRIS blood pressure – Low Risk (SBP 150-159, DBP 95-99 mmHg)', 'Urogenital system finding, Cardiovascular system finding, Abdominal finding'),
('11783-VENOM', '11783', 'IRIS blood pressure – Minimal Risk (SBP <150, DBP <95 mmHg)', 'Urogenital system finding, Cardiovascular system finding, Abdominal finding'),
('11787-VENOM', '11787', 'IRIS blood pressure – Moderate Risk (SBP 160-179, DBP 100-119 mmHg)', 'Urogenital system finding, Cardiovascular system finding, Abdominal finding'),
('11789-VENOM', '11789', 'IRIS blood pressure – Severe Risk (SBP >180, DBP >120 mmHg)', 'Urogenital system finding, Cardiovascular system finding, Abdominal finding'),
('11779-VENOM', '11779', 'IRIS borderline proteinuric (0.2-0.5 dogs, 0.2-0.4 cats)', 'Urogenital system finding, Abdominal finding'),
('18738-VENOM', '18738', 'Iris coloboma - Atypical', 'Ophthalmology/vision finding, Congenital finding, Head and neck finding'),
('18739-VENOM', '18739', 'Iris coloboma - Typical', 'Ophthalmology/vision finding, Congenital finding, Head and neck finding'),
('1109-VENOM', '1109', 'Iris cyst', 'Ophthalmology/vision finding, Mass lesion/swelling finding, Head and neck finding'),
('11777-VENOM', '11777', 'IRIS non-proteinuric (<0.2 dogs & cats)', 'Urogenital system finding, Abdominal finding'),
('1110-VENOM', '1110', 'Iris prolapse', 'Ophthalmology/vision finding, Head and neck finding'),
('11781-VENOM', '11781', 'IRIS proteinuric (>0.5 dogs, >0.4 cats)', 'Urogenital system finding, Abdominal finding'),
('2129-VENOM', '2129', 'Iritis', 'Ophthalmology/vision finding, Inflammatory disorder finding, Head and neck finding'),
('1111-VENOM', '1111', 'Iron deficiency anaemia', 'Haematopoietic system finding, Nutritional finding'),
('16410-VENOM', '16410', 'Ischaemic dermatopathy syndrome - dermatomyositis', 'Haircoat/skin (+ appendages) finding, Immune system finding, Inflammatory disorder finding, Musculoskeletal system finding'),
('15827-VENOM', '15827', 'Ischaemic dermatopathy syndrome (type unspecified)', 'Haircoat/skin (+ appendages) finding, Immune system finding'),
('1138-VENOM', '1138', 'Joint instability', 'Musculoskeletal system finding'),
('1139-VENOM', '1139', 'Joint instability - carpal', 'Musculoskeletal system finding, Limb finding'),
('1140-VENOM', '1140', 'Joint instability - hock', 'Musculoskeletal system finding, Limb finding'),
('1141-VENOM', '1141', 'Joint instability - sacroiliac', 'Musculoskeletal system finding, Spine (vertebral column) finding, Pelvic structure finding'),
('1142-VENOM', '1142', 'Joint instability - shoulder', 'Musculoskeletal system finding, Limb finding'),
('1112-VENOM', '1112', 'Joint luxation', 'Musculoskeletal system finding'),
('1113-VENOM', '1113', 'Joint luxation - carpus', 'Musculoskeletal system finding, Limb finding'),
('1114-VENOM', '1114', 'Joint luxation - elbow', 'Musculoskeletal system finding, Limb finding'),
('1115-VENOM', '1115', 'Joint luxation - hip', 'Musculoskeletal system finding, Limb finding'),
('1116-VENOM', '1116', 'Joint luxation - hock', 'Musculoskeletal system finding, Limb finding'),
('1117-VENOM', '1117', 'Joint luxation - sacroiliac', 'Musculoskeletal system finding, Spine (vertebral column) finding, Pelvic structure finding'),
('1118-VENOM', '1118', 'Joint luxation - shoulder', 'Musculoskeletal system finding, Limb finding'),
('1119-VENOM', '1119', 'Joint luxation - tarsometatarsal', 'Musculoskeletal system finding, Limb finding'),
('1120-VENOM', '1120', 'Joint luxation - temporomandibular (TMJ)', 'Musculoskeletal system finding, Head and neck finding'),
('1121-VENOM', '1121', 'Joint luxation - vertebral', 'Musculoskeletal system finding, Spine (vertebral column) finding'),
('1122-VENOM', '1122', 'Joint luxation - vertebral, sacrococcygeal', 'Musculoskeletal system finding, Spine (vertebral column) finding, Tail finding'),
('1123-VENOM', '1123', 'Joint luxation, acquired', 'Musculoskeletal system finding'),
('1124-VENOM', '1124', 'Joint luxation, congenital', 'Musculoskeletal system finding, Congenital finding'),
('1125-VENOM', '1125', 'Joint subluxation', 'Musculoskeletal system finding'),
('1126-VENOM', '1126', 'Joint subluxation - carpus', 'Musculoskeletal system finding, Limb finding'),
('1127-VENOM', '1127', 'Joint subluxation - elbow', 'Musculoskeletal system finding, Limb finding'),
('1128-VENOM', '1128', 'Joint subluxation - hip', 'Musculoskeletal system finding, Limb finding'),
('1129-VENOM', '1129', 'Joint subluxation - hock', 'Musculoskeletal system finding, Limb finding'),
('1130-VENOM', '1130', 'Joint subluxation - sacroiliac', 'Musculoskeletal system finding, Spine (vertebral column) finding, Pelvic structure finding'),
('1131-VENOM', '1131', 'Joint subluxation - shoulder', 'Musculoskeletal system finding, Limb finding'),
('1132-VENOM', '1132', 'Joint subluxation - temporomandibular (TMJ)', 'Musculoskeletal system finding, Head and neck finding'),
('1133-VENOM', '1133', 'Joint subluxation - vertebral', 'Musculoskeletal system finding, Spine (vertebral column) finding'),
('1136-VENOM', '1136', 'Joint subluxation, acquired', 'Musculoskeletal system finding'),
('1137-VENOM', '1137', 'Joint subluxation, congenital', 'Musculoskeletal system finding, Congenital finding'),
('1134-VENOM', '1134', 'Joint subluxation, vertebral - Atlantoaxial subluxation ', 'Musculoskeletal system finding, Spine (vertebral column) finding, Head and neck finding'),
('1135-VENOM', '1135', 'Joint subluxation, vertebral - Atlanto-occipital subluxation', 'Musculoskeletal system finding, Spine (vertebral column) finding, Head and neck finding'),
('15553-VENOM', '15553', 'Junctionopathy (unspecified)', 'Nervous system finding, Musculoskeletal system finding'),
('1143-VENOM', '1143', 'Juvenile cellulitis/pyoderma', 'Haircoat/skin (+ appendages) finding, Inflammatory disorder finding, Immune system finding, Infectious disorder finding, Soft tissue finding'),
('1144-VENOM', '1144', 'Keratitis', 'Ophthalmology/vision finding, Inflammatory disorder finding, Head and neck finding'),
('1145-VENOM', '1145', 'Keratitis - chronic superficial (pannus)', 'Ophthalmology/vision finding, Inflammatory disorder finding, Head and neck finding'),
('1146-VENOM', '1146', 'Keratitis - eosinophilic', 'Ophthalmology/vision finding, Inflammatory disorder finding, Head and neck finding'),
('1147-VENOM', '1147', 'Keratitis - infectious', 'Ophthalmology/vision finding, Inflammatory disorder finding, Infectious disorder finding, Head and neck finding'),
('1148-VENOM', '1148', 'Keratitis - superficial', 'Ophthalmology/vision finding, Inflammatory disorder finding, Head and neck finding'),
('1149-VENOM', '1149', 'Keratitis - traumatic', 'Ophthalmology/vision finding, Inflammatory disorder finding, Traumatic episode finding, Head and neck finding'),
('1150-VENOM', '1150', 'Keratoconjunctivitis sicca (KCS, Dry Eye)', 'Ophthalmology/vision finding, Inflammatory disorder finding, Immune system finding, Soft tissue finding, Head and neck finding'),
('9573-VENOM', '9573', 'L-2-Hydroxyglutaric aciduria (L-2-HGA)', 'Endocrine/metabolic finding, Nervous system finding, Hereditary disorder finding'),
('1151-VENOM', '1151', 'Laryngeal collapse', 'Respiratory system finding, Head and neck finding'),
('1152-VENOM', '1152', 'Laryngeal obstruction', 'Respiratory system finding, Head and neck finding'),
('1153-VENOM', '1153', 'Laryngeal paralysis', 'Respiratory system finding, Head and neck finding'),
('1154-VENOM', '1154', 'Laryngeal paralysis - hemiplegia', 'Respiratory system finding, Head and neck finding'),
('1155-VENOM', '1155', 'Laryngeal paresis', 'Respiratory system finding, Head and neck finding'),
('1156-VENOM', '1156', 'Laryngitis', 'Respiratory system finding, Inflammatory disorder finding, Head and neck finding'),
('1157-VENOM', '1157', 'Laryngitis - infectious', 'Respiratory system finding, Inflammatory disorder finding, Infectious disorder finding, Head and neck finding'),
('17118-VENOM', '17118', 'Leiomyoma - large intestine (site unspecified)', 'Musculoskeletal system finding, Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding'),
('17120-VENOM', '17120', 'Leiomyoma - oesophageal/gastric (stomach) junction', 'Musculoskeletal system finding, Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding'),
('17124-VENOM', '17124', 'Leiomyoma - oesophagus', 'Musculoskeletal system finding, Alimentary tract/Digestive system finding, Mass lesion/swelling finding'),
('17126-VENOM', '17126', 'Leiomyoma - rectum', 'Musculoskeletal system finding, Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding, Pelvic structure finding'),
('17128-VENOM', '17128', 'Leiomyoma - small intestine', 'Musculoskeletal system finding, Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding'),
('17125-VENOM', '17125', 'Leiomyoma - stomach (gastric)', 'Musculoskeletal system finding, Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding'),
('17130-VENOM', '17130', 'Leiomyoma - trachea', 'Musculoskeletal system finding, Respiratory system finding, Mass lesion/swelling, Head and neck finding'),
('17132-VENOM', '17132', 'Leiomyoma - urinary bladder', 'Musculoskeletal system finding, Urogenital system finding, Mass lesion/swelling finding, Abdominal finding'),
('17134-VENOM', '17134', 'Leiomyoma - uterus', 'Musculoskeletal system finding, Reproductive system finding, Mass lesion/swelling finding, Abdominal finding'),
('17136-VENOM', '17136', 'Leiomyoma - vagina', 'Musculoskeletal system finding, Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding, Pelvic structure finding'),
('16426-VENOM', '16426', 'Leiomyoma (site unspecified)', 'Musculoskeletal system finding, Mass lesion/swelling finding'),
('17142-VENOM', '17142', 'Leiomyosarcoma - ileocaecal junction', 'Musculoskeletal system finding, Connective tissue finding, Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding'),
('17145-VENOM', '17145', 'Leiomyosarcoma - large intestine (site unspecified)', 'Musculoskeletal system finding, Connective tissue finding, Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding'),
('17147-VENOM', '17147', 'Leiomyosarcoma - rectum', 'Musculoskeletal system finding, Connective tissue finding, Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding, Pelvic structure finding'),
('17149-VENOM', '17149', 'Leiomyosarcoma - small intestine', 'Musculoskeletal system finding, Connective tissue finding, Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding'),
('17150-VENOM', '17150', 'Leiomyosarcoma - stomach (gastric)', 'Musculoskeletal system finding, Connective tissue finding, Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding'),
('17152-VENOM', '17152', 'Leiomyosarcoma - trachea', 'Musculoskeletal system finding, Connective tissue finding, Respiratory system finding, Mass lesion/swelling, Head and neck finding'),
('17154-VENOM', '17154', 'Leiomyosarcoma - urinary bladder', 'Musculoskeletal system finding, Connective tissue finding, Urogenital system finding, Mass lesion/swelling finding, Abdominal finding'),
('17156-VENOM', '17156', 'Leiomyosarcoma - uterus', 'Musculoskeletal system finding, Connective tissue finding, Reproductive system finding, Mass lesion/swelling finding, Abdominal finding'),
('17158-VENOM', '17158', 'Leiomyosarcoma - vagina', 'Musculoskeletal system finding, Connective tissue finding, Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding, Pelvic structure finding'),
('16428-VENOM', '16428', 'Leiomyosarcoma (site unspecified)', 'Musculoskeletal system finding, Connective tissue finding, Mass lesion/swelling finding'),
('18743-VENOM', '18743', 'Lens "coloboma" (zonular anomaly) - Atypical', 'Ophthalmology/vision finding, Congenital finding, Head and neck finding'),
('18744-VENOM', '18744', 'Lens "coloboma" (zonular anomaly) - Typical', 'Ophthalmology/vision finding, Congenital finding, Head and neck finding'),
('1160-VENOM', '1160', 'Lens injury', 'Ophthalmology/vision finding, Traumatic episode finding, Head and neck finding'),
('1161-VENOM', '1161', 'Lens luxation', 'Ophthalmology/vision finding, Head and neck finding'),
('1162-VENOM', '1162', 'Lens luxation - anterior', 'Ophthalmology/vision finding, Head and neck finding'),
('1163-VENOM', '1163', 'Lens luxation - posterior', 'Ophthalmology/vision finding, Head and neck finding'),
('1164-VENOM', '1164', 'Lens subluxation', 'Ophthalmology/vision finding, Head and neck finding'),
('1165-VENOM', '1165', 'Leptospirosis', 'Infectious disorder finding'),
('1166-VENOM', '1166', 'Leptospirosis, canine', 'Infectious disorder finding'),
('1167-VENOM', '1167', 'Lethal acrodermatitis', 'Haircoat/skin (+ appendages) finding, Inflammatory disorder finding, Endocrine/metabolic finding'),
('17160-VENOM', '17160', 'Leukaemia - Acute erythroid', 'Immune system finding, Haematopoietic system finding, Mass lesion/swelling finding'),
('16829-VENOM', '16829', 'Leukaemia - Acute leukaemia, No origin specified (NOS)', 'Immune system finding, Haematopoietic system finding, Mass lesion/swelling finding'),
('1170-VENOM', '1170', 'Leukaemia - Acute lymphoblastic leukaemia', 'Immune system finding, Haematopoietic system finding, Mass lesion/swelling finding'),
('16831-VENOM', '16831', 'Leukaemia - Acute myeloid leukaemia (subtype unspecified)', 'Immune system finding, Haematopoietic system finding, Mass lesion/swelling finding'),
('16833-VENOM', '16833', 'Leukaemia - Acute myeloid leukaemia, monocytic', 'Immune system finding, Haematopoietic system finding, Mass lesion/swelling finding'),
('16834-VENOM', '16834', 'Leukaemia - Acute myeloid leukaemia, myelomonocytic', 'Immune system finding, Haematopoietic system finding, Mass lesion/swelling finding'),
('17162-VENOM', '17162', 'Leukaemia - basophilic', 'Immune system finding, Haematopoietic system finding, Mass lesion/swelling finding'),
('16838-VENOM', '16838', 'Leukaemia - Chronic leukaemia, No origin specified (NOS)', 'Immune system finding, Haematopoietic system finding, Mass lesion/swelling finding'),
('1173-VENOM', '1173', 'Leukaemia - Chronic lymphoid leukaemia', 'Immune system finding, Haematopoietic system finding, Mass lesion/swelling finding'),
('1174-VENOM', '1174', 'Leukaemia - Chronic myeloid leukaemia', 'Immune system finding, Haematopoietic system finding, Mass lesion/swelling finding'),
('17164-VENOM', '17164', 'Leukaemia - eosinophilic', 'Immune system finding, Haematopoietic system finding, Mass lesion/swelling finding'),
('17166-VENOM', '17166', 'Leukaemia - Essential thrombocytosis', 'Immune system finding, Haematopoietic system finding, Haemostatic system finding, Mass lesion/swelling finding'),
('16839-VENOM', '16839', 'Leukaemia - Polycythaemia rubra vera (Primary polycythaemia)', 'Immune system finding, Haematopoietic system finding, Mass lesion/swelling finding'),
('16430-VENOM', '16430', 'Leukaemia (type unspecified)', 'Immune system finding, Haematopoietic system finding, Mass lesion/swelling finding'),
('1175-VENOM', '1175', 'Lichenoid dermatosis', 'Haircoat/skin (+ appendages) finding'),
('1176-VENOM', '1176', 'Ligament injury', '#NAME?'),
('1177-VENOM', '1177', 'Ligament instability', '#NAME?'),
('1178-VENOM', '1178', 'Ligament rupture', '#NAME?'),
('1179-VENOM', '1179', 'Limb deformity, acquired', 'Musculoskeletal system finding, Limb finding'),
('1180-VENOM', '1180', 'Limb deformity, congenital', 'Musculoskeletal system finding, Congenital finding, Limb finding'),
('1182-VENOM', '1182', 'Limb deformity, developmental - carpal valgus', 'Musculoskeletal system finding, Developmental disorder finding, Limb finding'),
('1181-VENOM', '1181', 'Limb deformity, developmental - carpal varus', 'Musculoskeletal system finding, Developmental disorder finding, Limb finding'),
('1184-VENOM', '1184', 'Limb deformity, developmental - genu valgum', 'Musculoskeletal system finding, Developmental disorder finding, Limb finding'),
('1183-VENOM', '1183', 'Limb deformity, developmental - genu varum', 'Musculoskeletal system finding, Developmental disorder finding, Limb finding'),
('1185-VENOM', '1185', 'Limb deformity, developmental - pes varus', 'Musculoskeletal system finding, Developmental disorder finding, Limb finding'),
('17168-VENOM', '17168', 'Lipoma - abdominal wall', 'Soft tissue finding, Mass lesion/swelling finding, Abdominal finding'),
('17179-VENOM', '17179', 'Lipoma - infiltrative', 'Soft tissue finding, Mass lesion/swelling finding'),
('17181-VENOM', '17181', 'Lipoma - intermuscular', 'Soft tissue finding, Mass lesion/swelling finding'),
('17175-VENOM', '17175', 'Lipoma - limb, upper', 'Soft tissue finding, Mass lesion/swelling finding, Limb finding'),
('17177-VENOM', '17177', 'Lipoma - thoracic (chest) wall', 'Soft tissue finding, Mass lesion/swelling finding, Thoracic finding'),
('16433-VENOM', '16433', 'Lipoma (site unspecified)', 'Soft tissue finding, Mass lesion/swelling finding'),
('16434-VENOM', '16434', 'Lipoma (type unspecified)', 'Soft tissue finding, Mass lesion/swelling finding'),
('1187-VENOM', '1187', 'Liposarcoma', 'Soft tissue finding, Mass lesion/swelling finding'),
('1188-VENOM', '1188', 'Listeriosis', 'Infectious disorder finding'),
('1189-VENOM', '1189', 'Lower motor neuron disease', 'Nervous system finding'),
('1190-VENOM', '1190', 'Lung (pulmonary) disorder', 'Respiratory system finding, Thoracic finding'),
('1191-VENOM', '1191', 'Lung lobe torsion', 'Respiratory system finding, Thoracic finding'),
('17183-VENOM', '17183', 'Luteoma', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding, Abdominal finding'),
('1192-VENOM', '1192', 'Lyme disease (Borrelia burgdorferi)', 'Infectious disorder finding'),
('1193-VENOM', '1193', 'Lymphadenitis', 'Haematopoietic system finding, Immune system finding, Inflammatory disorder finding'),
('16918-VENOM', '16918', 'Lymphadenopathy, generalised - aetiology unknown', 'Lymphatic system finding, Haematopoietic system finding, Immune system finding'),
('16919-VENOM', '16919', 'Lymphadenopathy, localised - aetiology unknown', 'Lymphatic system finding, Haematopoietic system finding, Immune system finding'),
('1200-VENOM', '1200', 'Lymphangiectasia', 'Lymphatic system finding'),
('1201-VENOM', '1201', 'Lymphangioma', 'Lymphatic system finding, Mass lesion/swelling finding'),
('1202-VENOM', '1202', 'Lymphangiosarcoma', 'Lymphatic system finding, Connective tissue finding, Mass lesion/swelling finding'),
('1203-VENOM', '1203', 'Lymphoedema', 'Lymphatic system finding'),
('17194-VENOM', '17194', 'Lymphoma - bone', 'Haematopoetic system finding, Immune system finding, Lymphatic system finding, Musculoskeletal system finding, Mass lesion/swelling finding'),
('17196-VENOM', '17196', 'Lymphoma - cardiac (heart)', 'Haematopoetic system finding, Immune system finding, Lymphatic system finding, Cardiovascular system finding, Mass lesion/swelling finding, Thoracic finding'),
('1206-VENOM', '1206', 'Lymphoma - central nervous system (CNS)', 'Nervous system finding, Haematopoetic system finding, Immune system finding, Lymphatic system finding, Mass lesion/swelling finding'),
('17198-VENOM', '17198', 'Lymphoma - cranial mediastinal/thymic', 'Haematopoetic system finding, Immune system finding, Lymphatic system finding, Endocrine/Metabolic finding, Mass lesion/swelling finding, Thoracic finding'),
('16910-VENOM', '16910', 'Lymphoma - cutaneous (skin), primary', 'Haircoat/skin (+ appendages) finding, Haematopoetic system finding, Immune system finding, Lymphatic system finding, Mass lesion/swelling finding'),
('16911-VENOM', '16911', 'Lymphoma - cutaneous (skin), primary - epitheliotropic', 'Haircoat/skin (+ appendages) finding, Haematopoetic system finding, Immune system finding, Lymphatic system finding, Mass lesion/swelling finding'),
('16913-VENOM', '16913', 'Lymphoma - cutaneous (skin), secondary - B-cell', 'Haircoat/skin (+ appendages) finding, Haematopoetic system finding, Immune system finding, Lymphatic system finding, Mass lesion/swelling finding'),
('16915-VENOM', '16915', 'Lymphoma - cutaneous (skin), secondary - immunophenotype unknown', 'Haircoat/skin (+ appendages) finding, Haematopoetic system finding, Immune system finding, Lymphatic system finding, Mass lesion/swelling finding'),
('16909-VENOM', '16909', 'Lymphoma - cutaneous (skin), secondary - T-cell', 'Haircoat/skin (+ appendages) finding, Haematopoetic system finding, Immune system finding, Lymphatic system finding, Mass lesion/swelling finding'),
('1208-VENOM', '1208', 'Lymphoma - gastric (stomach)', 'Alimentary tract/Digestive system finding, Haematopoetic system finding, Immune system finding, Lymphatic system finding, Mass lesion/swelling finding, Abdominal finding'),
('17200-VENOM', '17200', 'Lymphoma - hepatic (liver)', 'Haematopoetic system finding, Immune system finding, Lymphatic system finding, Hepatobiliary system finding, Mass lesion/swelling finding, Abdominal finding'),
('1209-VENOM', '1209', 'Lymphoma - intestinal', 'Alimentary tract/Digestive system finding, Haematopoetic system finding, Immune system finding, Lymphatic system finding, Mass lesion/swelling finding, Abdominal finding'),
('17202-VENOM', '17202', 'Lymphoma - larynx', 'Haematopoetic system finding, Immune system finding, Lymphatic system finding, Respiratory system finding, Mass lesion/swelling finding, Head and neck finding'),
('1210-VENOM', '1210', 'Lymphoma - lymph node(s)', 'Haematopoetic system finding, Immune system finding, Lymphatic system finding, Mass lesion/swelling finding'),
('17188-VENOM', '17188', 'Lymphoma - multicentric - B-cell', 'Haematopoetic system finding, Immune system finding, Lymphatic system finding, Mass lesion/swelling finding'),
('17190-VENOM', '17190', 'Lymphoma - multicentric - immunophenotype unknown', 'Haematopoetic system finding, Immune system finding, Lymphatic system finding, Mass lesion/swelling finding'),
('16899-VENOM', '16899', 'Lymphoma - multicentric - stage I', 'Haematopoetic system finding, Immune system finding, Lymphatic system finding, Mass lesion/swelling finding'),
('16900-VENOM', '16900', 'Lymphoma - multicentric - stage II', 'Haematopoetic system finding, Immune system finding, Lymphatic system finding, Mass lesion/swelling finding'),
('16901-VENOM', '16901', 'Lymphoma - multicentric - stage III', 'Haematopoetic system finding, Immune system finding, Lymphatic system finding, Mass lesion/swelling finding'),
('16902-VENOM', '16902', 'Lymphoma - multicentric - stage IV', 'Haematopoetic system finding, Immune system finding, Lymphatic system finding, Mass lesion/swelling finding'),
('16903-VENOM', '16903', 'Lymphoma - multicentric - stage V', 'Haematopoetic system finding, Immune system finding, Lymphatic system finding, Mass lesion/swelling finding'),
('17189-VENOM', '17189', 'Lymphoma - multicentric - T-cell', 'Haematopoetic system finding, Immune system finding, Lymphatic system finding, Mass lesion/swelling finding'),
('17185-VENOM', '17185', 'Lymphoma - multicentric (stage unspecified)', 'Haematopoetic system finding, Immune system finding, Lymphatic system finding, Mass lesion/swelling finding'),
('1211-VENOM', '1211', 'Lymphoma - ocular (eye)', 'Ophthalmology/vision finding, Haematopoetic system finding, Immune system finding, Lymphatic system finding, Mass lesion/swelling finding, Head and neck finding'),
('1212-VENOM', '1212', 'Lymphoma - orbital', 'Ophthalmology/vision finding, Haematopoetic system finding, Immune system finding, Lymphatic system finding, Musculoskeletal system finding, Mass lesion/swelling finding, Head and neck finding'),
('17204-VENOM', '17204', 'Lymphoma - rectum', 'Alimentary tract/Digestive system finding, Haematopoetic system finding, Immune system finding, Lymphatic system finding, Mass lesion/swelling finding, Abdominal finding, Pelvic structure finding'),
('1213-VENOM', '1213', 'Lymphoma - renal (kidney)', 'Urogenital system finding, Haematopoetic system finding, Immune system finding, Lymphatic system finding, Mass lesion/swelling finding, Abdominal finding'),
('1214-VENOM', '1214', 'Lymphoma - splenic', 'Haematopoetic system finding, Immune system finding, Lymphatic system finding, Mass lesion/swelling finding, Abdominal finding'),
('17206-VENOM', '17206', 'Lymphoma (site unspecified)', 'Haematopoetic system finding, Immune system finding, Lymphatic system finding, Mass lesion/swelling finding'),
('1221-VENOM', '1221', 'Lysosomal enzyme disorder', 'Endocrine/metabolic finding, Hereditary disorder finding, Congenital finding'),
('1222-VENOM', '1222', 'Macroblepharon/macropalpebral fissure', 'Ophthalmology/vison finding, Congenital finding, Head and neck finding'),
('1223-VENOM', '1223', 'Malabsorption, intestinal', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('1224-VENOM', '1224', 'Malassezia dermatitis', 'Haircoat/skin (+ appendages) finding, Inflammatory disorder finding, Infectious disorder finding'),
('1225-VENOM', '1225', 'Malignant hyperthermia', 'Body temperature finding, Musculoskeletal system finding, Hereditary disorder finding'),
('1227-VENOM', '1227', 'Mass lesion - abdominal', 'Abdominal finding, Mass lesion/swelling finding'),
('1228-VENOM', '1228', 'Mass lesion - adrenal', 'Endocrine/Metabolic finding, Abdominal finding, Mass lesion/swelling finding'),
('1229-VENOM', '1229', 'Mass lesion - anal', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding'),
('1230-VENOM', '1230', 'Mass lesion - anal gland/sac', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding'),
('1232-VENOM', '1232', 'Mass lesion - axilla', 'Mass lesion/swelling finding, Thoracic finding'),
('1233-VENOM', '1233', 'Mass lesion - bile duct', 'Hepatobiliary system finding, Mass lesion/swelling finding, Abdominal finding'),
('1234-VENOM', '1234', 'Mass lesion - bone', 'Musculoskeletal system finding, Mass lesion/swelling finding'),
('16967-VENOM', '16967', 'Mass lesion - bony pelvis', 'Musculoskeletal system finding, Mass lesion/swelling finding, Pelvic structure finding'),
('1235-VENOM', '1235', 'Mass lesion - brain', 'Nervous system finding, Head and neck finding, Mass lesion/swelling finding'),
('16353-VENOM', '16353', 'Mass lesion - calvarium (bone unspecified)', 'Musculoskeletal system finding, Mass lesion/swelling finding, Head and neck finding'),
('1236-VENOM', '1236', 'Mass lesion - cardiac (heart)', 'Cardiovascular system finding, Thoracic finding, Mass lesion/swelling finding'),
('1238-VENOM', '1238', 'Mass lesion - conjunctival', 'Ophthalmology/vision finding, Soft tissue finding, Mass lesion/swelling finding, Head and neck finding'),
('3911-VENOM', '3911', 'Mass lesion - ear (aural)', 'Auditory system finding, Mass lesion/swelling finding, Head and neck finding'),
('1239-VENOM', '1239', 'Mass lesion - eyelid', 'Ophthalmology/vision finding, Mass lesion/swelling finding, Head and neck finding'),
('1240-VENOM', '1240', 'Mass lesion - facial', 'Mass lesion/swelling finding, Head and neck finding'),
('1241-VENOM', '1241', 'Mass lesion - facial, lip', 'Alimentary tract/Digestive system finding, Soft tissue finding, Mass lesion/swelling finding, Head and neck finding'),
('1242-VENOM', '1242', 'Mass lesion - gallbladder', 'Hepatobiliary system finding, Mass lesion/swelling finding, Abdominal finding'),
('1243-VENOM', '1243', 'Mass lesion - gastric (stomach)', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding'),
('4665-VENOM', '4665', 'Mass lesion - gingival (gum)', 'Alimentary tract/Digestive system finding, Soft tissue finding, Mass lesion/swelling finding, Head and neck finding'),
('1244-VENOM', '1244', 'Mass lesion - heart base', 'Cardiovascular system finding, Thoracic finding, Mass lesion/swelling finding'),
('1245-VENOM', '1245', 'Mass lesion - hepatic (liver)', 'Hepatobiliary system finding, Mass lesion/swelling finding, Abdominal finding'),
('7511-VENOM', '7511', 'Mass lesion - injection site reaction', 'Complication of medical care finding, Mass lesion/swelling finding'),
('1246-VENOM', '1246', 'Mass lesion - intestinal', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding'),
('1247-VENOM', '1247', 'Mass lesion - intra-ocular', 'Ophthalmology/vision finding, Mass lesion/swelling finding, Head and neck finding'),
('1248-VENOM', '1248', 'Mass lesion - iris', 'Ophthalmology/vision finding, Mass lesion/swelling finding, Head and neck finding'),
('16438-VENOM', '16438', 'Mass lesion - jaw (site unspecified)', 'Musculoskeletal system finding, Mass lesion/swelling finding, Head and neck finding'),
('1250-VENOM', '1250', 'Mass lesion - jaw, mandible', 'Musculoskeletal system finding, Mass lesion/swelling finding, Head and neck finding'),
('1251-VENOM', '1251', 'Mass lesion - jaw, maxilla', 'Musculoskeletal system finding, Mass lesion/swelling finding, Head and neck finding'),
('1252-VENOM', '1252', 'Mass lesion - joint', 'Musculoskeletal system finding, Mass lesion/swelling finding'),
('1253-VENOM', '1253', 'Mass lesion - laryngeal', 'Respiratory system finding, Mass lesion/swelling finding, Head and neck finding'),
('1254-VENOM', '1254', 'Mass lesion - limb, lower', 'Limb finding, Mass lesion/swelling finding'),
('1255-VENOM', '1255', 'Mass lesion - limb, upper', 'Limb finding, Mass lesion/swelling finding'),
('1256-VENOM', '1256', 'Mass lesion - lymph node', 'Lymphatic system finding, Immune system finding, Haematopoietic system finding, Mass lesion/swelling finding'),
('1257-VENOM', '1257', 'Mass lesion - mammary', 'Reproductive system finding, Mass lesion/swelling finding'),
('1259-VENOM', '1259', 'Mass lesion - mediastinal', 'Thoracic finding, Mass lesion/swelling finding'),
('1260-VENOM', '1260', 'Mass lesion - mediastinal, anterior', 'Thoracic finding, Mass lesion/swelling finding'),
('1261-VENOM', '1261', 'Mass lesion - meningeal', 'Nervous system finding, Mass lesion/swelling finding'),
('1262-VENOM', '1262', 'Mass lesion - muscle', 'Musculoskeletal system finding, Mass lesion/swelling finding'),
('1263-VENOM', '1263', 'Mass lesion - nasal sinus', 'Respiratory system finding, Mass lesion/swelling finding, Head and neck finding'),
('1264-VENOM', '1264', 'Mass lesion - nasal sinus, frontal sinus', 'Respiratory system finding, Mass lesion/swelling finding, Head and neck finding'),
('1265-VENOM', '1265', 'Mass lesion - neck (cervical)', 'Mass lesion/swelling finding, Head and neck finding'),
('1266-VENOM', '1266', 'Mass lesion - nose (nasal)', 'Respiratory system finding, Mass lesion/swelling finding, Head and neck finding'),
('1267-VENOM', '1267', 'Mass lesion - nose (nasal), nasal cavity', 'Respiratory system finding, Mass lesion/swelling finding, Head and neck finding'),
('1268-VENOM', '1268', 'Mass lesion - oesophageal', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding'),
('1269-VENOM', '1269', 'Mass lesion - oral cavity (mouth)', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Head and neck finding'),
('1270-VENOM', '1270', 'Mass lesion - orbital', 'Musculoskeletal system finding, Ophthalmology/vision finding, Mass lesion/swelling finding, Head and neck finding'),
('1271-VENOM', '1271', 'Mass lesion - ovarian', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding, Abdominal finding'),
('1272-VENOM', '1272', 'Mass lesion - pancreatic', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding'),
('1273-VENOM', '1273', 'Mass lesion - parathyroid', 'Endocrine/metabolic finding, Mass lesion/swelling finding, Head and neck finding'),
('1274-VENOM', '1274', 'Mass lesion - penile', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding'),
('1275-VENOM', '1275', 'Mass lesion - perianal', 'Perineal finding, Mass lesion/swelling finding'),
('5849-VENOM', '5849', 'Mass lesion - peritoneum', 'Mass lesion/swelling finding, Abdominal finding'),
('1276-VENOM', '1276', 'Mass lesion - pharyngeal', 'Respiratory system finding, Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Head and neck finding'),
('1277-VENOM', '1277', 'Mass lesion - pituitary', 'Nervous system finding, Endocrine/Metabolic finding, Mass lesion/swelling finding, Head and neck finding'),
('7395-VENOM', '7395', 'Mass lesion - prepuce', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding'),
('1278-VENOM', '1278', 'Mass lesion - prostatic', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding, Abdominal finding'),
('1279-VENOM', '1279', 'Mass lesion - pulmonary (lung)', 'Respiratory system finding, Mass lesion/swelling, Thoracic finding'),
('1280-VENOM', '1280', 'Mass lesion - pyloric', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding'),
('1281-VENOM', '1281', 'Mass lesion - rectal', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding, Pelvic structure finding'),
('1282-VENOM', '1282', 'Mass lesion - renal (kidney)', 'Urogenital system finding, Mass lesion/swelling finding, Abdominal finding'),
('1283-VENOM', '1283', 'Mass lesion - retrobulbar', 'Ophthalmology/vision finding, Musculoskeletal system finding, Mass lesion/swelling finding, Head and neck finding'),
('1284-VENOM', '1284', 'Mass lesion - retroperitoneal', 'Mass lesion/swelling finding, Abdominal finding'),
('7038-VENOM', '7038', 'Mass lesion - rib', 'Musculoskeletal system finding, Mass lesion/swelling finding, Thoracic finding'),
('3912-VENOM', '3912', 'Mass lesion - salivary', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Head and neck finding'),
('16971-VENOM', '16971', 'Mass lesion - scapula', 'Musculoskeletal system finding, Mass lesion/swelling finding, Limb finding'),
('1286-VENOM', '1286', 'Mass lesion - scrotal', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding'),
('1287-VENOM', '1287', 'Mass lesion - skin (cutaneous)', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('1288-VENOM', '1288', 'Mass lesion - skin (cutaneous), perianal', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding, Perianal finding'),
('1289-VENOM', '1289', 'Mass lesion - spinal cord', 'Nervous system finding, Mass lesion/swelling finding'),
('1290-VENOM', '1290', 'Mass lesion - splenic', 'Haematopoietic system finding, Mass lesion/swelling finding, Abdominal finding'),
('1291-VENOM', '1291', 'Mass lesion - subcutaneous tissue', 'Soft tissue finding, Mass lesion/swelling finding'),
('1292-VENOM', '1292', 'Mass lesion - synovial', 'Musculoskeletal system finding, Mass lesion/swelling finding'),
('1293-VENOM', '1293', 'Mass lesion - testicular', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding'),
('1294-VENOM', '1294', 'Mass lesion - third eyelid/nictitating membrane', 'Ophthalmology/vision finding, Soft tissue finding, Mass lesion/swelling finding, Head and neck finding'),
('1295-VENOM', '1295', 'Mass lesion - thoracic', 'Thoracic finding, Mass lesion/swelling finding'),
('7667-VENOM', '7667', 'Mass lesion - thoracic (chest) wall', 'Thoracic finding, Mass lesion/swelling finding'),
('1296-VENOM', '1296', 'Mass lesion - thyroid', 'Endocrine/metabolic finding, Mass lesion/swelling finding, Head and neck finding'),
('1297-VENOM', '1297', 'Mass lesion - tongue (lingual)', 'Alimentary tract/Digestive system finding, Soft tissue finding, Mass lesion/swelling finding, Head and neck finding'),
('1298-VENOM', '1298', 'Mass lesion - tonsillar', 'Respiratory system finding, Alimentary tract/Digestive system finding, Lymphatic system finding, Haematopoietic system finding, Immune system finding, Mass lesion/swelling finding, Head and neck finding'),
('1299-VENOM', '1299', 'Mass lesion - tracheal', 'Respiratory system finding, Mass lesion/swelling, Head and neck finding'),
('1300-VENOM', '1300', 'Mass lesion - urethral', 'Urogenital system finding, Mass lesion/swelling finding'),
('1301-VENOM', '1301', 'Mass lesion - urinary bladder', 'Urogenital system finding, Mass lesion/swelling finding, Abdominal finding'),
('16659-VENOM', '16659', 'Mass lesion - uterine', 'Reproductive system finding, Mass lesion/swelling finding, Abdominal finding'),
('1302-VENOM', '1302', 'Mass lesion - vaginal', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding, Pelvic structure finding'),
('17208-VENOM', '17208', 'Mass lesion - vertebral column', 'Musculoskeletal system finding, Spine (vertebral column) finding, Mass lesion/swelling finding'),
('1303-VENOM', '1303', 'Mass lesion - vulval', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding'),
('16436-VENOM', '16436', 'Mass lesion (site unspecified)', 'Mass lesion/swelling finding'),
('17218-VENOM', '17218', 'Mast cell tumour - cutaneous (grade unspecified)', 'Haircoat/skin (+ appendages) finding, Haematopoietic system finding, Mass lesion/swelling finding'),
('17212-VENOM', '17212', 'Mast cell tumour - cutaneous, grade I', 'Haircoat/skin (+ appendages) finding, Haematopoietic system finding, Mass lesion/swelling finding'),
('17214-VENOM', '17214', 'Mast cell tumour - cutaneous, grade II', 'Haircoat/skin (+ appendages) finding, Haematopoietic system finding, Mass lesion/swelling finding'),
('17216-VENOM', '17216', 'Mast cell tumour - cutaneous, grade III', 'Haircoat/skin (+ appendages) finding, Haematopoietic system finding, Mass lesion/swelling finding'),
('17220-VENOM', '17220', 'Mast cell tumour - cutaneous, multiple', 'Haircoat/skin (+ appendages) finding, Haematopoietic system finding, Mass lesion/swelling finding'),
('17222-VENOM', '17222', 'Mast cell tumour - subcutaneous', 'Haematopoietic system finding, Soft tissue finding, Mass lesion/swelling finding'),
('16441-VENOM', '16441', 'Mast cell tumour (site unspecified)', 'Haematopoietic system finding, Mass lesion/swelling finding'),
('17224-VENOM', '17224', 'Mast cell tumour, visceral - hepatic (liver)', 'Haematopoietic system finding, Hepatobiliary system finding, Mass lesion/swelling finding, Abdominal finding'),
('17225-VENOM', '17225', 'Mast cell tumour, visceral - intestine', 'Haematopoietic system finding, Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding'),
('17226-VENOM', '17226', 'Mast cell tumour, visceral - spleen', 'Haematopoietic system finding, Mass lesion/swelling finding, Abdominal finding'),
('1306-VENOM', '1306', 'Masticatory myositis', 'Musculoskeletal system finding, Inflammatory disorder finding, Immune system finding'),
('1307-VENOM', '1307', 'Mastitis', 'Reproductive system finding, Inflammatory disorder finding'),
('7473-VENOM', '7473', 'Mastocytosis, systemic', 'Haematopoietic system finding, Immune system finding, Mass lesion/swelling finding'),
('1309-VENOM', '1309', 'Mediastinitis', 'Thoracic finding, Inflammatory disorder finding'),
('1310-VENOM', '1310', 'Megacolon', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('1311-VENOM', '1311', 'Megacolon - acquired', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('1312-VENOM', '1312', 'Megacolon - idiopathic', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('1313-VENOM', '1313', 'Megaoesophagus', 'Alimentary tract/Digestive system finding'),
('1314-VENOM', '1314', 'Megaoesophagus, canine idiopathic', 'Alimentary tract/Digestive system finding'),
('1315-VENOM', '1315', 'Meibomian adenoma', 'Ophthalmology/vision finding, Mass lesion/swelling finding, Head and neck finding'),
('1316-VENOM', '1316', 'Meibomianitis', 'Ophthalmology/vision finding, Inflammatory disorder finding, Head and neck finding'),
('16443-VENOM', '16443', 'Melanocytic neoplasm - benign melanoma (site unspecified)', 'Mass lesion/swelling finding'),
('1318-VENOM', '1318', 'Melanocytic neoplasm - benign melanoma, iris', 'Ophthalmology/vision finding, Mass lesion/swelling finding, Head and neck finding'),
('17230-VENOM', '17230', 'Melanocytic neoplasm - benign melanoma, oral (mouth)', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Head and neck finding'),
('16445-VENOM', '16445', 'Melanocytic neoplasm - malignant melanoma (site unspecified)', 'Mass lesion/swelling finding'),
('17233-VENOM', '17233', 'Melanocytic neoplasm - malignant melanoma, mucocutaneous', 'Soft tissue finding, Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('17235-VENOM', '17235', 'Melanocytic neoplasm - malignant melanoma, oral (mouth)', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Head and neck finding'),
('17237-VENOM', '17237', 'Melanocytic neoplasm - malignant melanoma, subungual', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding, Limb finding'),
('1320-VENOM', '1320', 'Meningeal cyst', 'Nervous system finding, Mass lesion/swelling finding'),
('1322-VENOM', '1322', 'Meningioma - intra-cranial', 'Nervous system finding, Mass lesion/swelling finding, Head and neck finding'),
('1323-VENOM', '1323', 'Meningioma - spinal', 'Nervous system finding, Mass lesion/swelling finding'),
('16447-VENOM', '16447', 'Meningioma (site unspecified)', 'Nervous system finding, Mass lesion/swelling finding'),
('1324-VENOM', '1324', 'Meningitis', 'Nervous system finding, Inflammatory disorder finding'),
('1325-VENOM', '1325', 'Meningitis - eosinophilic', 'Nervous system finding, Inflammatory disorder finding'),
('1326-VENOM', '1326', 'Meningitis - septic', 'Nervous system finding, Inflammatory disorder finding, Infectious disorder finding'),
('1327-VENOM', '1327', 'Meningitis - sterile/aseptic', 'Nervous system finding, Inflammatory disorder finding'),
('1328-VENOM', '1328', 'Meningoencephalitis', 'Nervous system finding, Inflammatory disorder finding, Head and neck finding'),
('17981-VENOM', '17981', 'Meningoencephalitis of unknown aetiology (MUE)', 'Nervous system finding, Inflammatory disorder finding, Head and neck finding'),
('1329-VENOM', '1329', 'Meningomyelocele', 'Nervous system finding, Congenital finding'),
('7452-VENOM', '7452', 'Meniscal (stifle) injury', 'Musculoskeletal system finding, Limb finding'),
('5767-VENOM', '5767', 'Mesenteric torsion/volvulus', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('17239-VENOM', '17239', 'Mesothelioma, malignant - pericardial', 'Cardiovascular system finding, Mass lesion/swelling finding, Thoracic finding'),
('17241-VENOM', '17241', 'Mesothelioma, malignant - peritoneal', 'Mass lesion/swelling finding, Abdominal finding'),
('1333-VENOM', '1333', 'Mesothelioma, malignant - pleural', 'Mass lesion/swelling finding, Thoracic finding'),
('16449-VENOM', '16449', 'Mesothelioma, malignant (site unspecified)', 'Mass lesion/swelling finding'),
('15579-VENOM', '15579', 'Metabolic acidosis', 'Endocrine/Metabolic finding'),
('15581-VENOM', '15581', 'Metabolic alkalosis', 'Endocrine/Metabolic finding'),
('4192-VENOM', '4192', 'Metaphyseal osteopathy (Hypertrophic osteodystrophy)', 'Musculoskeletal system finding, Development disorder finding, Limb finding'),
('1336-VENOM', '1336', 'Metatarsal fistulae of GSD', 'Haircoat/skin (+ appendages) finding, Immune system finding, Limb finding'),
('1337-VENOM', '1337', 'Methicillin-resistant staphylococcus aureus (MRSA) colonisation', 'Infectious disorder finding'),
('1338-VENOM', '1338', 'Methicillin-resistant staphylococcus aureus (MRSA) infection', 'Infectious disorder finding'),
('16770-VENOM', '16770', 'Methicillin-resistant staphylococcus colonisation (species unidentified)', 'Infectious disorder finding'),
('16771-VENOM', '16771', 'Methicillin-resistant staphylococcus infection (species unidentified)', 'Infectious disorder finding'),
('16768-VENOM', '16768', 'Methicillin-resistant staphylococcus pseudointermedius colonisation', 'Infectious disorder finding'),
('16769-VENOM', '16769', 'Methicillin-resistant staphylococcus pseudointermedius infection', 'Infectious disorder finding'),
('1339-VENOM', '1339', 'Metritis', 'Reproductive system finding, Inflammatory disorder finding, Abdominal finding'),
('1340-VENOM', '1340', 'Misalliance', 'Reproductive system finding'),
('1341-VENOM', '1341', 'Mitral valve disorder', 'Cardiovascular system finding, Thoracic finding'),
('1342-VENOM', '1342', 'Mitral valve dysplasia', 'Cardiovascular system finding, Thoracic finding'),
('1343-VENOM', '1343', 'Mitral valve myxomatous degeneration', 'Cardiovascular system finding, Degenerative disorder finding, Thoracic finding'),
('1344-VENOM', '1344', 'Mitral valve prolapse', 'Cardiovascular system finding, Thoracic finding'),
('1345-VENOM', '1345', 'Mitral valve regurgitation/insufficiency', 'Cardiovascular system finding, Thoracic finding'),
('1346-VENOM', '1346', 'Mitral valve stenosis', 'Cardiovascular system finding, Thoracic finding'),
('1347-VENOM', '1347', 'Mucinosis', 'Haircoat/skin (+ appendages) finding, Hereditary disorder finding'),
('1348-VENOM', '1348', 'Mucinosis - diffuse cutaneous ', 'Haircoat/skin (+ appendages) finding, Hereditary disorder finding'),
('1349-VENOM', '1349', 'Multidrug-resistant E. coli infection', 'Infectious disorder finding'),
('1350-VENOM', '1350', 'Multiple collagenous naevi of GSD', 'Haircoat/skin (+ appendages) finding'),
('16840-VENOM', '16840', 'Multiple myeloma (plasma cell myeloma)', 'Lymphatic system finding, Immune system finding, Haematopoietic system finding, Mass lesion/swelling finding'),
('1352-VENOM', '1352', 'Multiple organ dysfunction syndrome (MODS)', 'Clinical finding'),
('5788-VENOM', '5788', 'Muscle injury', 'Musculoskeletal system finding, Traumatic episode finding'),
('1353-VENOM', '1353', 'Muscle rupture', 'Musculoskeletal system finding, Traumatic episode finding'),
('7672-VENOM', '7672', 'Myalgia (muscle pain)', 'Musculoskeletal system finding, Nervous system finding'),
('1355-VENOM', '1355', 'Myasthenia gravis', 'Nervous system finding, Musculoskeletal system finding, Immune system finding'),
('1356-VENOM', '1356', 'Mycobacterial infection', 'Infectious disorder finding'),
('1357-VENOM', '1357', 'Mycobacterial infection - cutaneous (skin)', 'Infectious disorder finding, Haircoat/skin (+ appendages) finding'),
('1358-VENOM', '1358', 'Mycoplasma haemocanis (Haemobartonella canis) infection', 'Infectious disorder finding'),
('1359-VENOM', '1359', 'Mycoplasma haemofelis (Haemobartonella felis) infection', 'Infectious disorder finding'),
('1360-VENOM', '1360', 'Myelitis (spinal cord)', 'Nervous system finding, Inflammatory disorder finding'),
('1361-VENOM', '1361', 'Myelodysplastic syndrome', 'Haematopoietic system finding'),
('1362-VENOM', '1362', 'Myelofibrosis', 'Haematopoietic system finding'),
('1365-VENOM', '1365', 'Myelomalacia', 'Nervous system finding'),
('1367-VENOM', '1367', 'Myelopathy, Degenerative', 'Nervous system finding, Degenerative disorder finding'),
('1363-VENOM', '1363', 'Myeloproliferative disease (syndrome)', 'Haematopoietic system finding'),
('1364-VENOM', '1364', 'Myelosuppression', 'Haematopoietic system finding'),
('4554-VENOM', '4554', 'Myocardial failure', 'Cardiovascular system finding, Musculoskeletal system finding, Thoracic finding'),
('1368-VENOM', '1368', 'Myocardial infarction (MI)', 'Cardiovascular system finding, Musculoskeletal system finding, Thoracic finding'),
('1369-VENOM', '1369', 'Myocarditis', 'Cardiovascular system finding, Musculoskeletal system finding, Inflammatory disorder finding, Thoracic finding'),
('1371-VENOM', '1371', 'Myopathy (muscle disorder) - congenital', 'Musculoskeletal system finding, Congenital finding'),
('1372-VENOM', '1372', 'Myopathy (muscle disorder) - endocrine', 'Musculoskeletal system finding, Endocrine/Metabolic finding'),
('1373-VENOM', '1373', 'Myopathy (muscle disorder) - steroid-induced', 'Musculoskeletal system finding, Adverse reaction finding, Drug-related disorder finding'),
('15552-VENOM', '15552', 'Myopathy (muscle disorder) (unspecified)', 'Musculoskeletal system finding'),
('1374-VENOM', '1374', 'Myositis', 'Musculoskeletal system finding, Inflammatory disorder finding'),
('1375-VENOM', '1375', 'Myotonia', 'Musculoskeletal system finding'),
('1376-VENOM', '1376', 'Myotonia congenita', 'Musculoskeletal system finding, Congenital finding, Hereditary disorder finding'),
('1377-VENOM', '1377', 'Myxoedema', 'Endocrine/Metabolic finding, Haircoat/skin (+ appendages) finding, Head and neck finding'),
('7668-VENOM', '7668', 'Myxoma - cardiac (heart)', 'Connective tissue finding, Cardiovascular system finding, Mass lesion/swelling finding, Thoracic finding'),
('17243-VENOM', '17243', 'Myxoma - cutaneous (skin)', 'Haircoat/skin (+ appendages) finding, Connective tissue finding, Mass lesion/swelling finding'),
('17246-VENOM', '17246', 'Myxoma - subcutaneous', 'Soft tissue finding, Connective tissue finding, Mass lesion/swelling finding'),
('16451-VENOM', '16451', 'Myxoma (site unspecified)', 'Connective tissue finding, Mass lesion/swelling finding'),
('7669-VENOM', '7669', 'Myxosarcoma - cardiac (heart)', 'Connective tissue finding, Cardiovascular system finding, Mass lesion/swelling finding, Thoracic finding'),
('17248-VENOM', '17248', 'Myxosarcoma - cutaneous (skin)', 'Haircoat/skin (+ appendages) finding, Connective tissue finding, Mass lesion/swelling finding'),
('17250-VENOM', '17250', 'Myxosarcoma - subcutaneous', 'Soft tissue finding, Connective tissue finding, Mass lesion/swelling finding'),
('16453-VENOM', '16453', 'Myxosarcoma (site unspecified)', 'Connective tissue finding, Mass lesion/swelling finding'),
('16843-VENOM', '16843', 'Naevus - collagenous', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('16844-VENOM', '16844', 'Naevus - vascular', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('16841-VENOM', '16841', 'Naevus (type unspecified)', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('1381-VENOM', '1381', 'Narcolepsy', 'Nervous system finding, Head and neck finding'),
('15944-VENOM', '15944', 'Narrow complex tachycardia', 'Cardiovascular system finding, Thoracic finding'),
('1382-VENOM', '1382', 'Nasal (nose) disorder', 'Respiratory system finding, Head and neck finding'),
('1383-VENOM', '1383', 'Nasal (nose) disorder, congenital', 'Respiratory system finding, Congenital finding, Head and neck finding'),
('1384-VENOM', '1384', 'Nasal (nose) hyperkeratosis', 'Respiratory system finding, Haircoat/skin (+ appendages) finding, Head and neck finding'),
('1385-VENOM', '1385', 'Nasal (nose) infection', 'Respiratory system finding, Infectious disorder finding, Head and neck finding'),
('1386-VENOM', '1386', 'Nasal (nose) obstruction', 'Respiratory system finding, Head and neck finding'),
('1387-VENOM', '1387', 'Nasal sinus obstruction', 'Respiratory system finding, Head and neck finding'),
('9599-VENOM', '9599', 'Nasolacrimal (tear) duct abnormality', 'Ophthalmology/vision finding, Head and neck finding'),
('1388-VENOM', '1388', 'Nasolacrimal duct abnormality - acquired obstruction', 'Ophthalmology/vision finding, Head and neck finding'),
('1389-VENOM', '1389', 'Nasolacrimal duct abnormality - dacryocystitis', 'Ophthalmology/vision finding, Inflammatory disorder finding, Head and neck finding'),
('1390-VENOM', '1390', 'Nasolacrimal duct abnormality - imperforate punctum (lacrimal punctal aplasia)', 'Ophthalmology/vision finding, Congenital finding, Head and neck finding'),
('1391-VENOM', '1391', 'Nasolacrimal duct abnormality - trauma', 'Ophthalmology/vision finding, Traumatic episode finding, Head and neck finding'),
('9489-VENOM', '9489', 'Nasopharyngeal stenosis', 'Respiratory system finding, Alimentary tract/Digestive system finding, Head and neck finding'),
('1392-VENOM', '1392', 'Necrotising fasciitis', 'Infectious disorder finding, Inflammatory disorder finding, Soft tissue finding'),
('9694-VENOM', '9694', 'Neoplasm - abdominal (unspecified)', 'Mass lesion/swelling finding, Abdominal finding'),
('1400-VENOM', '1400', 'Neoplasm - adrenal', 'Endocrine/Metabolic finding, Abdominal finding, Mass lesion/swelling finding'),
('1401-VENOM', '1401', 'Neoplasm - anal', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding'),
('1402-VENOM', '1402', 'Neoplasm - anal gland/sac', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding'),
('1403-VENOM', '1403', 'Neoplasm - aural (ear)', 'Auditory system finding, Mass lesion/swelling finding, Head and neck finding'),
('1404-VENOM', '1404', 'Neoplasm - axilla', 'Mass lesion/swelling finding, Thoracic finding'),
('1405-VENOM', '1405', 'Neoplasm - bile duct', 'Hepatobiliary system finding, Mass lesion/swelling finding, Abdominal finding'),
('1406-VENOM', '1406', 'Neoplasm - bone', 'Musculoskeletal system finding, Mass lesion/swelling finding'),
('16969-VENOM', '16969', 'Neoplasm - bony pelvis', 'Musculoskeletal system finding, Mass lesion/swelling finding, Pelvic structure finding'),
('1407-VENOM', '1407', 'Neoplasm - brain', 'Nervous system finding, Head and neck finding, Mass lesion/swelling finding'),
('16354-VENOM', '16354', 'Neoplasm - calvarium (bone unspecified)', 'Musculoskeletal system finding, Mass lesion/swelling finding, Head and neck finding'),
('1408-VENOM', '1408', 'Neoplasm - cardiac (heart)', 'Cardiovascular system finding, Thoracic finding, Mass lesion/swelling finding'),
('4613-VENOM', '4613', 'Neoplasm - central nervous system (CNS)', 'Nervous system finding, Mass lesion/swelling finding'),
('1410-VENOM', '1410', 'Neoplasm - conjunctival', 'Ophthalmology/vision finding, Soft tissue finding, Mass lesion/swelling finding, Head and neck finding'),
('1411-VENOM', '1411', 'Neoplasm - cutaneous epithelial', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('1412-VENOM', '1412', 'Neoplasm - eyelid', 'Ophthalmology/vision finding, Mass lesion/swelling finding, Head and neck finding'),
('1413-VENOM', '1413', 'Neoplasm - facial', 'Mass lesion/swelling finding, Head and neck finding'),
('1414-VENOM', '1414', 'Neoplasm - facial, lip', 'Alimentary tract/Digestive system finding, Soft tissue finding, Mass lesion/swelling finding, Head and neck finding'),
('1415-VENOM', '1415', 'Neoplasm - gallbladder', 'Hepatobiliary system finding, Mass lesion/swelling finding, Abdominal finding'),
('1416-VENOM', '1416', 'Neoplasm - gastric (stomach)', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding'),
('4666-VENOM', '4666', 'Neoplasm - gingival (gum)', 'Alimentary tract/Digestive system finding, Soft tissue finding, Mass lesion/swelling finding, Head and neck finding'),
('1417-VENOM', '1417', 'Neoplasm - hair follicle', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('1418-VENOM', '1418', 'Neoplasm - heart base', 'Cardiovascular system finding, Thoracic finding, Mass lesion/swelling finding'),
('1419-VENOM', '1419', 'Neoplasm - hepatic (liver)', 'Hepatobiliary system finding, Mass lesion/swelling finding, Abdominal finding'),
('1420-VENOM', '1420', 'Neoplasm - intestinal, large', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding'),
('1421-VENOM', '1421', 'Neoplasm - intestinal, small', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding'),
('1422-VENOM', '1422', 'Neoplasm - intra-ocular', 'Ophthalmology/vision finding, Mass lesion/swelling finding, Head and neck finding'),
('1423-VENOM', '1423', 'Neoplasm - intra-ocular, uveal tract', 'Ophthalmology/vision finding, Mass lesion/swelling finding, Head and neck finding'),
('1424-VENOM', '1424', 'Neoplasm - iris', 'Ophthalmology/vision finding, Mass lesion/swelling finding, Head and neck finding'),
('16455-VENOM', '16455', 'Neoplasm - jaw (site unspecified)', 'Musculoskeletal system finding, Mass lesion/swelling finding, Head and neck finding'),
('1426-VENOM', '1426', 'Neoplasm - jaw, mandible', 'Musculoskeletal system finding, Mass lesion/swelling finding, Head and neck finding'),
('1427-VENOM', '1427', 'Neoplasm - jaw, maxilla', 'Musculoskeletal system finding, Mass lesion/swelling finding, Head and neck finding'),
('1428-VENOM', '1428', 'Neoplasm - joint', 'Musculoskeletal system finding, Mass lesion/swelling finding'),
('1429-VENOM', '1429', 'Neoplasm - laryngeal', 'Respiratory system finding, Mass lesion/swelling finding, Head and neck finding'),
('1430-VENOM', '1430', 'Neoplasm - limb, lower', 'Limb finding, Mass lesion/swelling finding'),
('1431-VENOM', '1431', 'Neoplasm - limb, upper', 'Limb finding, Mass lesion/swelling finding'),
('1432-VENOM', '1432', 'Neoplasm - lymph node', 'Lymphatic system finding, Immune system finding, Haematopoietic system finding, Mass lesion/swelling finding'),
('1433-VENOM', '1433', 'Neoplasm - mammary', 'Reproductive system finding, Mass lesion/swelling finding'),
('1435-VENOM', '1435', 'Neoplasm - mediastinal', 'Thoracic finding, Mass lesion/swelling finding'),
('1436-VENOM', '1436', 'Neoplasm - mediastinal, anterior', 'Thoracic finding, Mass lesion/swelling finding'),
('1437-VENOM', '1437', 'Neoplasm - meningeal', 'Nervous system finding, Mass lesion/swelling finding'),
('1438-VENOM', '1438', 'Neoplasm - muscle', 'Musculoskeletal system finding, Mass lesion/swelling finding'),
('1439-VENOM', '1439', 'Neoplasm - nasal sinus', 'Respiratory system finding, Mass lesion/swelling finding, Head and neck finding'),
('1440-VENOM', '1440', 'Neoplasm - nasal sinus, frontal sinus', 'Respiratory system finding, Mass lesion/swelling finding, Head and neck finding'),
('1441-VENOM', '1441', 'Neoplasm - neck (cervical)', 'Mass lesion/swelling finding, Head and neck finding'),
('1442-VENOM', '1442', 'Neoplasm - nose (nasal)', 'Respiratory system finding, Mass lesion/swelling finding, Head and neck finding'),
('1443-VENOM', '1443', 'Neoplasm - nose, nasal cavity', 'Respiratory system finding, Mass lesion/swelling finding, Head and neck finding'),
('1444-VENOM', '1444', 'Neoplasm - oesophageal', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding'),
('1445-VENOM', '1445', 'Neoplasm - oral cavity (mouth)', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Head and neck finding'),
('1446-VENOM', '1446', 'Neoplasm - orbital', 'Musculoskeletal system finding, Ophthalmology/vision finding, Mass lesion/swelling finding, Head and neck finding'),
('1447-VENOM', '1447', 'Neoplasm - ovarian', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding, Abdominal finding'),
('1448-VENOM', '1448', 'Neoplasm - pancreatic', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding'),
('1449-VENOM', '1449', 'Neoplasm - parathyroid', 'Endocrine/metabolic finding, Mass lesion/swelling finding, Head and neck finding'),
('1450-VENOM', '1450', 'Neoplasm - penile', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding'),
('1451-VENOM', '1451', 'Neoplasm - perianal', 'Perineal finding, Mass lesion/swelling finding'),
('1452-VENOM', '1452', 'Neoplasm - perianal gland', 'Haircoat/skin (+ appendages) finding, Perineal finding, Mass lesion/swelling finding'),
('1453-VENOM', '1453', 'Neoplasm - peripheral nerve', 'Nervous system finding, Mass lesion/swelling finding'),
('1455-VENOM', '1455', 'Neoplasm - peripheral nerve sheath', 'Nervous system finding, Mass lesion/swelling finding'),
('1454-VENOM', '1454', 'Neoplasm - peripheral nerve, trigeminal', 'Nervous system finding, Mass lesion/swelling finding'),
('9691-VENOM', '9691', 'Neoplasm - peritoneum (unspecified)', 'Mass lesion/swelling finding, Abdominal finding'),
('1457-VENOM', '1457', 'Neoplasm - pharyngeal', 'Respiratory system finding, Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Head and neck finding'),
('1458-VENOM', '1458', 'Neoplasm - pituitary', 'Nervous system finding, Endocrine/Metabolic finding, Mass lesion/swelling finding, Head and neck finding'),
('1459-VENOM', '1459', 'Neoplasm - pleural', 'Thoracic finding, Mass lesion/swelling finding'),
('1460-VENOM', '1460', 'Neoplasm - prostatic', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding, Abdominal finding'),
('1461-VENOM', '1461', 'Neoplasm - pulmonary (lung)', 'Respiratory system finding, Mass lesion/swelling, Thoracic finding'),
('1462-VENOM', '1462', 'Neoplasm - pyloric', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding'),
('1463-VENOM', '1463', 'Neoplasm - rectal', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding, Pelvic structure finding'),
('1464-VENOM', '1464', 'Neoplasm - renal (kidney)', 'Urogenital system finding, Mass lesion/swelling finding, Abdominal finding'),
('1465-VENOM', '1465', 'Neoplasm - retrobulbar', 'Ophthalmology/vision finding, Musculoskeletal system finding, Mass lesion/swelling finding, Head and neck finding'),
('1466-VENOM', '1466', 'Neoplasm - retroperitoneal', 'Mass lesion/swelling finding, Abdominal finding'),
('7040-VENOM', '7040', 'Neoplasm - rib', 'Musculoskeletal system finding, Mass lesion/swelling finding, Thoracic finding'),
('1467-VENOM', '1467', 'Neoplasm - salivary', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Head and neck finding'),
('16973-VENOM', '16973', 'Neoplasm - scapula', 'Musculoskeletal system finding, Mass lesion/swelling finding, Limb finding'),
('1468-VENOM', '1468', 'Neoplasm - scrotal', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding'),
('1469-VENOM', '1469', 'Neoplasm - sebaceous gland', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('1470-VENOM', '1470', 'Neoplasm - skin (cutaneous)', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('1471-VENOM', '1471', 'Neoplasm - skin (cutaneous), perianal', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding, Perianal finding'),
('1472-VENOM', '1472', 'Neoplasm - spinal cord', 'Nervous system finding, Mass lesion/swelling finding'),
('1473-VENOM', '1473', 'Neoplasm - splenic', 'Haematopoietic system finding, Mass lesion/swelling finding, Abdominal finding'),
('1474-VENOM', '1474', 'Neoplasm - sweat gland', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('1475-VENOM', '1475', 'Neoplasm - synovial', 'Musculoskeletal system finding, Mass lesion/swelling finding'),
('1476-VENOM', '1476', 'Neoplasm - testicular', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding'),
('1477-VENOM', '1477', 'Neoplasm - third eyelid/nictitating membrane', 'Ophthalmology/vision finding, Soft tissue finding, Mass lesion/swelling finding, Head and neck finding'),
('7666-VENOM', '7666', 'Neoplasm - thoracic (chest) wall', 'Thoracic finding, Mass lesion/swelling finding'),
('1478-VENOM', '1478', 'Neoplasm - thyroid', 'Endocrine/metabolic finding, Mass lesion/swelling finding, Head and neck finding'),
('1479-VENOM', '1479', 'Neoplasm - tongue (lingual)', 'Alimentary tract/Digestive system finding, Soft tissue finding, Mass lesion/swelling finding, Head and neck finding'),
('1480-VENOM', '1480', 'Neoplasm - tonsillar', 'Respiratory system finding, Alimentary tract/Digestive system finding, Lymphatic system finding, Haematopoietic system finding, Immune system finding, Mass lesion/swelling finding, Head and neck finding'),
('1481-VENOM', '1481', 'Neoplasm - tracheal', 'Respiratory system finding, Mass lesion/swelling, Head and neck finding'),
('1482-VENOM', '1482', 'Neoplasm - urethral', 'Urogenital system finding, Mass lesion/swelling finding'),
('1483-VENOM', '1483', 'Neoplasm - urinary bladder', 'Urogenital system finding, Mass lesion/swelling finding, Abdominal finding'),
('16657-VENOM', '16657', 'Neoplasm - uterine', 'Reproductive system finding, Mass lesion/swelling finding, Abdominal finding'),
('1484-VENOM', '1484', 'Neoplasm - vaginal', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding, Pelvic structure finding'),
('1485-VENOM', '1485', 'Neoplasm - vascular origin', 'Cardiovascular system finding, Mass lesion/swelling finding'),
('17252-VENOM', '17252', 'Neoplasm - vertebral column', 'Musculoskeletal system finding, Spine (vertebral column) finding, Mass lesion/swelling finding'),
('1486-VENOM', '1486', 'Neoplasm - vulval', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding'),
('1397-VENOM', '1397', 'Neoplasm, benign', 'Mass lesion/swelling finding'),
('1398-VENOM', '1398', 'Neoplasm, malignant', 'Mass lesion/swelling finding'),
('1399-VENOM', '1399', 'Neoplasm, uncertain behaviour', 'Mass lesion/swelling finding'),
('1487-VENOM', '1487', 'Neoplastic metastasis', 'Mass lesion/swelling finding'),
('1488-VENOM', '1488', 'Neoplastic metastasis - to bone', 'Musculoskeletal system finding, Mass lesion/swelling finding'),
('1489-VENOM', '1489', 'Neoplastic metastasis - to central nervous system (CNS)', 'Nervous system finding, Mass lesion/swelling finding'),
('1490-VENOM', '1490', 'Neoplastic metastasis - to liver (hepatic)', 'Hepatobiliary system system finding, Mass lesion/swelling finding, Abdominal finding'),
('1491-VENOM', '1491', 'Neoplastic metastasis - to lung (pulmonary)', 'Respiratory system finding, Mass lesion/swelling finding, Thoracic finding'),
('1492-VENOM', '1492', 'Neoplastic metastasis - to regional lymph node(s)', 'Lymphatic system finding, Immune system finding, Haematopoietic system finding, Mass lesion/swelling finding'),
('1493-VENOM', '1493', 'Neoplastic metastasis - to spleen', 'Haematopoietic system finding, Mass lesion/swelling finding, Abdominal finding'),
('1494-VENOM', '1494', 'Neoplastic metastasis not detected', 'Mass lesion/swelling finding'),
('1495-VENOM', '1495', 'Neospora infection', 'Infectious disorder finding'),
('17254-VENOM', '17254', 'Neovascularisation/benign vascular proliferation', 'Cardiovascular system finding, Mass lesion/swelling finding'),
('17260-VENOM', '17260', 'Nephroblastoma - renal (kidney)', 'Urogenital system finding, Mass lesion/swelling finding, Abdominal finding'),
('17258-VENOM', '17258', 'Nephroblastoma (site unspecified)', 'Mass lesion/swelling finding'),
('17264-VENOM', '17264', 'Nephroblastoma, extrarenal - spinal cord', 'Nervous system finding, Mass lesion/swelling finding'),
('17262-VENOM', '17262', 'Nephroblastoma, extrarenal (site unspecified)', 'Mass lesion/swelling finding'),
('1497-VENOM', '1497', 'Nephrocalcinosis', 'Urogenital system finding, Endocrine/metabolic finding, Abdominal finding'),
('1499-VENOM', '1499', 'Nephrotic syndrome', 'Urogenital system finding, Abdominal finding'),
('1500-VENOM', '1500', 'Nerve root disorder', 'Nervous system finding'),
('17266-VENOM', '17266', 'Neurofibroma', 'Nervous system finding, Mass lesion/swelling finding'),
('17268-VENOM', '17268', 'Neurofibrosarcoma', 'Nervous system finding, Mass lesion/swelling finding'),
('15551-VENOM', '15551', 'Neurological (nervous system) disorder (unspecified)', 'Nervous system finding'),
('15557-VENOM', '15557', 'Neuromuscular disorder (unspecified)', 'Nervous system finding, Musculoskeletal system finding'),
('17271-VENOM', '17271', 'Neuronoma', 'Nervous system finding, Mass lesion/swelling finding'),
('1501-VENOM', '1501', 'Neuropraxia', 'Nervous system finding, Traumatic episode finding'),
('7030-VENOM', '7030', 'Neutropenia', 'Haematopoietic system finding'),
('11018-VENOM', '11018', 'Neutropenia - immune-mediated', 'Haematopoietic system finding, Immune system finding'),
('2122-VENOM', '2122', 'Noise phobia', 'Nervous system finding'),
('1503-VENOM', '1503', 'Nuclear sclerosis', 'Ophthalmology/vision finding, Head and neck finding'),
('1504-VENOM', '1504', 'Nyctalopia (night blindness)', 'Ophthalmology/vision finding, Head and neck finding'),
('1505-VENOM', '1505', 'Obesity', 'Weight/Body condition finding'),
('1506-VENOM', '1506', 'Obesity - morbid obesity', 'Weight/Body condition finding'),
('1507-VENOM', '1507', 'Obstipation', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('18730-VENOM', '18730', 'Ocular (eye) coloboma (site unspecified)', 'Ophthalmology/vision finding, Congenital finding, Head and neck finding'),
('16847-VENOM', '16847', 'Odontoma - mandible', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Head and neck finding'),
('16848-VENOM', '16848', 'Odontoma - maxilla', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Head and neck finding'),
('1509-VENOM', '1509', 'Oesophageal diverticulum', 'Alimentary tract/Digestive system finding'),
('1510-VENOM', '1510', 'Oesophageal dysmotility', 'Alimentary tract/Digestive system finding'),
('1511-VENOM', '1511', 'Oesophageal injury', 'Alimentary tract/Digestive system finding, Traumatic episode finding'),
('1512-VENOM', '1512', 'Oesophageal perforation', 'Alimentary tract/Digestive system finding'),
('1513-VENOM', '1513', 'Oesophageal rupture', 'Alimentary tract/Digestive system finding'),
('1514-VENOM', '1514', 'Oesophageal stenosis/stricture', 'Alimentary tract/Digestive system finding'),
('1515-VENOM', '1515', 'Oesophageal ulceration', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding'),
('1516-VENOM', '1516', 'Oesophagitis', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding'),
('1517-VENOM', '1517', 'Oesophagitis - drug-related', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding, Drug-related disorder, Adverse reaction finding'),
('1518-VENOM', '1518', 'Oesophagitis - reflux', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding'),
('7710-VENOM', '7710', 'Oesophagitis - reflux (post-anaesthesia)', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding, Complication of medical care finding, Anaesthesia finding'),
('1520-VENOM', '1520', 'Oestrus', 'Reproductive system finding'),
('1521-VENOM', '1521', 'Onychodystrophy, idiopathic', 'Haircoat/skin (+ appendages) finding, Limb finding'),
('1522-VENOM', '1522', 'Onychodystrophy, symmetric lupoid', 'Haircoat/skin (+ appendages) finding, Limb finding'),
('1523-VENOM', '1523', 'Onychodystrophy, unspecified', 'Haircoat/skin (+ appendages) finding, Limb finding'),
('1524-VENOM', '1524', 'Onychomadesis, idiopathic', 'Haircoat/skin (+ appendages) finding, Limb finding'),
('1525-VENOM', '1525', 'Ophthalmoplegia', 'Ophthalmology/vision finding, Musculoskeletal system finding, Nervous system finding, Head and neck finding'),
('1526-VENOM', '1526', 'Optic disc cupping', 'Ophthalmology/vision finding, Nervous system finding, Head and neck finding'),
('18733-VENOM', '18733', 'Optic nerve head/optic disc coloboma - Atypical', 'Ophthalmology/vision finding, Congenital finding, Head and neck finding'),
('18734-VENOM', '18734', 'Optic nerve head/optic disc coloboma - Typical', 'Ophthalmology/vision finding, Congenital finding, Head and neck finding'),
('1527-VENOM', '1527', 'Optic neuritis', 'Ophthalmology/vision finding, Nervous system finding, Inflammatory disorder finding, Head and neck finding'),
('1528-VENOM', '1528', 'Orbital cellulitis', 'Ophthalmology/vision finding, Soft tissue finding, Haircoat/skin (+ appendages) finding, Musculoskeletal system finding, Inflammatory disorder finding, Head and neck finding'),
('1529-VENOM', '1529', 'Orbital trauma', 'Ophthalmology/vision finding, Musculoskeletal system finding, Traumatic episode finding, Head and neck finding'),
('1531-VENOM', '1531', 'Orchitis', 'Urogenital system finding, Reproductive system finding, Inflammatory disorder finding'),
('1530-VENOM', '1530', 'Oronasal fistula', 'Alimentary tract/Digestive system finding, Respiratory system finding, Head and neck finding'),
('1533-VENOM', '1533', 'Osteoarthritis - carpus', 'Musculoskeletal system finding, Limb finding'),
('1534-VENOM', '1534', 'Osteoarthritis - elbow', 'Musculoskeletal system finding, Limb finding'),
('1535-VENOM', '1535', 'Osteoarthritis - hip', 'Musculoskeletal system finding, Limb finding'),
('1536-VENOM', '1536', 'Osteoarthritis - hock', 'Musculoskeletal system finding, Limb finding'),
('1537-VENOM', '1537', 'Osteoarthritis - metacarpophalangeal joint', 'Musculoskeletal system finding, Limb finding'),
('1538-VENOM', '1538', 'Osteoarthritis - metatarsophalangeal joint', 'Musculoskeletal system finding, Limb finding'),
('1539-VENOM', '1539', 'Osteoarthritis - paw', 'Musculoskeletal system finding, Limb finding'),
('1540-VENOM', '1540', 'Osteoarthritis - shoulder', 'Musculoskeletal system finding, Limb finding'),
('1541-VENOM', '1541', 'Osteoarthritis - stifle', 'Musculoskeletal system finding, Limb finding'),
('1532-VENOM', '1532', 'Osteoarthritis (osteoarthrosis, degenerative joint disease (DJD))', 'Musculoskeletal system finding, Limb finding'),
('1542-VENOM', '1542', 'Osteoarthritis, multiple sites', 'Musculoskeletal system finding'),
('1543-VENOM', '1543', 'Osteoarthritis, secondary', 'Musculoskeletal system finding'),
('1544-VENOM', '1544', 'Osteochondritis dissecans (OCD)', 'Musculoskeletal system finding, Developmental disorder finding'),
('1546-VENOM', '1546', 'Osteochondritis dissecans (OCD) - elbow, humeral condyle', 'Musculosksletal system finding, Hereditary disorder finding, Developmental disorder finding, Limb finding'),
('1547-VENOM', '1547', 'Osteochondritis dissecans (OCD) - femoral condyle, lateral', 'Musculoskeletal system finding, Developmental disorder finding, Limb finding'),
('1548-VENOM', '1548', 'Osteochondritis dissecans (OCD) - femoral condyle, medial', 'Musculoskeletal system finding, Developmental disorder finding, Limb finding'),
('1549-VENOM', '1549', 'Osteochondritis dissecans (OCD) - shoulder', 'Musculoskeletal system finding, Developmental disorder finding, Limb finding'),
('1550-VENOM', '1550', 'Osteochondritis dissecans (OCD) - shoulder, humeral head', 'Musculoskeletal system finding, Developmental disorder finding, Limb finding'),
('1551-VENOM', '1551', 'Osteochondritis dissecans (OCD) - talus', 'Musculoskeletal system finding, Developmental disorder finding, Limb finding'),
('17273-VENOM', '17273', 'Osteochondroma - pelvis', 'Musculoskeletal system finding, Mass lesion/swelling finding, Pelvic structure finding'),
('17275-VENOM', '17275', 'Osteochondroma - rib', 'Musculoskeletal system finding, Mass lesion/swelling finding, Thoracic finding'),
('17277-VENOM', '17277', 'Osteochondroma - scapula', 'Musculoskeletal system finding, Mass lesion/swelling finding, Limb finding'),
('17279-VENOM', '17279', 'Osteochondroma - trachea', 'Musculoskeletal system finding, Respiratory system finding, Mass lesion/swelling, Head and neck finding'),
('17281-VENOM', '17281', 'Osteochondroma - vertebral column', 'Musculoskeletal system finding, Spine (vertebral column) finding, Mass lesion/swelling finding'),
('16457-VENOM', '16457', 'Osteochondroma (site unspecified)', 'Musculoskeletal system finding, Mass lesion/swelling finding'),
('17283-VENOM', '17283', 'Osteochondromatosis', 'Musculoskeletal system finding, Mass lesion/swelling finding'),
('1553-VENOM', '1553', 'Osteochondrosis (Juvenile osteochondritis, dyschondroplasia)', 'Musculoskeletal system finding, Developmental disorder finding'),
('5771-VENOM', '5771', 'Osteogenesis imperfecta', 'Musculoskeletal system finding, Connective tissue finding, Hereditary disorder finding'),
('17289-VENOM', '17289', 'Osteoma - multilobular', 'Musculoskeletal system finding, Mass lesion/swelling finding'),
('17287-VENOM', '17287', 'Osteoma (type unspecified)', 'Musculoskeletal system finding, Mass lesion/swelling finding'),
('1554-VENOM', '1554', 'Osteomyelitis', 'Haematopoietic system finding, Inflammatory disorder finding'),
('1555-VENOM', '1555', 'Osteomyelitis - femur', 'Haematopoietic system finding, Inflammatory disorder finding, Musculoskeletal system finding, Limb finding'),
('1556-VENOM', '1556', 'Osteomyelitis - humerus', 'Haematopoietic system finding, Inflammatory disorder finding, Musculoskeletal system finding, Limb finding'),
('1557-VENOM', '1557', 'Osteomyelitis - mandible', 'Haematopoietic system finding, Inflammatory disorder finding, Musculoskeletal system finding, Head and neck finding'),
('1558-VENOM', '1558', 'Osteomyelitis - maxilla', 'Haematopoietic system finding, Inflammatory disorder finding, Musculoskeletal system finding, Head and neck finding'),
('1559-VENOM', '1559', 'Osteomyelitis - paw', 'Haematopoietic system finding, Inflammatory disorder finding, Limb finding'),
('1560-VENOM', '1560', 'Osteomyelitis - radius', 'Haematopoietic system finding, Inflammatory disorder finding, Musculoskeletal system finding, Limb finding'),
('9426-VENOM', '9426', 'Osteomyelitis - scapula', 'Haematopoietic system finding, Inflammatory disorder finding, Musculoskeletal system finding, Limb finding'),
('1562-VENOM', '1562', 'Osteomyelitis - tibia', 'Haematopoietic system finding, Inflammatory disorder finding, Musculoskeletal system finding, Limb finding'),
('1563-VENOM', '1563', 'Osteomyelitis - ulna', 'Haematopoietic system finding, Inflammatory disorder finding, Musculoskeletal system finding, Limb finding'),
('1564-VENOM', '1564', 'Osteomyelitis - vertebral column', 'Haematopoietic system finding, Inflammatory disorder finding, Musculoskeletal system finding, Spine (vertebral column) finding'),
('1569-VENOM', '1569', 'Osteomyelitis of other aetiology', 'Haematopoietic system finding, Inflammatory disorder finding'),
('1565-VENOM', '1565', 'Osteomyelitis, acute', 'Haematopoietic system finding, Inflammatory disorder finding'),
('1566-VENOM', '1566', 'Osteomyelitis, bacterial', 'Haematopoietic system finding, Inflammatory disorder finding, Infectious disorder finding'),
('1567-VENOM', '1567', 'Osteomyelitis, chronic', 'Haematopoietic system finding, Inflammatory disorder finding'),
('1568-VENOM', '1568', 'Osteomyelitis, fungal', 'Haematopoietic system finding, Inflammatory disorder finding, Infectious disorder finding'),
('1570-VENOM', '1570', 'Osteopenia', 'Musculoskeletal system finding'),
('1571-VENOM', '1571', 'Osteopetrosis', 'Musculoskeletal system finding, Hereditary disorder finding'),
('16852-VENOM', '16852', 'Osteosarcoma - appendicular (type unspecified)', 'Musculoskeletal system finding, Connective tissue finding, Mass lesion/swelling finding, Limb finding'),
('16853-VENOM', '16853', 'Osteosarcoma - appendicular, parosteal', 'Musculoskeletal system finding, Connective tissue finding, Mass lesion/swelling finding, Limb finding'),
('16855-VENOM', '16855', 'Osteosarcoma - appendicular, periosteal', 'Musculoskeletal system finding, Connective tissue finding, Mass lesion/swelling finding, Limb finding'),
('16857-VENOM', '16857', 'Osteosarcoma - axial (site unspecified)', 'Musculoskeletal system finding, Connective tissue finding, Mass lesion/swelling finding'),
('16861-VENOM', '16861', 'Osteosarcoma - axial, mandible', 'Musculoskeletal system finding, Connective tissue finding, Mass lesion/swelling finding, Head and neck finding'),
('16862-VENOM', '16862', 'Osteosarcoma - axial, maxilla', 'Musculoskeletal system finding, Connective tissue finding, Mass lesion/swelling finding, Head and neck finding'),
('16859-VENOM', '16859', 'Osteosarcoma - axial, rib', 'Musculoskeletal system finding, Connective tissue finding, Mass lesion/swelling finding, Thoracic finding'),
('16867-VENOM', '16867', 'Osteosarcoma - extraskeletal (site unspecified)', 'Musculoskeletal system finding, Connective tissue finding, Mass lesion/swelling finding'),
('16868-VENOM', '16868', 'Osteosarcoma - extraskeletal, cutaneous (skin)/subcutaneous', 'Musculoskeletal system finding, Haircoat/skin (+ appendages) finding, Connective tissue finding, Mass lesion/swelling finding'),
('16870-VENOM', '16870', 'Osteosarcoma - extraskeletal, gastrointestinal', 'Alimentary tract/Digestive system finding, Musculoskeletal system finding, Connective tissue finding, Mass lesion/swelling finding, Abdominal finding'),
('16872-VENOM', '16872', 'Osteosarcoma - extraskeletal, hepatic (liver)', 'Hepatobiliary system finding, Musculoskeletal system finding, Connective tissue finding, Mass lesion/swelling finding, Abdominal finding'),
('16874-VENOM', '16874', 'Osteosarcoma - extraskeletal, mammary', 'Reproductive system finding, Musculoskeletal system finding, Connective tissue finding, Mass lesion/swelling finding'),
('16876-VENOM', '16876', 'Osteosarcoma - extraskeletal, muscle', 'Musculoskeletal system finding, Connective tissue finding, Mass lesion/swelling finding'),
('16878-VENOM', '16878', 'Osteosarcoma - extraskeletal, splenic', 'Haematopoietic system finding, Musculoskeletal system finding, Connective tissue finding, Mass lesion/swelling finding, Abdominal finding'),
('16880-VENOM', '16880', 'Osteosarcoma - extraskeletal, thyroid', 'Endocrine/metabolic finding, Musculoskeletal system finding, Connective tissue finding, Mass lesion/swelling finding, Head and neck finding'),
('16459-VENOM', '16459', 'Osteosarcoma (site unspecified)', 'Musculoskeletal system finding, Connective tissue finding, Mass lesion/swelling finding'),
('1576-VENOM', '1576', 'Otitis', 'Auditory system finding, Inflammatory disorder finding, Head and neck finding'),
('1577-VENOM', '1577', 'Otitis externa', 'Auditory system finding, Inflammatory disorder finding, Head and neck finding'),
('1578-VENOM', '1578', 'Otitis externa, acute', 'Auditory system finding, Inflammatory disorder finding, Head and neck finding'),
('1579-VENOM', '1579', 'Otitis externa, chronic', 'Auditory system finding, Inflammatory disorder finding, Head and neck finding'),
('1582-VENOM', '1582', 'Otitis externa, infective', 'Auditory system finding, Inflammatory disorder finding, Infective disorder finding, Head and neck finding'),
('1580-VENOM', '1580', 'Otitis externa, meatus and canal only', 'Auditory system finding, Inflammatory disorder finding, Head and neck finding'),
('1581-VENOM', '1581', 'Otitis externa, pinna only', 'Auditory system finding, Inflammatory disorder finding, Haircoat/skin (+ appendages) finding, Head and neck finding'),
('1583-VENOM', '1583', 'Otitis media', 'Auditory system finding, Inflammatory disorder finding, Head and neck finding'),
('1584-VENOM', '1584', 'Otitis media, acute', 'Auditory system finding, Inflammatory disorder finding, Head and neck finding'),
('1585-VENOM', '1585', 'Otitis media, chronic', 'Auditory system finding, Inflammatory disorder finding, Head and neck finding'),
('1586-VENOM', '1586', 'Otitis media, infective', 'Auditory system finding, Inflammatory disorder finding, Infective disorder finding, Head and neck finding'),
('1587-VENOM', '1587', 'Ovarian cyst', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding, Abdominal finding'),
('1588-VENOM', '1588', 'Ovarian disorder', 'Urogenital system finding, Reproductive system finding, Abdominal finding'),
('16999-VENOM', '16999', 'Ovarian dysgerminoma', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding, Abdominal finding'),
('1589-VENOM', '1589', 'Ovarian remnant syndrome (residual ovary syndrome)', 'Urogenital system finding, Reproductive system finding, Abdominal finding'),
('2896-VENOM', '2896', 'Overgrown haircoat', 'Haircoat/skin (+ appendages) finding'),
('9639-VENOM', '9639', 'Overgrown incisor(s)', 'Alimentary tract/Digestive system finding, Head and neck finding'),
('9641-VENOM', '9641', 'Overgrown molar(s)', 'Alimentary tract/Digestive system finding, Head and neck finding'),
('2895-VENOM', '2895', 'Overgrown nail(s)', 'Haircoat/skin (+ appendages) finding, Limb finding'),
('1590-VENOM', '1590', 'Pancreatic cyst', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding'),
('1591-VENOM', '1591', 'Pancreatic pseudocyst', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Abdominal finding'),
('1592-VENOM', '1592', 'Pancreatitis', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding, Abdominal finding'),
('1593-VENOM', '1593', 'Pancreatitis - acute', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding, Abdominal finding'),
('1594-VENOM', '1594', 'Pancreatitis - acute, necrotising', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding, Abdominal finding'),
('1595-VENOM', '1595', 'Pancreatitis - chronic', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding, Abdominal finding'),
('1596-VENOM', '1596', 'Pancreatitis - recurrent/relapsing', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding, Abdominal finding'),
('1597-VENOM', '1597', 'Pancytopenia', 'Haematopoietic system finding, Haemostatic system finding'),
('1598-VENOM', '1598', 'Panniculitis', 'Haircoat/skin (+ appendages) finding, Soft tissue finding, Inflammatory disorder finding'),
('1599-VENOM', '1599', 'Panosteitis, Canine', 'Musculoskeletal system finding'),
('17295-VENOM', '17295', 'Papilloma, solitary - choroid  plexus', 'Nervous system finding, Head and neck finding, Mass lesion/swelling finding'),
('17299-VENOM', '17299', 'Papilloma, solitary - mammary', 'Reproductive system finding, Mass lesion/swelling finding'),
('9463-VENOM', '9463', 'Papilloma, solitary – mucous membrane', 'Soft tissue finding, Mass lesion/swelling finding'),
('9464-VENOM', '9464', 'Papilloma, solitary – other', 'Mass lesion/swelling finding'),
('9465-VENOM', '9465', 'Papilloma, solitary – skin (cutaneous)', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('17297-VENOM', '17297', 'Papillomas, multiple - choroid plexus', 'Nervous system finding, Head and neck finding, Mass lesion/swelling finding'),
('17302-VENOM', '17302', 'Papillomas, multiple - mammary', 'Reproductive system finding, Mass lesion/swelling finding'),
('9466-VENOM', '9466', 'Papillomas, multiple – mucous membrane', 'Soft tissue finding, Mass lesion/swelling finding'),
('9467-VENOM', '9467', 'Papillomas, multiple – other', 'Mass lesion/swelling finding'),
('9468-VENOM', '9468', 'Papillomas, multiple – skin (cutaneous)', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('1602-VENOM', '1602', 'Paraneoplastic alopecia', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('1603-VENOM', '1603', 'Paraneoplastic neuromyopathy/neuropathy', 'Nervous system finding, Musculoskeletal system finding, Mass lesion/swelling finding'),
('1604-VENOM', '1604', 'Paraneoplastic syndrome', 'Mass lesion/swelling finding'),
('1605-VENOM', '1605', 'Paraphimosis', 'Urogenital system finding, Reproductive system finding'),
('1606-VENOM', '1606', 'Paraprostatic cyst', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding, Abdominal finding'),
('1608-VENOM', '1608', 'Parasite infestation', 'Infectious disorder finding'),
('1623-VENOM', '1623', 'Parasite infestation - (Neo)trombiculosis', 'Infectious disorder finding'),
('1609-VENOM', '1609', 'Parasite infestation - Aelurostrongylosis', 'Infectious disorder finding'),
('1610-VENOM', '1610', 'Parasite infestation - Angiostrongylosis', 'Infectious disorder finding'),
('1611-VENOM', '1611', 'Parasite infestation - Ascariasis', 'Infectious disorder finding'),
('1612-VENOM', '1612', 'Parasite infestation - Babesiosis', 'Infectious disorder finding'),
('1613-VENOM', '1613', 'Parasite infestation - Capillariasis', 'Infectious disorder finding'),
('4556-VENOM', '4556', 'Parasite infestation - Cestodes (tapeworms)', 'Infectious disorder finding'),
('1614-VENOM', '1614', 'Parasite infestation - Cheyletiellosis', 'Infectious disorder finding'),
('1615-VENOM', '1615', 'Parasite infestation - Demodicosis', 'Infectious disorder finding'),
('1616-VENOM', '1616', 'Parasite infestation - Dermanyssus gallinae', 'Infectious disorder finding'),
('1617-VENOM', '1617', 'Parasite infestation - Dirofilariasis', 'Infectious disorder finding'),
('1618-VENOM', '1618', 'Parasite infestation - Ehrlichiosis', 'Infectious disorder finding'),
('1619-VENOM', '1619', 'Parasite infestation - Filaroidosis', 'Infectious disorder finding'),
('1620-VENOM', '1620', 'Parasite infestation - fleas', 'Infectious disorder finding'),
('5865-VENOM', '5865', 'Parasite infestation - hookworm', 'Infectious disorder finding'),
('5866-VENOM', '5866', 'Parasite infestation - hookworm - Ancylostomiasis', 'Infectious disorder finding'),
('5867-VENOM', '5867', 'Parasite infestation - hookworm - Uncinariasis', 'Infectious disorder finding'),
('1621-VENOM', '1621', 'Parasite infestation - Leishmaniasis', 'Infectious disorder finding'),
('1622-VENOM', '1622', 'Parasite infestation - Myiasis (flystrike)', 'Infectious disorder finding'),
('1624-VENOM', '1624', 'Parasite infestation - Otodectes cynotis', 'Infectious disorder finding'),
('8545-VENOM', '8545', 'Parasite infestation - Pediculosis (lice)', 'Infectious disorder finding'),
('1626-VENOM', '1626', 'Parasite infestation - Scabies', 'Infectious disorder finding'),
('1627-VENOM', '1627', 'Parasite infestation - ticks', 'Infectious disorder finding'),
('1628-VENOM', '1628', 'Parasite infestation - Toxocariasis', 'Infectious disorder finding'),
('1607-VENOM', '1607', 'Parasitic disease', 'Infectious disorder finding'),
('1629-VENOM', '1629', 'Paronychia', 'Haircoat/skin (+ appendages) finding, Inflammatory disorder finding, Limb finding'),
('1630-VENOM', '1630', 'Paronychia, bacterial', 'Haircoat/skin (+ appendages) finding, Inflammatory disorder finding, Infectious disorder finding, Limb finding'),
('1631-VENOM', '1631', 'Paronychia, fungal', 'Haircoat/skin (+ appendages) finding, Inflammatory disorder finding, Infectious disorder finding, Limb finding'),
('1633-VENOM', '1633', 'Patellar luxation', 'Musculoskeletal system finding, Limb finding'),
('1634-VENOM', '1634', 'Patellar luxation - lateral', 'Musculoskeletal system finding, Limb finding'),
('1635-VENOM', '1635', 'Patellar luxation - medial', 'Musculoskeletal system finding, Limb finding'),
('1636-VENOM', '1636', 'Patent ductus arteriosus (PDA)', 'Cardiovascular system finding, Congenital finding, Thoracic finding'),
('1637-VENOM', '1637', 'Patent ductus arteriosus (PDA) with left-to-right shunt', 'Cardiovascular system finding, Congenital finding, Thoracic finding'),
('1638-VENOM', '1638', 'Patent ductus arteriosus (PDA) with right-to-left shunt', 'Cardiovascular system finding, Congenital finding, Thoracic finding'),
('1639-VENOM', '1639', 'Patent ductus venosus', 'Cardiovascular system finding, Congenital finding, Abdominal finding'),
('1640-VENOM', '1640', 'Pattern baldness', 'Haircoat/skin (+ appendages) finding'),
('11097-VENOM', '11097', 'Peliosis hepatis', 'Hepatobiliary system finding, Abdominal finding'),
('1641-VENOM', '1641', 'Pelodera dermatitis', 'Infectious disorder finding, Haircoat/skin (+ appendages) finding, Inflammatory disorder finding'),
('1642-VENOM', '1642', 'Pemphigus disorder', 'Haircoat/skin (+ appendages) finding, Immune system finding'),
('1643-VENOM', '1643', 'Pemphigus foliaceus', 'Haircoat/skin (+ appendages) finding, Immune system finding'),
('1644-VENOM', '1644', 'Penile disorder', 'Urogenital system finding, Reproductive system finding'),
('1645-VENOM', '1645', 'Pericardial effusion', 'Cardiovascular system finding, Thoracic finding'),
('1646-VENOM', '1646', 'Pericardial effusion - idiopathic', 'Cardiovascular system finding, Thoracic finding'),
('1647-VENOM', '1647', 'Pericardial effusion - neoplastic', 'Cardiovascular system finding, Thoracic finding, Mass lesion/swelling finding'),
('1648-VENOM', '1648', 'Pericardial effusion - septic', 'Cardiovascular system finding, Infectious disorder finding, Thoracic finding'),
('1649-VENOM', '1649', 'Pericardial effusion - traumatic', 'Cardiovascular system finding, Traumatic episode finding, Thoracic finding'),
('1650-VENOM', '1650', 'Pericarditis', 'Cardiovascular system finding, Inflammatory disorder finding, Thoracic finding'),
('1658-VENOM', '1658', 'Periodontal disease', 'Alimentary tract/Digestive system finding, Head and neck finding'),
('1659-VENOM', '1659', 'Periodontitis', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding, Head and neck finding'),
('1651-VENOM', '1651', 'Peripheral neuritis', 'Nervous system finding, Inflammatory disorder finding'),
('1652-VENOM', '1652', 'Peripheral neuropathy', 'Nervous system finding'),
('1653-VENOM', '1653', 'Peripheral neuropathy - radial neuropathy', 'Nervous system finding, Limb finding'),
('1654-VENOM', '1654', 'Peripheral neuropathy - sciatic neuropathy', 'Nervous system finding, Limb finding'),
('1655-VENOM', '1655', 'Peripheral neuropathy - ulnar neuropathy', 'Nervous system finding, Limb finding'),
('1656-VENOM', '1656', 'Peripheral neuropathy, endocrinopathy-related', 'Nervous system finding, Endocrine/Metabolic finding'),
('1657-VENOM', '1657', 'Peripheral neuropathy, idiopathic', 'Nervous system finding'),
('15597-VENOM', '15597', 'Peripheral oedema', 'Soft tissue finding'),
('1660-VENOM', '1660', 'Peritonitis', 'Inflammatory disorder finding, Abdominal finding'),
('1661-VENOM', '1661', 'Peritonitis, chemical - biliary', 'Hepatobiliary system finding, Inflammatory disorder finding, Abdominal finding'),
('1662-VENOM', '1662', 'Peritonitis, chemical - urinary (uroperitoneum)', 'Urogenital system finding, Inflammatory disorder finding, Abdominal finding'),
('1663-VENOM', '1663', 'Peritonitis, infectious', 'Infectious disorder finding, Inflammatory disorder finding, Abdominal finding'),
('1664-VENOM', '1664', 'Peritonitis, infectious - bacterial', 'Infectious disorder finding, Inflammatory disorder finding, Abdominal finding'),
('1665-VENOM', '1665', 'Peritonitis, infectious - bacterial, faecal', 'Infectious disorder finding, Inflammatory disorder finding, Abdominal finding'),
('1666-VENOM', '1666', 'Peritonitis, infectious - purulent', 'Infectious disorder finding, Inflammatory disorder finding, Abdominal finding'),
('7382-VENOM', '7382', 'Persistent left cranial vena cava', 'Cardiovascular system finding, Congenital finding, Thoracic finding'),
('1667-VENOM', '1667', 'Persistent right aortic arch (PRAA)', 'Cardiovascular system finding, Congenital finding, Thoracic finding'),
('1668-VENOM', '1668', 'Phaeochromocytoma', 'Endocrine/Metabolic finding, Abdominal finding, Mass lesion/swelling finding'),
('1669-VENOM', '1669', 'Pharyngeal injury', 'Respiratory system finding, Alimentary tract/Digestive system finding, Traumatic episode finding, Head and neck finding'),
('1670-VENOM', '1670', 'Pharyngeal obstruction', 'Respiratory system finding, Alimentary tract/Digestive system finding, Head and neck finding'),
('1671-VENOM', '1671', 'Pharyngitis', 'Respiratory system finding, Alimentary tract/Digestive system finding, Inflammatory disorder finding, Head and neck finding'),
('1672-VENOM', '1672', 'Pharyngitis - infectious', 'Respiratory system finding, Alimentary tract/Digestive system finding, Inflammatory disorder finding, Infectious disorder finding, Head and neck finding'),
('9651-VENOM', '9651', 'Phenobarbital responsive hypersialosis ', 'Alimentary tract/Digestive system finding, Head and neck finding'),
('7026-VENOM', '7026', 'Phosphofructokinase (PFK) deficiency', 'Endocrine/Metabolic finding, Hereditary disorder finding'),
('1673-VENOM', '1673', 'Physeal disorder - premature physeal closure', 'Musculoskeletal system finding'),
('17304-VENOM', '17304', 'Pilomatrixoma', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('1674-VENOM', '1674', 'Pinnal alopecia', 'Haircoat/skin (+ appendages) finding, Auditory system finding, Head and neck finding'),
('1675-VENOM', '1675', 'Pituitary macroadenoma', 'Nervous system finding, Endocrine/Metabolic finding, Mass lesion/swelling finding, Head and neck finding'),
('1676-VENOM', '1676', 'Pituitary microadenoma', 'Nervous system finding, Endocrine/Metabolic finding, Mass lesion/swelling finding, Head and neck finding'),
('17307-VENOM', '17307', 'Plasmacytoma - cutaneous (skin)', 'Haircoat/skin (+ appendages) finding, Lymphatic system finding, Immune system finding, Haematopoietic system finding, Mass lesion/swelling finding'),
('17309-VENOM', '17309', 'Plasmacytoma - gastrointestinal', 'Alimentary tract/Digestive system finding, Lymphatic system finding, Immune system finding, Haematopoietic system finding, Mass lesion/swelling finding, Abdominal finding'),
('17313-VENOM', '17313', 'Plasmacytoma - mucocutaneous', 'Haircoat/skin (+ appendages) finding, Soft tissue finding, Lymphatic system finding, Immune system finding, Haematopoietic system finding, Mass lesion/swelling finding'),
('17311-VENOM', '17311', 'Plasmacytoma - mucosal', 'Soft tissue finding, Lymphatic system finding, Immune system finding, Haematopoietic system finding, Mass lesion/swelling finding'),
('17315-VENOM', '17315', 'Plasmacytoma - oral (mouth)', 'Alimentary tract/Digestive system finding, Lymphatic system finding, Immune system finding, Haematopoietic system finding, Mass lesion/swelling finding, Head and neck finding'),
('16461-VENOM', '16461', 'Plasmacytoma (site unspecified)', 'Lymphatic system finding, Immune system finding, Haematopoietic system finding, Mass lesion/swelling finding'),
('1678-VENOM', '1678', 'Pleural effusion', 'Effusion finding, Thoracic finding'),
('1679-VENOM', '1679', 'Pleural effusion - neoplastic', 'Effusion finding, Thoracic finding, Mass lesion/swelling finding'),
('1680-VENOM', '1680', 'Pleuritis', 'Inflammatory disorder finding, Thoracic finding'),
('10098-VENOM', '10098', 'Pleuroperitoneal hernia', 'Musculoskeletal system finding, Congenital finding, Thoracic finding, Abdominal finding'),
('1681-VENOM', '1681', 'Pneumomediastinum', 'Thoracic finding'),
('1682-VENOM', '1682', 'Pneumonia', 'Respiratory system finding, Inflammatory disorder finding, Thoracic finding'),
('1683-VENOM', '1683', 'Pneumonia - aspiration', 'Respiratory system finding, Inflammatory disorder finding, Thoracic finding'),
('1684-VENOM', '1684', 'Pneumonia - bacterial', 'Respiratory system finding, Inflammatory disorder finding, Infectious disorder finding, Thoracic finding'),
('1685-VENOM', '1685', 'Pneumonia - bronchopneumonia', 'Respiratory system finding, Inflammatory disorder finding, Thoracic finding'),
('1686-VENOM', '1686', 'Pneumonia - infectious', 'Respiratory system finding, Inflammatory disorder finding, Infectious disorder finding, Thoracic finding'),
('1687-VENOM', '1687', 'Pneumonia - interstitial', 'Respiratory system finding, Inflammatory disorder finding, Thoracic finding'),
('1688-VENOM', '1688', 'Pneumonia - lobar', 'Respiratory system finding, Inflammatory disorder finding, Thoracic finding'),
('1689-VENOM', '1689', 'Pneumonia - non-infectious', 'Respiratory system finding, Inflammatory disorder finding, Thoracic finding'),
('1690-VENOM', '1690', 'Pneumonia - parasitic', 'Respiratory system finding, Inflammatory disorder finding, Infectious disorder finding, Thoracic finding'),
('1691-VENOM', '1691', 'Pneumonia due to Pneumocystis carinii infection', 'Respiratory system finding, Inflammatory disorder finding, Infectious disorder finding, Thoracic finding'),
('1692-VENOM', '1692', 'Pneumonitis - uraemic', 'Respiratory system finding, Inflammatory disorder finding, Endocrine/Metabolic finding, Thoracic finding'),
('1693-VENOM', '1693', 'Pneumoperitoneum', 'Abdominal finding'),
('1694-VENOM', '1694', 'Pneumoretroperitoneum', 'Abdominal finding'),
('1695-VENOM', '1695', 'Pneumothorax', 'Thoracic finding'),
('1696-VENOM', '1696', 'Pneumothorax - spontaneous', 'Thoracic finding'),
('1697-VENOM', '1697', 'Pneumothorax - tension pneumothorax', 'Thoracic finding'),
('1698-VENOM', '1698', 'Pneumothorax - traumatic', 'Traumatic episode finding, Thoracic finding'),
('1699-VENOM', '1699', 'Pododermatitis', 'Haircoat/skin (+ appendages) finding, Inflammatory disorder finding, Limb finding'),
('1700-VENOM', '1700', 'Pododermatitis - infectious', 'Haircoat/skin (+ appendages) finding, Inflammatory disorder finding, Infectious disorder finding, Limb finding'),
('1701-VENOM', '1701', 'Pododermatitis - plasma cell', 'Haircoat/skin (+ appendages) finding, Inflammatory disorder finding, Haematopoietic system finding, Immune system finding, Lymphatic system finding, Limb finding'),
('1703-VENOM', '1703', 'Polyarthritis', 'Musculoskeletal system finding, Inflammatory disorder finding'),
('1704-VENOM', '1704', 'Polyarthritis - erosive', 'Musculoskeletal system finding, Inflammatory disorder finding'),
('1705-VENOM', '1705', 'Polyarthritis - idiopathic', 'Musculoskeletal system finding, Inflammatory disorder finding'),
('1706-VENOM', '1706', 'Polyarthritis - infectious', 'Musculoskeletal system finding, Inflammatory disorder finding, Infectious disorder finding'),
('1707-VENOM', '1707', 'Polyarthritis, acute', 'Musculoskeletal system finding, Inflammatory disorder finding'),
('1708-VENOM', '1708', 'Polyarthritis, chronic', 'Musculoskeletal system finding, Inflammatory disorder finding'),
('1709-VENOM', '1709', 'Polyarthropathy', 'Musculoskeletal system finding'),
('1710-VENOM', '1710', 'Polyarthropathy - immune-mediated', 'Musculoskeletal system finding, Immune system finding'),
('1702-VENOM', '1702', 'Polycystic ovary(ies)', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding, Abdominal finding'),
('16463-VENOM', '16463', 'Polycythaemia (type unspecified)', 'Haematopoietic system finding'),
('1713-VENOM', '1713', 'Polycythaemia, secondary', 'Haematopoietic system finding'),
('19081-VENOM', '19081', 'Polydactyly', 'Congenital finding, Limb finding'),
('1714-VENOM', '1714', 'Polymyositis', 'Musculoskeletal system finding, Inflammatory disorder finding'),
('1715-VENOM', '1715', 'Polyneuritis', 'Nervous system finding, Inflammatory disorder finding'),
('1716-VENOM', '1716', 'Polyneuropathy', 'Nervous system finding'),
('1717-VENOM', '1717', 'Polyneuropathy - toxic', 'Nervous system finding, Intoxication (poisoning)/toxin exposure finding'),
('1720-VENOM', '1720', 'Polyp - anal', 'Alimentary tract/Digestive system finding, Soft tissue finding, Mass lesion/swelling finding'),
('17975-VENOM', '17975', 'Polyp - ear canal', 'Auditory system finding, Soft tissue finding, Mass lesion/swelling finding, Head and neck finding'),
('1721-VENOM', '1721', 'Polyp - laryngeal', 'Respiratory system finding, Soft tissue finding, Mass lesion/swelling finding, Head and neck finding'),
('1722-VENOM', '1722', 'Polyp - middle ear', 'Auditory system finding, Soft tissue finding, Mass lesion/swelling finding, Head and neck finding'),
('1723-VENOM', '1723', 'Polyp - nasal (nose)', 'Respiratory system finding, Soft tissue finding, Mass lesion/swelling finding, Head and neck finding'),
('1724-VENOM', '1724', 'Polyp - nasopharyngeal', 'Respiratory system finding, Alimentary tract/Digestive system finding, Soft tissue finding, Mass lesion/swelling finding, Head and neck finding'),
('1725-VENOM', '1725', 'Polyp - rectal', 'Alimentary tract/Digestive system finding, Soft tissue finding, Mass lesion/swelling finding, Abdominal finding, Pelvic structure finding'),
('1726-VENOM', '1726', 'Polyp - urethral', 'Urogenital system finding, Soft tissue finding, Mass lesion/swelling finding'),
('1727-VENOM', '1727', 'Polyp - urinary bladder', 'Urogenital system finding, Soft tissue finding, Mass lesion/swelling finding, Abdominal finding'),
('16465-VENOM', '16465', 'Polyp (site unspecified)', 'Soft tissue finding, Mass lesion/swelling finding'),
('1718-VENOM', '1718', 'Polyradiculoneuropathy', 'Nervous system finding'),
('11001-VENOM', '11001', 'Portosystemic (portasystemic, portacaval) shunt - acquired', 'Cardiovascular system finding, Abdominal finding'),
('11002-VENOM', '11002', 'Portosystemic (portasystemic, portacaval) shunt - congenital', 'Cardiovascular system finding, Congenital finding, Abdominal finding'),
('11003-VENOM', '11003', 'Portosystemic (portasystemic, portacaval) shunt - congenital extra-hepatic', 'Cardiovascular system finding, Congenital finding, Abdominal finding'),
('11004-VENOM', '11004', 'Portosystemic (portasystemic, portacaval) shunt - congenital intra-hepatic', 'Cardiovascular system finding, Congenital finding, Abdominal finding'),
('11005-VENOM', '11005', 'Portosystemic (portasystemic, portacaval) shunt - extrahepatic', 'Cardiovascular system finding, Congenital finding, Abdominal finding'),
('11006-VENOM', '11006', 'Portosystemic (portasystemic, portacaval) shunt - intrahepatic', 'Cardiovascular system finding, Congenital finding, Abdominal finding'),
('11007-VENOM', '11007', 'Portosystemic (portasystemic, portacaval) shunt disorder', 'Cardiovascular system finding, Congenital finding, Endocrine/Metabolic finding, Abdominal finding'),
('1735-VENOM', '1735', 'Post-clipping alopecia', 'Haircoat/skin (+ appendages) finding'),
('17971-VENOM', '17971', 'Post-operative complication - excessive bruising', 'Complication of medical care finding, Wound finding, Cardiovascular system finding'),
('17969-VENOM', '17969', 'Post-operative complication - excessive pain', 'Complication of medical care finding, Nervous system finding'),
('17973-VENOM', '17973', 'Post-operative complication - excessive swelling', 'Complication of medical care finding, Wound finding, Mass lesion/swelling finding'),
('1737-VENOM', '1737', 'Post-operative complication - gastrointestinal anastomotic dehiscence', 'Complication of medical care finding, Alimentary tract/Digestive system finding, Abdominal finding'),
('1738-VENOM', '1738', 'Post-operative complication - haemorrhage', 'Complication of medical care finding, Cardiovascular system finding'),
('1739-VENOM', '1739', 'Post-operative complication - incisional hernia', 'Complication of medical care finding, Musculoskeletal system finding'),
('1740-VENOM', '1740', 'Post-operative complication - infection', 'Complication of medical care finding, Infectious disorder finding'),
('1741-VENOM', '1741', 'Post-operative complication - infection with septicaemia', 'Complication of medical care finding, Infectious disorder finding'),
('17961-VENOM', '17961', 'Post-operative complication - other', 'Complication of medical care finding'),
('17967-VENOM', '17967', 'Post-operative complication – self-trauma', 'Complication of medical care finding'),
('17965-VENOM', '17965', 'Post-operative complication - vomiting', 'Alimentary tract/Digestive system finding, Complication of medical care finding'),
('1742-VENOM', '1742', 'Post-operative complication - wound breakdown', 'Complication of medical care finding, Wound finding'),
('1743-VENOM', '1743', 'Post-operative complication - wound infection', 'Complication of medical care finding, Wound finding, Infectious disorder finding'),
('1744-VENOM', '1744', 'Post-operative complication - wound infection with MRSA', 'Complication of medical care finding, Wound finding, Infectious disorder finding'),
('1745-VENOM', '1745', 'Post-operative complication - wound necrosis', 'Complication of medical care finding, Wound finding'),
('9524-VENOM', '9524', 'Post-operative complication - wound seroma', 'Complication of medical care finding, Wound finding'),
('1746-VENOM', '1746', 'Poxvirus infection', 'Infectious disorder finding'),
('1747-VENOM', '1747', 'Preauricular alopecia', 'Haircoat/skin (+ appendages) finding'),
('1748-VENOM', '1748', 'Pregnancy', 'Reproductive system finding'),
('1749-VENOM', '1749', 'Premature epiphyseal closure', 'Musculoskeletal system finding'),
('7397-VENOM', '7397', 'Prepuce disorder', 'Urogenital system finding, Reproductive system finding'),
('11099-VENOM', '11099', 'Pressure-related skin lesion (pressure sore)', 'Haircoat/skin (+ appendages) finding'),
('5862-VENOM', '5862', 'Primary hyperaldosteronism', 'Endocrine/Metabolic finding, Abdominal finding'),
('9671-VENOM', '9671', 'Primary ileus', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('4573-VENOM', '4573', 'PROBLEM NOT IDENTIFIED', 'Clinical finding'),
('1750-VENOM', '1750', 'Proctitis', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding, Abdominal finding, Pelvic structure finding'),
('2123-VENOM', '2123', 'Progesterone deficiency', 'Endocrine/Metabolic finding'),
('1751-VENOM', '1751', 'Progressive retinal atrophy (PRA)', 'Ophthalmology/vision finding, Hereditary disorder finding, Degenerative disorder finding, Head and neck finding'),
('1752-VENOM', '1752', 'Prostatic cyst', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding, Abdominal finding'),
('1753-VENOM', '1753', 'Prostatic disorder', 'Urogenital system finding, Reproductive system finding, Abdominal finding'),
('1754-VENOM', '1754', 'Prostatitis', 'Urogenital system finding, Reproductive system finding, Inflammatory disorder finding, Abdominal finding'),
('1755-VENOM', '1755', 'Prostatomegaly', 'Urogenital system finding, Reproductive system finding, Abdominal finding'),
('1756-VENOM', '1756', 'Protein-losing enteropathy (PLE)', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('11749-VENOM', '11749', 'Protein-losing nephropathy (PLN) – azotaemic', 'Urogenital system finding, Endocrine/Metabolic finding, Abdominal finding'),
('11751-VENOM', '11751', 'Protein-losing nephropathy (PLN) – non-azotaemic', 'Urogenital system finding, Abdominal finding'),
('11747-VENOM', '11747', 'Protein-losing nephropathy (PLN) (azotaemia unspecified)', 'Urogenital system finding, Abdominal finding'),
('1758-VENOM', '1758', 'Pseudohyperparathyroidism', 'Endocrine/Metabolic finding, Mass lesion/swelling finding'),
('1759-VENOM', '1759', 'Pseudophacia', 'Ophthalmology/vision finding, Head and neck finding'),
('1760-VENOM', '1760', 'Pseudopregnancy (phantom pregnancy)', 'Reproductive system finding, Endocrine/Metabolic finding'),
('1761-VENOM', '1761', 'Psychogenic dermatosis', 'Haircoat/skin (+ appendages) finding, Nervous system finding, Head and neck finding'),
('1762-VENOM', '1762', 'Psychogenic dysuria', 'Urogenital system finding, Nervous system finding, Head and neck finding'),
('1763-VENOM', '1763', 'Psychogenic polydipsia', 'Nervous system finding, Head and neck finding'),
('1764-VENOM', '1764', 'Ptosis', 'Ophthalmology/vision finding, Head and neck finding'),
('1765-VENOM', '1765', 'Pulmonary artery aneurysm', 'Cardiovascular system finding, Respiratory system finding, Thoracic finding'),
('1766-VENOM', '1766', 'Pulmonary contusion', 'Respiratory system finding, Traumatic episode finding, Thoracic finding'),
('1767-VENOM', '1767', 'Pulmonary emphysema', 'Respiratory system finding, Thoracic finding'),
('1768-VENOM', '1768', 'Pulmonary fibrosis', 'Respiratory system finding, Thoracic finding'),
('1769-VENOM', '1769', 'Pulmonary fibrosis, idiopathic', 'Respiratory system finding, Thoracic finding'),
('1770-VENOM', '1770', 'Pulmonary haemorrhage', 'Cardiovascular system finding, Respiratory system finding, Thoracic finding'),
('1772-VENOM', '1772', 'Pulmonary oedema', 'Respiratory system finding, Thoracic finding'),
('1773-VENOM', '1773', 'Pulmonary oedema - cardiogenic', 'Cardiovascular system finding, Respiratory system finding, Thoracic finding'),
('1774-VENOM', '1774', 'Pulmonary oedema - non-cardiogenic', 'Respiratory system finding, Thoracic finding'),
('1775-VENOM', '1775', 'Pulmonic valve regurgitation/insufficiency', 'Cardiovascular system finding, Thoracic finding'),
('1776-VENOM', '1776', 'Pulmonic valve stenosis', 'Cardiovascular system finding, Thoracic finding'),
('1777-VENOM', '1777', 'Pure red cell aplasia', 'Haematopoietic system finding'),
('1778-VENOM', '1778', 'Pyelectasia', 'Urogenital system finding, Abdominal finding'),
('1779-VENOM', '1779', 'Pyelonephritis', 'Urogenital system finding, Inflammatory disorder finding, Abdominal finding'),
('5773-VENOM', '5773', 'Pyloric stenosis', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('5775-VENOM', '5775', 'Pyloric stenosis - congenital', 'Alimentary tract/Digestive system finding, Congenital finding, Abdominal finding'),
('1780-VENOM', '1780', 'Pyoderma', 'Haircoat/skin (+ appendages) finding, Infectious disorder finding'),
('1781-VENOM', '1781', 'Pyoderma - botryomycosis', 'Haircoat/skin (+ appendages) finding, Infectious disorder finding'),
('1782-VENOM', '1782', 'Pyoderma - deep', 'Haircoat/skin (+ appendages) finding, Infectious disorder finding'),
('1783-VENOM', '1783', 'Pyoderma - mucocutaneous', 'Haircoat/skin (+ appendages) finding, Soft tissue finding, Infectious disorder finding'),
('1784-VENOM', '1784', 'Pyoderma - superficial', 'Haircoat/skin (+ appendages) finding, Infectious disorder finding'),
('1785-VENOM', '1785', 'Pyometra', 'Reproductive system finding, Infectious disorder finding, Abdominal finding'),
('1786-VENOM', '1786', 'Pyrexia of unknown origin (PUO)', 'Body temperature finding'),
('7024-VENOM', '7024', 'Pyruvate kinase (PK) deficiency', 'Endocrine/Metabolic finding, Hereditary disorder finding'),
('1787-VENOM', '1787', 'Rabies', 'Infectious disorder finding'),
('1788-VENOM', '1788', 'Rathke''s pouch cyst', 'Nervous system finding, Endocrine/Metabolic finding, Congenital finding, Mass lesion/swelling finding, Head and neck finding'),
('1789-VENOM', '1789', 'Rectal prolapse', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('1790-VENOM', '1790', 'Rectal stricture', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('1791-VENOM', '1791', 'Refeeding syndrome', 'Endocrine/Metabolic finding'),
('17962-VENOM', '17962', 'Regurgitation during/post-anaesthesia', 'Alimentary tract/Digestive system finding, Complication of medical care finding, Anaesthesia finding'),
('11765-VENOM', '11765', 'Renal (kidney) anomaly, congenital – aplasia', 'Urogenital system finding, Congenital finding, Abdominal finding'),
('11767-VENOM', '11767', 'Renal (kidney) anomaly, congenital – dysgenesis', 'Urogenital system finding, Congenital finding, Abdominal finding'),
('1795-VENOM', '1795', 'Renal (kidney) anomaly, congenital - dysplasia', 'Urogenital system finding, Congenital finding, Abdominal finding'),
('1796-VENOM', '1796', 'Renal (kidney) anomaly, congenital - hypoplasia', 'Urogenital system finding, Congenital finding, Abdominal finding'),
('1797-VENOM', '1797', 'Renal (kidney) anomaly, congenital - Polycystic kidney disease (PKD)', 'Urogenital system finding, Congenital finding, Abdominal finding'),
('11763-VENOM', '11763', 'Renal (kidney) anomaly, congenital (unspecified)', 'Urogenital system finding, Congenital finding, Abdominal finding'),
('16776-VENOM', '16776', 'Renal (kidney) cyst', 'Urogenital system finding, Mass lesion/swelling finding, Abdominal finding'),
('4334-VENOM', '4334', 'Renal (kidney) disorder', 'Urogenital system finding, Abdominal finding'),
('8372-VENOM', '8372', 'Renal tubular glucosuria', 'Urogenital system finding, Abdominal finding'),
('15583-VENOM', '15583', 'Respiratory acidosis', 'Endocrine/Metabolic finding'),
('15585-VENOM', '15585', 'Respiratory alkalosis', 'Endocrine/Metabolic finding'),
('1806-VENOM', '1806', 'Respiratory failure', 'Respiratory system finding'),
('1807-VENOM', '1807', 'Respiratory tract infection', 'Respiratory system finding, Infectious disorder finding'),
('1808-VENOM', '1808', 'Respiratory tract infection - bacterial', 'Respiratory system finding, Infectious disorder finding'),
('1809-VENOM', '1809', 'Respiratory tract infection - fungal', 'Respiratory system finding, Infectious disorder finding'),
('1810-VENOM', '1810', 'Respiratory tract infection - parasitic', 'Respiratory system finding, Infectious disorder finding'),
('1811-VENOM', '1811', 'Respiratory tract infection - viral', 'Respiratory system finding, Infectious disorder finding'),
('1812-VENOM', '1812', 'Respiratory tract infection, lower (LRTI)', 'Respiratory system finding, Infectious disorder finding, Thoracic finding'),
('1813-VENOM', '1813', 'Respiratory tract infection, upper (URTI)', 'Respiratory system finding, Infectious disorder finding, Head and neck finding'),
('1814-VENOM', '1814', 'Respiratory tract obstruction, lower', 'Respiratory system finding, Thoracic finding'),
('1815-VENOM', '1815', 'Respiratory tract obstruction, upper', 'Respiratory system finding, Head and neck finding'),
('2124-VENOM', '2124', 'Retained deciduous tooth', 'Alimentary tract/Digestive system finding, Developmental disorder finding, Head and neck finding'),
('1816-VENOM', '1816', 'Retinal atrophy/degeneration', 'Ophthalmology/vision finding, Degenerative disorder finding, Head and neck finding'),
('1817-VENOM', '1817', 'Retinal atrophy/degeneration - focal', 'Ophthalmology/vision finding, Degenerative disorder finding, Head and neck finding'),
('1818-VENOM', '1818', 'Retinal atrophy/degeneration - generalised', 'Ophthalmology/vision finding, Degenerative disorder finding, Head and neck finding'),
('1819-VENOM', '1819', 'Retinal atrophy/degeneration - multifocal', 'Ophthalmology/vision finding, Degenerative disorder finding, Head and neck finding'),
('1820-VENOM', '1820', 'Retinal haemorrhage', 'Ophthalmology/vision finding, Cardiovascular system finding, Head and neck finding'),
('1821-VENOM', '1821', 'Retinal separation/detachment', 'Ophthalmology/vision finding, Head and neck finding'),
('1822-VENOM', '1822', 'Retinopathy (retinal disorder)', 'Ophthalmology/vision finding, Head and neck finding'),
('1823-VENOM', '1823', 'Retinopathy (retinal disorder) - hypertensive', 'Ophthalmology/vision finding, Cardiovascular system finding, Head and neck finding'),
('18039-VENOM', '18039', 'Retroperitoneal haemorrhage', 'Cardiovascular system finding, Abdominal finding'),
('1824-VENOM', '1824', 'Reverse sneezing', 'Respiratory system finding, Head and neck finding'),
('1825-VENOM', '1825', 'Rhabdomyolysis', 'Musculoskeletal system finding'),
('1826-VENOM', '1826', 'Rhabdomyolysis - exertional', 'Musculoskeletal system finding'),
('17317-VENOM', '17317', 'Rhabdomyoma', 'Musculoskeletal system finding, Mass lesion/swelling finding'),
('17319-VENOM', '17319', 'Rhabdomyosarcoma', 'Musculoskeletal system finding, Connective tissue finding, Mass lesion/swelling finding'),
('1827-VENOM', '1827', 'Rheumatoid arthritis (RA)', 'Musculoskeletal system finding, Immune system finding'),
('1828-VENOM', '1828', 'Rhinitis', 'Respiratory system finding, Inflammatory disorder finding, Head and neck finding'),
('1829-VENOM', '1829', 'Rhinitis - acute', 'Respiratory system finding, Inflammatory disorder finding, Head and neck finding'),
('1830-VENOM', '1830', 'Rhinitis - chronic', 'Respiratory system finding, Inflammatory disorder finding, Head and neck finding'),
('1831-VENOM', '1831', 'Rhinitis - lymphoplasmacytic', 'Respiratory system finding, Inflammatory disorder finding, Head and neck finding'),
('1832-VENOM', '1832', 'Rhinitis, allergic', 'Respiratory system finding, Inflammatory disorder finding, Immune system finding, Head and neck finding'),
('1833-VENOM', '1833', 'Rhinitis, infectious - bacterial', 'Respiratory system finding, Inflammatory disorder finding, Infectious disorder finding, Head and neck finding'),
('1834-VENOM', '1834', 'Rhinitis, infectious - fungal', 'Respiratory system finding, Inflammatory disorder finding, Infectious disorder finding, Head and neck finding'),
('1835-VENOM', '1835', 'Rhinitis, infectious - viral', 'Respiratory system finding, Inflammatory disorder finding, Infectious disorder finding, Head and neck finding'),
('9537-VENOM', '9537', 'Salivary mucocele (sialocele)', 'Alimentary tract/Digestive system finding, Head and neck finding'),
('1836-VENOM', '1836', 'Salmonellosis', 'Infectious disorder finding'),
('1837-VENOM', '1837', 'Salmonellosis - gastrointestinal', 'Alimentary tract/Digestive system finding, Infectious disorder finding, Abdominal finding'),
('1838-VENOM', '1838', 'Salmonellosis - systemic', 'Infectious disorder finding'),
('17321-VENOM', '17321', 'Sarcoma - round cell/spindle cell', 'Connective tissue finding, Mass lesion/swelling finding'),
('1840-VENOM', '1840', 'Sarcoma - soft tissue', 'Connective tissue finding, Soft tissue finding, Mass lesion/swelling finding'),
('1841-VENOM', '1841', 'Sarcoma - synovial', 'Connective tissue finding, Musculoskeletal system finding, Mass lesion/swelling finding'),
('17323-VENOM', '17323', 'Sarcoma - undifferentiated/anaplastic', 'Connective tissue finding, Mass lesion/swelling finding'),
('16468-VENOM', '16468', 'Sarcoma (site unspecified)', 'Connective tissue finding, Mass lesion/swelling finding'),
('16469-VENOM', '16469', 'Sarcoma (type unspecified)', 'Connective tissue finding, Mass lesion/swelling finding'),
('18035-VENOM', '18035', 'SCCED (Spontaneous Chronic Corneal Epithelial Defect)', 'Ophthalmology/vision finding, Inflammatory disorder finding, Head and neck finding'),
('1842-VENOM', '1842', 'Schnauzer comedo syndrome', 'Haircoat/skin (+ appendages) finding'),
('17270-VENOM', '17270', 'Schwannoma (neurilemmoma)', 'Nervous system finding, Mass lesion/swelling finding'),
('5778-VENOM', '5778', 'Scleral injury', 'Ophthalmology/vision finding, Traumatic episode finding, Head and neck finding'),
('1843-VENOM', '1843', 'Scleroderma', 'Connective tissue finding'),
('9677-VENOM', '9677', 'Sclerosing enteropathy', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('1844-VENOM', '1844', 'Scottie cramp', 'Nervous system finding, Hereditary disorder finding'),
('7407-VENOM', '7407', 'Scrotal disorder', 'Urogenital system finding, Reproductive system finding'),
('1845-VENOM', '1845', 'Sebaceous adenitis', 'Haircoat/skin (+ appendages) finding, Immune system finding'),
('1846-VENOM', '1846', 'Sebaceous adenitis and mural folliculitis', 'Haircoat/skin (+ appendages) finding, Immune system finding'),
('4565-VENOM', '4565', 'Second degree AV block', 'Cardiovascular system finding, Thoracic finding'),
('9673-VENOM', '9673', 'Secondary ileus', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('9675-VENOM', '9675', 'Secondary ileus - postoperative', 'Alimentary tract/Digestive system finding, Abdominal finding, Complication of medical care finding'),
('1847-VENOM', '1847', 'Seizure disorder', 'Nervous system finding, Head and neck finding'),
('1848-VENOM', '1848', 'Seminoma', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding'),
('1849-VENOM', '1849', 'Sepsis', 'Infectious disorder finding'),
('1850-VENOM', '1850', 'Septic shock', 'Infectious disorder finding, Cardiovascular system finding'),
('1851-VENOM', '1851', 'Septicaemia', 'Infectious disorder finding'),
('1852-VENOM', '1852', 'Septicaemia - coliform', 'Infectious disorder finding'),
('1853-VENOM', '1853', 'Septicaemia - streptococcal', 'Infectious disorder finding'),
('1855-VENOM', '1855', 'Sertoli-cell tumour', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding'),
('10096-VENOM', '10096', 'Shar Pei Fever', 'Hereditary disorder finding'),
('1856-VENOM', '1856', 'Sialoadenitis', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding, Head and neck finding'),
('1857-VENOM', '1857', 'Sialoadenitis - submandibular gland', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding, Head and neck finding'),
('1859-VENOM', '1859', 'Sialolithiasis', 'Alimentary tract/Digestive system finding, Head and neck finding'),
('1860-VENOM', '1860', 'Sick sinus syndrome', 'Cardiovascular system finding, Thoracic finding'),
('7374-VENOM', '7374', 'Sinus arrest', 'Cardiovascular system finding, Thoracic finding'),
('7376-VENOM', '7376', 'Sinus bradycardia', 'Cardiovascular system finding, Thoracic finding'),
('1861-VENOM', '1861', 'Sinusitis', 'Respiratory system finding, Inflammatory disorder finding, Head and neck finding'),
('1862-VENOM', '1862', 'Skin (cutaneous) cyst', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('1863-VENOM', '1863', 'Skin (cutaneous) disorder', 'Haircoat/skin (+ appendages) finding'),
('1864-VENOM', '1864', 'Skin (cutaneous) disorder - acquired skin fragility', 'Haircoat/skin (+ appendages) finding'),
('1865-VENOM', '1865', 'Skin (cutaneous) disorder, bacterial', 'Haircoat/skin (+ appendages) finding, Infectious disorder finding'),
('1866-VENOM', '1866', 'Skin (cutaneous) disorder, developmental/congenital', 'Haircoat/skin (+ appendages) finding, Developmental disorder finding, Congenital finding'),
('1867-VENOM', '1867', 'Skin (cutaneous) disorder, endocrine/metabolic', 'Haircoat/skin (+ appendages) finding, Endocrine/Metabolic finding'),
('1868-VENOM', '1868', 'Skin (cutaneous) disorder, fungal', 'Haircoat/skin (+ appendages) finding, Infectious disorder finding'),
('1869-VENOM', '1869', 'Skin (cutaneous) disorder, immune-mediated', 'Haircoat/skin (+ appendages) finding, Immune system finding'),
('1870-VENOM', '1870', 'Skin (cutaneous) disorder, parasitic', 'Haircoat/skin (+ appendages) finding, Infectious disorder finding'),
('1871-VENOM', '1871', 'Skin (cutaneous) disorder, pigmentary', 'Haircoat/skin (+ appendages) finding'),
('1872-VENOM', '1872', 'Slipped capital femoral epiphysis, slipped upper femoral epiphysis', 'Musculoskeletal system finding, Limb finding'),
('1873-VENOM', '1873', 'Smoke inhalation', 'Respiratory system finding, Traumatic episode finding'),
('1874-VENOM', '1874', 'Snake bite', 'Wound finding, Traumatic episode finding, Intoxication (poisoning)/toxin exposure finding'),
('1875-VENOM', '1875', 'Spinal (vertebral column) anomaly, congenital', 'Musculoskeletal system finding, Congenital finding, Spine (vertebral column) finding'),
('1876-VENOM', '1876', 'Spinal (vertebral column) anomaly, congenital - cervical malformation/malarticulation', 'Musculoskeletal system finding, Congenital finding, Spine (vertebral column) finding, Head and neck finding'),
('1877-VENOM', '1877', 'Spinal (vertebral column) disorder', 'Musculoskeletal system finding, Spine (vertebral column) finding'),
('1878-VENOM', '1878', 'Spinal cord anomaly, congenital', 'Nervous system finding, Congenital finding'),
('1880-VENOM', '1880', 'Spinal cord disorder - vascular myelopathy', 'Nervous system finding, Cardiovascular system finding'),
('1879-VENOM', '1879', 'Spinal cord disorder (myelopathy)', 'Nervous system finding'),
('1881-VENOM', '1881', 'Spinal cord injury', 'Nervous system finding, Traumatic episode finding'),
('1882-VENOM', '1882', 'Spinal stenosis', 'Nervous system finding, Musculoskeletal system finding, Spine (vertebral column) finding'),
('1883-VENOM', '1883', 'Spinal stenosis - cervical', 'Nervous system finding, Musculoskeletal system finding, Spine (vertebral column) finding. Head and neck finding'),
('1884-VENOM', '1884', 'Spinal stenosis - lumbar', 'Nervous system finding, Musculoskeletal system finding, Spine (vertebral column) finding, Abdominal finding'),
('1885-VENOM', '1885', 'Spinal stenosis - lumbosacral', 'Nervous system finding, Musculoskeletal system finding, Spine (vertebral column) finding, Abdominal finding, Pelvic structure finding'),
('1886-VENOM', '1886', 'Spinal stenosis - thoracic', 'Nervous system finding, Musculoskeletal system finding, Spine (vertebral column) finding. Thoracic finding'),
('1887-VENOM', '1887', 'Splenic haematoma', 'Haematopoietic system finding, Cardiovascular system finding, Abdominal finding'),
('1888-VENOM', '1888', 'Splenic haemorrhage', 'Haematopoietic system finding, Cardiovascular system finding, Abdominal finding'),
('1889-VENOM', '1889', 'Splenic infarction', 'Haematopoietic system finding, Cardiovascular system finding, Abdominal finding'),
('1890-VENOM', '1890', 'Splenic rupture', 'Haematopoietic system finding, Abdominal finding'),
('1891-VENOM', '1891', 'Splenic rupture - traumatic', 'Haematopoietic system finding, Traumatic episode finding, Abdominal finding'),
('1892-VENOM', '1892', 'Splenic torsion', 'Haematopoietic system finding, Abdominal finding'),
('1893-VENOM', '1893', 'Splenitis', 'Haematopoietic system finding, Inflammatory disorder finding, Abdominal finding'),
('1894-VENOM', '1894', 'Spondylitis', 'Musculoskeletal system finding, Inflammatory disorder finding, Spine (vertebral column) finding'),
('1895-VENOM', '1895', 'Spondylosis', 'Musculoskeletal system finding, Spine (vertebral column) finding'),
('1896-VENOM', '1896', 'Spondylosis - cervical', 'Musculoskeletal system finding, Spine (vertebral column) finding. Head and neck finding'),
('1897-VENOM', '1897', 'Spondylosis - lumbar', 'Musculoskeletal system finding, Spine (vertebral column) finding, Abdominal finding'),
('1898-VENOM', '1898', 'Spondylosis - lumbosacral', 'Musculoskeletal system finding, Spine (vertebral column) finding, Abdominal finding, Pelvic structure finding'),
('1899-VENOM', '1899', 'Spondylosis - thoracic', 'Musculoskeletal system finding, Spine (vertebral column) finding, Thoracic finding'),
('1900-VENOM', '1900', 'Spondylosis with myelopathy - cervical', 'Nervous system finding, Musculoskeletal system finding, Spine (vertebral column) finding. Head and neck finding'),
('1901-VENOM', '1901', 'Spondylosis with myelopathy - lumbar', 'Nervous system finding, Musculoskeletal system finding, Spine (vertebral column) finding, Abdominal finding'),
('1902-VENOM', '1902', 'Spondylosis with myelopathy - lumbosacral', 'Nervous system finding, Musculoskeletal system finding, Spine (vertebral column) finding, Abdominal finding, Pelvic structure finding'),
('1903-VENOM', '1903', 'Spondylosis with myelopathy - thoracic', 'Nervous system finding, Musculoskeletal system finding, Spine (vertebral column) finding. Thoracic finding'),
('1905-VENOM', '1905', 'Squamous cell carcinoma - anal margin', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding'),
('1906-VENOM', '1906', 'Squamous cell carcinoma - bronchus', 'Respiratory system finding, Mass lesion/swelling finding, Thoracic finding'),
('16474-VENOM', '16474', 'Squamous cell carcinoma - cutaneous (skin)(site unspecified)', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('17325-VENOM', '17325', 'Squamous cell carcinoma - cutaneous (skin), digit', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding, Limb finding'),
('17327-VENOM', '17327', 'Squamous cell carcinoma - cutaneous (skin), ear (aural)', 'Auditory system finding, Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding, Head and neck finding'),
('1908-VENOM', '1908', 'Squamous cell carcinoma - cutaneous (skin), eyelid', 'Haircoat/skin (+ appendages) finding, Ophthalmology/vision finding, Mass lesion/swelling finding, Head and neck finding'),
('17329-VENOM', '17329', 'Squamous cell carcinoma - frontal sinus', 'Respiratory system finding, Mass lesion/swelling finding, Head and neck finding'),
('17332-VENOM', '17332', 'Squamous cell carcinoma - mammary', 'Reproductive system finding, Mass lesion/swelling finding'),
('17335-VENOM', '17335', 'Squamous cell carcinoma - nasal cavity', 'Mass lesion/swelling finding, Respiratory system finding, Head and neck finding'),
('17337-VENOM', '17337', 'Squamous cell carcinoma - nasal planum', 'Mass lesion/swelling finding, Respiratory system finding, Head and neck finding'),
('16476-VENOM', '16476', 'Squamous cell carcinoma - oral (mouth)(site unspecified)', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Head and neck finding'),
('1910-VENOM', '1910', 'Squamous cell carcinoma - oral (mouth), gingival (gum)', 'Alimentary tract/Digestive system finding, Soft tissue finding, Mass lesion/swelling finding, Head and neck finding'),
('1911-VENOM', '1911', 'Squamous cell carcinoma - oral (mouth), tongue (lingual)', 'Alimentary tract/Digestive system finding, Soft tissue finding, Mass lesion/swelling finding, Head and neck finding'),
('17339-VENOM', '17339', 'Squamous cell carcinoma - oral (mouth), tonsil', 'Mass lesion/swelling finding, Haematopoietic system finding, Immune system finding, Respiratory system finding, Alimentary tract/Digestive system finding, Head and neck finding'),
('16471-VENOM', '16471', 'Squamous cell carcinoma (site unspecified)', 'Mass lesion/swelling finding'),
('1912-VENOM', '1912', 'Starvation', 'Nutritional finding'),
('1913-VENOM', '1913', 'Status epilepticus', 'Nervous system finding, Head and neck finding'),
('1914-VENOM', '1914', 'Stenotic nares', 'Respiratory system finding, Congenital finding, Head and neck finding'),
('1915-VENOM', '1915', 'Sterile pustular dermatosis (not PF)', 'Haircoat/skin (+ appendages) finding, Immune system finding'),
('1916-VENOM', '1916', 'Steroid responsive meningitis-arteritis (SRMA)', 'Nervous system finding, Cardiovascular system finding, Inflammatory disorder finding, Immune system finding'),
('1917-VENOM', '1917', 'Stick injury', 'Wound finding, Traumatic episode finding'),
('1918-VENOM', '1918', 'Stomatitis', 'Alimentary tract/Digestive system finding, Soft tissue finding, Inflammatory disorder finding, Head and neck finding'),
('1919-VENOM', '1919', 'Stomatitis - ulcerative', 'Alimentary tract/Digestive system finding, Soft tissue finding, Inflammatory disorder finding, Head and neck finding'),
('1920-VENOM', '1920', 'Storage disease', 'Endocrine/metabolic finding, Hereditary disorder finding, Congenital finding'),
('1921-VENOM', '1921', 'Subconjunctival haemorrhage', 'Ophthalmology/vision finding, Cardiovascular system finding, Soft tissue finding, Head and neck finding'),
('1922-VENOM', '1922', 'Subdural haematoma', 'Nervous system finding, Cardiovascular system finding'),
('1924-VENOM', '1924', 'Subepidermal blistering disease', 'Haircoat/skin (+ appendages) finding, Developmental disorder finding, Congenital finding'),
('1923-VENOM', '1923', 'Subepidermal bullous disease', 'Haircoat/skin (+ appendages) finding, Immune system finding'),
('1925-VENOM', '1925', 'Subvalvular aortic stenosis (SAS)', 'Cardiovascular system finding, Thoracic finding'),
('1926-VENOM', '1926', 'Subvalvular pulmonic stenosis', 'Cardiovascular system finding, Thoracic finding'),
('1927-VENOM', '1927', 'Sudden acquired retinal degeneration (SARD)', 'Ophthalmology/vision finding, Degenerative disorder finding, Head and neck finding'),
('15948-VENOM', '15948', 'Supravalvular aortic stenosis', 'Cardiovascular system finding, Thoracic finding'),
('15946-VENOM', '15946', 'Supravalvular pulmonic stenosis', 'Cardiovascular system finding, Thoracic finding'),
('4566-VENOM', '4566', 'Supraventricular tachycardia (SVT)', 'Cardiovascular system finding, Thoracic finding'),
('1929-VENOM', '1929', 'Syncope', 'Cardiovascular system finding, Collapse/syncopal episodes finding'),
('1930-VENOM', '1930', 'Syncope - carotid sinus syncope', 'Cardiovascular system finding, Collapse/syncopal episodes finding, Nervous system finding'),
('1931-VENOM', '1931', 'Syncope - exertional', 'Cardiovascular system finding, Collapse/syncopal episodes finding'),
('1932-VENOM', '1932', 'Syncope - tussive', 'Cardiovascular system finding, Collapse/syncopal episodes finding'),
('1933-VENOM', '1933', 'Syncope - vasovagal', 'Cardiovascular system finding, Collapse/syncopal episodes finding'),
('1934-VENOM', '1934', 'Synostosis', 'Musculoskeletal system finding, Limb finding'),
('1935-VENOM', '1935', 'Synostosis - radioulnar', 'Musculoskeletal system finding, Limb finding'),
('1936-VENOM', '1936', 'Synovial cyst', 'Musculoskeletal system finding, Mass lesion/swelling finding'),
('1937-VENOM', '1937', 'Synovitis', 'Musculoskeletal system finding, Mass lesion/swelling finding'),
('1938-VENOM', '1938', 'Syringomyelia', 'Nervous system finding'),
('4418-VENOM', '4418', 'Systemic hypertension', 'Cardiovascular system finding'),
('15593-VENOM', '15593', 'Systemic hypotension', 'Cardiovascular system finding'),
('1939-VENOM', '1939', 'Systemic inflammatory response syndrome (SIRS)', 'Inflammatory disorder finding'),
('1940-VENOM', '1940', 'Systemic lupus erythematosus (SLE)', 'Immune system finding'),
('4380-VENOM', '4380', 'Systolic dysfunction', 'Cardiovascular system finding, Thoracic finding'),
('7380-VENOM', '7380', 'Tachydysrhythmia - unknown', 'Cardiovascular system finding, Thoracic finding'),
('2635-VENOM', '2635', 'Tail conformation abnormal', 'Tail finding'),
('2636-VENOM', '2636', 'Tail conformation abnormal - corkscrew tail', 'Tail finding'),
('1941-VENOM', '1941', 'Tail gland hyperplasia', 'Haircoat/skin (+ appendages) finding, Tail finding'),
('1942-VENOM', '1942', 'Tail injury', 'Tail finding, Traumatic episode finding'),
('1943-VENOM', '1943', 'Tail pull injury', 'Tail finding, Traumatic episode finding'),
('4677-VENOM', '4677', 'Taurine deficiency', 'Nutritional finding'),
('16360-VENOM', '16360', 'Temporomandibular joint (TMJ) dysplasia', 'Musculoskeletal system finding, Head and neck finding, Developmental disorder finding'),
('1945-VENOM', '1945', 'Tendinitis', 'Musculoskeletal system finding, Connective tissue finding, Inflammatory disorder finding'),
('1946-VENOM', '1946', 'Tendinitis - Achilles tendon', 'Musculoskeletal system finding, Connective tissue finding, Inflammatory disorder finding, Limb finding'),
('1947-VENOM', '1947', 'Tendinitis - biceps tendon', 'Musculoskeletal system finding, Connective tissue finding, Inflammatory disorder finding, Limb finding'),
('1948-VENOM', '1948', 'Tendon disorder', 'Musculoskeletal system finding, Connective tissue finding'),
('1949-VENOM', '1949', 'Tendon injury', 'Musculoskeletal system finding, Connective tissue finding'),
('1950-VENOM', '1950', 'Tendon laceration', 'Musculoskeletal system finding, Connective tissue finding, Wound finding, Traumatic episode finding'),
('1951-VENOM', '1951', 'Tendon rupture', 'Musculoskeletal system finding, Connective tissue finding'),
('1952-VENOM', '1952', 'Tendon rupture - Achilles tendon', 'Musculoskeletal system finding, Connective tissue finding, Limb finding'),
('1953-VENOM', '1953', 'Tendon rupture - Achilles tendon, complete', 'Musculoskeletal system finding, Connective tissue finding, Limb finding'),
('1954-VENOM', '1954', 'Tendon rupture - Achilles tendon, partial', 'Musculoskeletal system finding, Connective tissue finding, Limb finding'),
('1955-VENOM', '1955', 'Tendon rupture - biceps tendon', 'Musculoskeletal system finding, Connective tissue finding, Limb finding'),
('1956-VENOM', '1956', 'Tendon rupture - gastrocnemius tendon', 'Musculoskeletal system finding, Connective tissue finding, Limb finding'),
('1957-VENOM', '1957', 'Tendon rupture - hip', 'Musculoskeletal system finding, Connective tissue finding, Limb finding'),
('1958-VENOM', '1958', 'Tendon rupture - triceps tendon', 'Musculoskeletal system finding, Connective tissue finding, Limb finding'),
('1959-VENOM', '1959', 'Tendon rupture, complete', 'Musculoskeletal system finding, Connective tissue finding'),
('1960-VENOM', '1960', 'Tendon rupture, partial', 'Musculoskeletal system finding, Connective tissue finding'),
('1961-VENOM', '1961', 'Tenosynovitis', 'Musculoskeletal system finding, Connective tissue finding, Inflammatory disorder finding'),
('1962-VENOM', '1962', 'Tenosynovitis - bicipital', 'Musculoskeletal system finding, Connective tissue finding, Inflammatory disorder finding, Limb finding'),
('4588-VENOM', '4588', 'Teratoma', 'Mass lesion/swelling finding'),
('7409-VENOM', '7409', 'Testicular disorder', 'Urogenital system finding, Reproductive system finding'),
('1963-VENOM', '1963', 'Testicular torsion', 'Urogenital system finding, Reproductive system finding'),
('1964-VENOM', '1964', 'Tetanus', 'Infectious disorder finding, Musculoskeletal system finding'),
('18078-VENOM', '18078', 'Thalamic infarction', 'Nervous system finding, Cardiovascular system finding, Head and neck finding'),
('17341-VENOM', '17341', 'Thecoma', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding, Abdominal finding'),
('1965-VENOM', '1965', 'Thermal injury to skin (cutaneous)', 'Haircoat/skin (+ appendages) finding, Traumatic episode finding'),
('1966-VENOM', '1966', 'Thiamine deficiency', 'Nutritional finding'),
('4567-VENOM', '4567', 'Third degree AV block', 'Cardiovascular system finding, Thoracic finding'),
('1968-VENOM', '1968', 'Third eyelid/nictitating membrane disorder - inverted cartilage', 'Ophthalmology/vision finding, Soft tissue finding, Head and neck finding'),
('1969-VENOM', '1969', 'Third eyelid/nictitating membrane disorder - laceration', 'Ophthalmology/vision finding, Soft tissue finding, Wound finding, Traumatic episode finding, Head and neck finding'),
('7477-VENOM', '7477', 'Third eyelid/nictitating membrane disorder - prolapsed gland (Cherry eye)', 'Ophthalmology/vision finding, Soft tissue finding, Head and neck finding'),
('7478-VENOM', '7478', 'Third eyelid/nictitating membrane disorder - scrolled cartilage (everted)', 'Ophthalmology/vision finding, Soft tissue finding, Head and neck finding'),
('1971-VENOM', '1971', 'Thrombocytopathy', 'Haematopoietic system finding, Haemostatic system finding'),
('11017-VENOM', '11017', 'Thrombocytopenia', 'Haematopoietic system finding, Haemostatic system finding'),
('11013-VENOM', '11013', 'Thrombocytopenia – immune-mediated – primary (idiopathic, ITP)', 'Haematopoietic system finding, Haemostatic system finding, Immune system finding'),
('11014-VENOM', '11014', 'Thrombocytopenia – immune-mediated – secondary', 'Haematopoietic system finding, Haemostatic system finding, Immune system finding'),
('1974-VENOM', '1974', 'Thromboembolic disease', 'Cardiovascular system finding'),
('1975-VENOM', '1975', 'Thromboembolism', 'Cardiovascular system finding'),
('1976-VENOM', '1976', 'Thromboembolism - aortic (ATE)', 'Cardiovascular system finding'),
('1977-VENOM', '1977', 'Thromboembolism - pulmonary (PTE)', 'Cardiovascular system finding, Respiratory system finding, Thoracic finding'),
('1978-VENOM', '1978', 'Thrombosis - arterial', 'Cardiovascular system finding'),
('1979-VENOM', '1979', 'Thrombosis - portal vein', 'Cardiovascular system finding, Abdominal finding'),
('1980-VENOM', '1980', 'Thrombosis - renal artery', 'Cardiovascular system finding, Urogenital system finding, Abdominal finding'),
('1981-VENOM', '1981', 'Thrombosis - renal vein', 'Cardiovascular system finding, Urogenital system finding, Abdominal finding'),
('1982-VENOM', '1982', 'Thrombosis - splenic vein', 'Cardiovascular system finding, Haematopoietic system finding, Abdominal finding'),
('1983-VENOM', '1983', 'Thrombosis - vena cava', 'Cardiovascular system finding'),
('1984-VENOM', '1984', 'Thrombotic disorder', 'Cardiovascular system finding'),
('4145-VENOM', '4145', 'Thrombus - atrial', 'Cardiovascular system finding, Thoracic finding'),
('1985-VENOM', '1985', 'Thrombus - mural thrombus of heart', 'Cardiovascular system finding, Thoracic finding'),
('1986-VENOM', '1986', 'Thrombus, possible', 'Clinical finding'),
('16884-VENOM', '16884', 'Thymoma - epithelial', 'Endocrine/Metabolic finding, Thoracic finding'),
('16885-VENOM', '16885', 'Thymoma - lymphocytic', 'Endocrine/Metabolic finding, Thoracic finding'),
('16883-VENOM', '16883', 'Thymoma (type unspecified)', 'Endocrine/Metabolic finding, Thoracic finding'),
('1988-VENOM', '1988', 'Thyroid storm', 'Endocrine/Metabolic finding, Head and neck finding'),
('1989-VENOM', '1989', 'Thyrotoxicosis', 'Endocrine/Metabolic finding, Head and neck finding'),
('1990-VENOM', '1990', 'Tick paralysis', 'Nervous system finding, Musculoskeletal system finding, Infectious disorder finding'),
('1991-VENOM', '1991', 'Tongue (lingual) disorder', 'Alimentary tract/Digestive system finding, Soft tissue finding, Head and neck finding'),
('1992-VENOM', '1992', 'Tongue (lingual) disorder - glossitis', 'Alimentary tract/Digestive system finding, Soft tissue finding, Inflammatory disorder finding, Head and neck finding'),
('1993-VENOM', '1993', 'Tongue (lingual) injury', 'Alimentary tract/Digestive system finding, Soft tissue finding, Traumatic episode finding, Head and neck finding'),
('1994-VENOM', '1994', 'Tongue (lingual) paralysis', 'Alimentary tract/Digestive system finding, Soft tissue finding, Head and neck finding'),
('1995-VENOM', '1995', 'Tonsillar enlargement', 'Respiratory system finding, Alimentary tract/Digestive system finding, Haematopoietic system finding, Immune system finding, Lymphatic system finding, Head and neck finding'),
('1996-VENOM', '1996', 'Tonsillitis', 'Respiratory system finding, Alimentary tract/Digestive system finding, Haematopoietic system finding, Immune system finding, Lymphatic system finding, Inflammatory disorder finding, Head and neck finding'),
('2125-VENOM', '2125', 'Torovirus infection', 'Infectious disorder finding'),
('1997-VENOM', '1997', 'Toxic epidermal necrolysis', 'Haircoat/skin (+ appendages) finding, Immune system finding'),
('1998-VENOM', '1998', 'Toxoplasmosis', 'Infectious disorder finding'),
('1999-VENOM', '1999', 'Toxoplasmosis chorioretinitis', 'Ophthalmology/vision finding, Inflammatory disorder finding, Infectious disorder finding, Head and neck finding'),
('2000-VENOM', '2000', 'Tracheal avulsion', 'Respiratory system finding, Traumatic episode finding, Head and neck finding'),
('2001-VENOM', '2001', 'Tracheal collapse', 'Respiratory system finding, Head and neck finding'),
('2002-VENOM', '2002', 'Tracheal collapse, acquired', 'Respiratory system finding, Head and neck finding'),
('2003-VENOM', '2003', 'Tracheal collapse, congenital', 'Respiratory system finding, Congenital finding, Head and neck finding'),
('2004-VENOM', '2004', 'Tracheal disorder', 'Respiratory system finding, Head and neck finding'),
('2011-VENOM', '2011', 'Tracheal obstruction', 'Respiratory system finding, Head and neck finding'),
('2005-VENOM', '2005', 'Tracheal perforation', 'Respiratory system finding, Traumatic episode finding, Head and neck finding'),
('2006-VENOM', '2006', 'Tracheal perforation - iatrogenic', 'Respiratory system finding, Traumatic episode finding, Head and neck finding'),
('2007-VENOM', '2007', 'Tracheal rupture', 'Respiratory system finding, Traumatic episode finding, Head and neck finding'),
('2008-VENOM', '2008', 'Tracheal stenosis', 'Respiratory system finding, Head and neck finding'),
('2009-VENOM', '2009', 'Tracheal stenosis, acquired', 'Respiratory system finding, Head and neck finding'),
('2010-VENOM', '2010', 'Tracheal stenosis, congenital', 'Respiratory system finding, Congenital finding, Head and neck finding'),
('2012-VENOM', '2012', 'Tracheitis', 'Respiratory system finding, Inflammatory disorder finding, Head and neck finding'),
('2013-VENOM', '2013', 'Tracheobronchitis - acute', 'Respiratory system finding, Inflammatory disorder finding, Thoracic finding, Head and neck finding'),
('2014-VENOM', '2014', 'Tracheobronchitis - chronic', 'Respiratory system finding, Inflammatory disorder finding, Thoracic finding, Head and neck finding'),
('2015-VENOM', '2015', 'Traction alopecia', 'Haircoat/skin (+ appendages) finding'),
('17353-VENOM', '17353', 'Tranisitonal cell carcinoma - ureter', 'Mass lesion/swelling finding, Urogenital system finding, Abdominal finding'),
('17346-VENOM', '17346', 'Transitional cell carcinoma - nasal cavity', 'Mass lesion/swelling finding, Respiratory system finding, Head and neck finding'),
('17349-VENOM', '17349', 'Transitional cell carcinoma - prostate', 'Mass lesion/swelling finding, Urogenital system finding, Reproductive system finding, Pelvic structure finding'),
('17351-VENOM', '17351', 'Transitional cell carcinoma - renal (kidney) pelvis', 'Mass lesion/swelling finding, Urogenital system finding, Abdominal finding'),
('17355-VENOM', '17355', 'Transitional cell carcinoma - urethra', 'Mass lesion/swelling finding, Urogenital system finding'),
('17344-VENOM', '17344', 'Transitional cell carcinoma (site unspecified)', 'Mass lesion/swelling finding'),
('16890-VENOM', '16890', 'Transitional cell carcinoma of urinary bladder - infiltrating', 'Urogenital system finding, Mass lesion/swelling finding, Abdominal finding'),
('16891-VENOM', '16891', 'Transitional cell carcinoma of urinary bladder - non-infiltrating', 'Urogenital system finding, Mass lesion/swelling finding, Abdominal finding'),
('16892-VENOM', '16892', 'Transitional cell carcinoma of urinary bladder - non-papillary', 'Urogenital system finding, Mass lesion/swelling finding, Abdominal finding'),
('16893-VENOM', '16893', 'Transitional cell carcinoma of urinary bladder - papillary', 'Urogenital system finding, Mass lesion/swelling finding, Abdominal finding'),
('16888-VENOM', '16888', 'Transitional cell carcinoma of urinary bladder (type unspecified)', 'Urogenital system finding, Mass lesion/swelling finding, Abdominal finding'),
('17357-VENOM', '17357', 'Transmissible veneral tumour (TVT)', 'Haematopoietic system finding, Mass lesion/swelling finding'),
('9572-VENOM', '9572', 'Traumatic ear canal avulsion', 'Auditory system finding, Traumatic episode finding, Head and neck finding'),
('2017-VENOM', '2017', 'Traumatic injury', 'Traumatic episode finding'),
('2018-VENOM', '2018', 'Trichiasis', 'Haircoat/skin (+ appendages) finding, Ophthalmology/vision finding, Head and neck finding'),
('2019-VENOM', '2019', 'Trichobezoar', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('2020-VENOM', '2020', 'Trichoepithelioma', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('17359-VENOM', '17359', 'Trichofolliculoma', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('17361-VENOM', '17361', 'Tricholemmoma', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding'),
('2021-VENOM', '2021', 'Tricuspid valve disorder', 'Cardiovascular system finding, Thoracic finding'),
('2022-VENOM', '2022', 'Tricuspid valve dysplasia', 'Cardiovascular system finding, Congenital finding, Thoracic finding'),
('2023-VENOM', '2023', 'Tricuspid valve prolapse', 'Cardiovascular system finding, Thoracic finding'),
('2024-VENOM', '2024', 'Tricuspid valve regurgitation/insufficiency', 'Cardiovascular system finding, Thoracic finding'),
('2025-VENOM', '2025', 'Tricuspid valve stenosis', 'Cardiovascular system finding, Thoracic finding'),
('2026-VENOM', '2026', 'Trigeminal neuropathy', 'Nervous system finding, Head and neck finding'),
('2027-VENOM', '2027', 'Trigeminal neuropathy, idiopathic', 'Nervous system finding, Head and neck finding'),
('2028-VENOM', '2028', 'Tumour lysis syndrome', 'Endocrine/Metabolic finding'),
('2029-VENOM', '2029', 'Upper motor neuron disease', 'Nervous system finding'),
('2030-VENOM', '2030', 'Ureteric anomaly, congenital', 'Urogenital system finding, Congenital finding, Abdominal finding'),
('2031-VENOM', '2031', 'Ureteric anomaly, congenital - ectopic ureter', 'Urogenital system finding, Congenital finding, Abdominal finding'),
('2034-VENOM', '2034', 'Ureteric obstruction', 'Urogenital system finding, Abdominal finding'),
('2032-VENOM', '2032', 'Ureteric rupture', 'Urogenital system finding, Abdominal finding'),
('2033-VENOM', '2033', 'Ureteric stricture', 'Urogenital system finding, Abdominal finding'),
('2035-VENOM', '2035', 'Urethral obstruction', 'Urogenital system finding'),
('7708-VENOM', '7708', 'Urethral prolapse', 'Urogenital system finding'),
('2036-VENOM', '2036', 'Urethral rupture', 'Urogenital system finding'),
('2037-VENOM', '2037', 'Urethral rupture - iatrogenic', 'Urogenital system finding'),
('2038-VENOM', '2038', 'Urethral stricture', 'Urogenital system finding'),
('2039-VENOM', '2039', 'Urethritis', 'Urogenital system finding, Inflammatory disorder finding'),
('10099-VENOM', '10099', 'Urinary bladder diverticulum', 'Urogenital system finding, Abdominal finding'),
('2045-VENOM', '2045', 'Urinary bladder rupture', 'Urogenital system finding, Abdominal finding'),
('2040-VENOM', '2040', 'Urinary bladder, functional disorder', 'Urogenital system finding, Abdominal finding'),
('2041-VENOM', '2041', 'Urinary bladder, functional disorder - atony', 'Urogenital system finding, Abdominal finding'),
('2042-VENOM', '2042', 'Urinary bladder, functional disorder - neurogenic dysfunction', 'Urogenital system finding, Nervous system finding, Abdominal finding'),
('2043-VENOM', '2043', 'Urinary bladder, functional disorder of bladder neck/sphincter mechanism', 'Urogenital system finding, Abdominal finding'),
('2044-VENOM', '2044', 'Urinary bladder, intra-pelvic position', 'Urogenital system finding, Abdominal finding'),
('2046-VENOM', '2046', 'Urinary incontinence', 'Urogenital system finding'),
('2047-VENOM', '2047', 'Urinary incontinence - overflow', 'Urogenital system finding'),
('2048-VENOM', '2048', 'Urinary incontinence - post-micturition', 'Urogenital system finding'),
('2049-VENOM', '2049', 'Urinary incontinence due to urethral sphincter mechanism incompetence (USMI)', 'Urogenital system finding'),
('2050-VENOM', '2050', 'Urinary retention', 'Urogenital system finding, Abdominal finding'),
('2051-VENOM', '2051', 'Urinary tract infection (UTI)', 'Urogenital system finding, Infectious disorder finding'),
('2052-VENOM', '2052', 'Urinary tract infection (UTI) - bacterial', 'Urogenital system finding, Infectious disorder finding'),
('2053-VENOM', '2053', 'Urinary tract infection (UTI) - bacterial, coliform', 'Urogenital system finding, Infectious disorder finding'),
('2054-VENOM', '2054', 'Urinary tract infection (UTI), chronic', 'Urogenital system finding, Infectious disorder finding'),
('2056-VENOM', '2056', 'Urinary tract infection (UTI), recurrent', 'Urogenital system finding, Infectious disorder finding'),
('2055-VENOM', '2055', 'Urinary tract infection, lower (LUTI)', 'Urogenital system finding, Infectious disorder finding'),
('9491-VENOM', '9491', 'Urinary tract rupture', 'Urogenital system finding'),
('2057-VENOM', '2057', 'Urolithiasis', 'Urogenital system finding'),
('2058-VENOM', '2058', 'Urolithiasis - renal (nephrolithiasis)', 'Urogenital system finding, Abdominal finding'),
('2059-VENOM', '2059', 'Urolithiasis - ureteric', 'Urogenital system finding, Abdominal finding'),
('2060-VENOM', '2060', 'Urolithiasis - urethral', 'Urogenital system finding'),
('2061-VENOM', '2061', 'Urolithiasis - urinary bladder', 'Urogenital system finding, Abdominal finding'),
('2062-VENOM', '2062', 'Urolithiasis/crystalluria - calcium oxalate', 'Urogenital system finding'),
('2063-VENOM', '2063', 'Urolithiasis/crystalluria - cystine', 'Urogenital system finding'),
('2064-VENOM', '2064', 'Urolithiasis/crystalluria - magnesium-ammonium-phosphate hexahydrate (struvite)', 'Urogenital system finding'),
('2065-VENOM', '2065', 'Urolithiasis/crystalluria - urate', 'Urogenital system finding'),
('2066-VENOM', '2066', 'Uveitis', 'Ophthalmology/vision finding, Inflammatory disorder finding, Head and neck finding'),
('2067-VENOM', '2067', 'Uveitis - anterior', 'Ophthalmology/vision finding, Inflammatory disorder finding, Head and neck finding'),
('2068-VENOM', '2068', 'Uveitis - idiopathic', 'Ophthalmology/vision finding, Inflammatory disorder finding, Head and neck finding'),
('2069-VENOM', '2069', 'Uveitis - lens-induced', 'Ophthalmology/vision finding, Inflammatory disorder finding, Head and neck finding'),
('2070-VENOM', '2070', 'Uveitis - posterior', 'Ophthalmology/vision finding, Inflammatory disorder finding, Head and neck finding'),
('2071-VENOM', '2071', 'Uveitis - traumatic', 'Ophthalmology/vision finding, Inflammatory disorder finding, Traumatic episode finding, Head and neck finding'),
('2072-VENOM', '2072', 'Vaginal disorder', 'Urogenital system finding, Reproductive system finding, Pelvic structure finding'),
('2073-VENOM', '2073', 'Vaginal hyperplasia/prolapse', 'Urogenital system finding, Reproductive system finding, Pelvic structure finding'),
('2074-VENOM', '2074', 'Vaginitis', 'Urogenital system finding, Reproductive system finding, Inflammatory disorder finding, Pelvic structure finding'),
('2075-VENOM', '2075', 'Vaginitis, Juvenile', 'Urogenital system finding, Reproductive system finding, Inflammatory disorder finding, Pelvic structure finding'),
('2077-VENOM', '2077', 'Vascular ring anomaly (aortic arch)', 'Cardiovascular system finding, Congenital finding, Thoracic finding'),
('2076-VENOM', '2076', 'Vasculitis/vasculopathy', 'Cardiovascular system finding, Inflammatory disorder finding'),
('18112-VENOM', '18112', 'Vena caval obstruction due to anterior mediastinal mass', 'Cardiovascular system finding, Mass lesion/swelling finding, Thoracic finding'),
('18110-VENOM', '18110', 'Vena caval obstruction due to fibrosis', 'Cardiovascular system finding'),
('18114-VENOM', '18114', 'Vena caval obstruction due to other cause', 'Cardiovascular system finding'),
('18108-VENOM', '18108', 'Vena caval obstruction, Dirofilarial', 'Cardiovascular system finding, Infectious disorder finding'),
('18118-VENOM', '18118', 'Vena caval syndrome due to anterior mediastinal mass', 'Cardiovascular system finding, Mass lesion/swelling finding, Thoracic finding'),
('18116-VENOM', '18116', 'Vena caval syndrome due to fibrosis', 'Cardiovascular system finding'),
('18120-VENOM', '18120', 'Vena caval syndrome due to other cause', 'Cardiovascular system finding'),
('2078-VENOM', '2078', 'Vena caval syndrome, Dirofilarial', 'Infectious disorder finding, Cardiovascular system finding'),
('2079-VENOM', '2079', 'Ventilatory failure', 'Respiratory system finding'),
('2080-VENOM', '2080', 'Ventral comedo syndrome', 'Haircoat/skin (+ appendages) finding'),
('2081-VENOM', '2081', 'Ventricular outflow tract obstruction - left (LVOTO)', 'Cardiovascular system finding, Thoracic finding'),
('2082-VENOM', '2082', 'Ventricular outflow tract obstruction - right (RVOTO)', 'Cardiovascular system finding, Thoracic finding'),
('4443-VENOM', '4443', 'Ventricular outflow tract obstruction, left (LVOTO) - dynamic', 'Cardiovascular system finding, Thoracic finding'),
('4500-VENOM', '4500', 'Ventricular outflow tract obstruction, right (RVOTO) - dynamic', 'Cardiovascular system finding, Thoracic finding'),
('4568-VENOM', '4568', 'Ventricular Premature Complexes (VPCs)', 'Cardiovascular system finding, Thoracic finding'),
('4569-VENOM', '4569', 'Ventricular tachycardia (VT)', 'Cardiovascular system finding, Thoracic finding'),
('2083-VENOM', '2083', 'Vestibular disease', 'Nervous system finding, Head and neck finding'),
('2084-VENOM', '2084', 'Vestibular disease - central', 'Nervous system finding, Head and neck finding'),
('2085-VENOM', '2085', 'Vestibular disease - peripheral', 'Nervous system finding, Auditory system finding, Head and neck finding'),
('9475-VENOM', '9475', 'Viral papillomatosis', 'Infectious disorder finding, Mass lesion/swelling finding'),
('2086-VENOM', '2086', 'Vision impaired', 'Ophthalmology/vision finding, Head and neck finding'),
('2087-VENOM', '2087', 'Vitamin A-responsive dermatosis', 'Haircoat/skin (+ appendages) finding'),
('2088-VENOM', '2088', 'Vitamin E deficiency', 'Nutritional finding'),
('2089-VENOM', '2089', 'Vogt-Koyanagi-Harada-like syndrome (Uveodermatologic syndrome)', 'Haircoat/skin (+ appendages) finding, Immune system finding, Ophthalmology/vision finding, Inflammatory disorder finding, Head and neck finding'),
('17963-VENOM', '17963', 'Vomiting during/post-anaesthesia', 'Alimentary tract/Digestive system finding, Complication of medical care finding, Anaesthesia finding'),
('2090-VENOM', '2090', 'Von Willebrand disorder', 'Haemostatic system finding'),
('2091-VENOM', '2091', 'Vulval disorder', 'Urogenital system finding, Reproductive system finding'),
('2092-VENOM', '2092', 'Wandering pacemaker', 'Cardiovascular system finding, Thoracic finding'),
('2093-VENOM', '2093', 'Wound', 'Wound finding'),
('2094-VENOM', '2094', 'Wound - abrasion', 'Wound finding, Traumatic episode finding'),
('2095-VENOM', '2095', 'Wound - degloving injury', 'Wound finding, Traumatic episode finding'),
('2096-VENOM', '2096', 'Wound - laceration', 'Wound finding, Traumatic episode finding'),
('2097-VENOM', '2097', 'Wound - penetrating injury', 'Wound finding, Traumatic episode finding'),
('2098-VENOM', '2098', 'Wound - puncture', 'Wound finding, Traumatic episode finding'),
('2099-VENOM', '2099', 'Wound - shear injury', 'Wound finding, Traumatic episode finding'),
('9595-VENOM', '9595', 'Wound necrosis', 'Wound finding'),
('2127-VENOM', '2127', 'Wound, infected', 'Wound finding, Infectious disorder finding'),
('2100-VENOM', '2100', 'Zinc-responsive dermatosis', 'Haircoat/skin (+ appendages) finding, Endocrine/Metabolic finding'),
('2101-VENOM', '2101', 'Zollinger-Ellison syndrome', 'Alimentary tract/Digestive system finding, Endocrine/Metabolic finding, Abdominal finding'),
('5780-VENOM', '5780', 'Diarrhoea - presumed self-limiting', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('5786-VENOM', '5786', 'Muscle strain', 'Musculoskeletal system finding, Traumatic episode finding'),
('5782-VENOM', '5782', 'Vomiting - presumed self-limiting', 'Alimentary tract/Digestive system finding'),
('5790-VENOM', '5790', 'Wart', 'Haircoat/skin (+ appendages) finding, Mass lesion/swelling finding');
INSERT INTO lookups (version, linkId, arch_short_name, active, arch_version, code, name, description, default_lookup)
  SELECT
    0,
    UUID(),
    'lookup.diagnosisVeNom',
    1,
    '1.0',
    d.code,
    d.term,
    d.modelling,
    0
  FROM venom_diagnosis d
  WHERE NOT exists(SELECT
                     *
                   FROM lookups e
                   WHERE e.arch_short_name = 'lookup.diagnosisVeNom' AND e.code = d.code);
INSERT INTO lookup_details (lookup_id, name, type, value)
  SELECT
    l.lookup_id,
    'dataDictionaryId',
    'string',
    d.dictionaryId
  FROM lookups l, venom_diagnosis d
  WHERE l.arch_short_name = 'lookup.diagnosisVeNom' AND l.code = d.code
        AND NOT exists(SELECT
                         *
                       FROM lookup_details d
                       WHERE d.lookup_id = l.lookup_id AND d.name = 'dataDictionaryId');
DROP TABLE venom_diagnosis;

#
# Add VeNom presenting complaint codes for OVPMS-1468
#

DROP TABLE IF EXISTS venom_presenting_complaint;
CREATE TABLE venom_presenting_complaint (
  code         VARCHAR(255) PRIMARY KEY,
  dictionaryId VARCHAR(255),
  term         VARCHAR(255),
  modelling    VARCHAR(255));
INSERT INTO venom_presenting_complaint (code, dictionaryId, term, modelling) VALUES
('249-VENOM', '249', 'Abdominal distension', 'Abdominal finding'),
('7538-VENOM', '7538', 'Abdominal injury', 'Abdominal finding, Traumatic episode finding'),
('13-VENOM', '13', 'Alopecia', 'Haircoat/skin (+ appendages) finding'),
('11101-VENOM', '11101', 'Anal irritation', 'Alimentary tract/Digestive system finding'),
('312-VENOM', '312', 'Anorexia', 'Nervous system finding'),
('178-VENOM', '178', 'Appetite decreased', 'Nervous system finding'),
('179-VENOM', '179', 'Appetite increased', 'Nervous system finding'),
('16-VENOM', '16', 'Behavioural abnormality', 'Nervous system finding, Head and neck finding'),
('18-VENOM', '18', 'Bleeding - bruising', 'Cardiovascular system finding, Wound finding'),
('2102-VENOM', '2102', 'Bleeding - epistaxis', 'Respiratory system finding, Cardiovascular system finding, Head and neck finding'),
('2104-VENOM', '2104', 'Bleeding - other orifice', 'Cardiovascular system finding'),
('2106-VENOM', '2106', 'Bleeding - petechiation/ecchymoses', 'Cardiovascular system finding'),
('20-VENOM', '20', 'Blindness', 'Ophthalmology/vision finding, Head and neck finding'),
('7711-VENOM', '7711', 'BLOOD/OTHER TEST APPOINTMENT', 'Clinical finding'),
('2108-VENOM', '2108', 'Cachexia', 'Weight/Body condition finding, Nutritional finding'),
('2117-VENOM', '2117', 'Cardiac (heart) abnormality - heart murmur', 'Cardiovascular system finding, Thoracic finding'),
('22-VENOM', '22', 'Cardiac (heart) abnormality - other', 'Cardiovascular system finding, Thoracic finding'),
('38-VENOM', '38', 'Cataract', 'Ophthalmology/vision finding, Head and neck finding'),
('4672-VENOM', '4672', 'CHEMOTHERAPY appointment', 'Clinical finding'),
('2132-VENOM', '2132', 'Collapse/syncopal episodes', 'Clinical finding'),
('7580-VENOM', '7580', 'Corneal abnormality', 'Ophthalmology/vision finding, Head and neck finding'),
('2134-VENOM', '2134', 'Corneal opacity or blue eye', 'Ophthalmology/vision finding, Head and neck finding'),
('29-VENOM', '29', 'Coughing', 'Respiratory system finding'),
('7634-VENOM', '7634', 'Cyanosis', 'Soft tissue finding, Haircoat/skin (+ appendages) finding'),
('7630-VENOM', '7630', 'Dead (Deceased) on presentation', 'Administrative statuses finding'),
('183-VENOM', '183', 'Defaecation abnormal - decreased frequency', 'Alimentary tract/Digestive system finding'),
('185-VENOM', '185', 'Defaecation abnormal - defaecatory tenesmus', 'Alimentary tract/Digestive system finding'),
('184-VENOM', '184', 'Defaecation abnormal - increased frequency', 'Alimentary tract/Digestive system finding'),
('186-VENOM', '186', 'Defaecation abnormal - other', 'Alimentary tract/Digestive system finding'),
('2137-VENOM', '2137', 'Dental (tooth) abnormality', 'Alimentary tract/Digestive system finding, Head and neck finding'),
('2141-VENOM', '2141', 'Dietary indiscretion - foreign body ingestion', 'Alimentary tract/Digestive system finding, Foreign body finding'),
('2144-VENOM', '2144', 'Dietary indiscretion - other', 'Alimentary tract/Digestive system finding'),
('7516-VENOM', '7516', 'Discharge - aural (ear)', 'Auditory system finding, Head and neck finding'),
('36-VENOM', '36', 'Discharge - nasal (nose)', 'Respiratory system finding, Head and neck finding'),
('43-VENOM', '43', 'Discharge - ocular (eye)', 'Ophthalmology/vision finding, Head and neck finding'),
('7615-VENOM', '7615', 'Discharge - penile', 'Urogenital system finding, Reproductive system finding'),
('2145-VENOM', '2145', 'Discharge - sinus tract', 'Soft tissue finding'),
('46-VENOM', '46', 'Discharge - vaginal', 'Urogenital system finding, Reproductive system finding, Pelvic structure finding'),
('11723-VENOM', '11723', 'Dressing-related complication', 'Episode of care finding'),
('2149-VENOM', '2149', 'Dry eye', 'Ophthalmology/vision finding, Head and neck finding'),
('2151-VENOM', '2151', 'Dysphagia', 'Alimentary tract/Digestive system finding'),
('49-VENOM', '49', 'Dysphonia', 'Respiratory system finding'),
('55-VENOM', '55', 'Dyspnoea - with coughing', 'Respiratory system finding'),
('51-VENOM', '51', 'Dyspnoea - without coughing', 'Respiratory system finding'),
('57-VENOM', '57', 'Dystocia', 'Reproductive system finding'),
('2153-VENOM', '2153', 'Ear (aural) abnormality', 'Auditory system finding, Head and neck finding'),
('7520-VENOM', '7520', 'Ear (aural) injury', 'Auditory system finding, Traumatic episode finding, Head and neck finding'),
('7528-VENOM', '7528', 'Ear (aural) smell', 'Auditory system finding, Head and neck finding'),
('18317-VENOM', '18317', 'Epiphora/tear staining', 'Ophthalmology/vision finding, Head and neck finding'),
('18320-VENOM', '18320', 'Excess tear production', 'Ophthalmology/vision finding, Head and neck finding'),
('59-VENOM', '59', 'Exercise intolerance - episodic', 'Musculoskeletal system finding'),
('2158-VENOM', '2158', 'Exercise intolerance - persistent', 'Musculoskeletal system finding'),
('2160-VENOM', '2160', 'Eyelid abnormality', 'Ophthalmology/vision finding, Head and neck finding'),
('32-VENOM', '32', 'Faecal appearance abnormal - diarrhoea', 'Alimentary tract/Digestive system finding, Abdominal finding'),
('2162-VENOM', '2162', 'Faecal appearance abnormal - haematochezia', 'Alimentary tract/Digestive system finding'),
('2164-VENOM', '2164', 'Faecal appearance abnormal - melaena', 'Alimentary tract/Digestive system finding'),
('187-VENOM', '187', 'Faecal appearance abnormal - other', 'Alimentary tract/Digestive system finding'),
('188-VENOM', '188', 'Failure to gain weight', 'Weight/Body condition finding'),
('2166-VENOM', '2166', 'Failure to grow', 'Height/growth finding'),
('7612-VENOM', '7612', 'Female reproductive abnormality', 'Reproductive system finding'),
('2169-VENOM', '2169', 'Gagging/retching', 'Alimentary tract/Digestive system finding'),
('62-VENOM', '62', 'Gait abnormality - ataxia', 'Musculoskeletal system finding, Nervous system finding'),
('65-VENOM', '65', 'Gait abnormality - lameness', 'Musculoskeletal system finding, Nervous system finding'),
('67-VENOM', '67', 'Gait abnormality - other', 'Musculoskeletal system finding, Nervous system finding'),
('7619-VENOM', '7619', 'Haemoptysis', 'Respiratory system finding'),
('7604-VENOM', '7604', 'Halitosis', 'Alimentary tract/Digestive system finding, Head and neck finding'),
('2176-VENOM', '2176', 'Head tilt', 'Nervous system finding, Head and neck finding'),
('2172-VENOM', '2172', 'Hearing impaired/deafness', 'Auditory system finding, Head and neck finding'),
('18031-VENOM', '18031', 'Hyphaema', 'Ophthalmology/vision finding, Cardiovascular system finding, Head and neck finding'),
('2297-VENOM', '2297', 'Hypothermia', 'Body temperature finding'),
('7637-VENOM', '7637', 'Icterus (jaundice)', 'Endocrine/Metabolic finding'),
('2180-VENOM', '2180', 'Incontinence - faecal', 'Alimentary tract/Digestive system finding'),
('2182-VENOM', '2182', 'Incontinence - urinary', 'Urogenital system finding'),
('71-VENOM', '71', 'Intoxication (poisoning)/toxin exposure', 'Clinical finding'),
('18323-VENOM', '18323', 'Iris abnormality - mass', 'Ophthalmology/vision finding, Mass lesion/swelling finding, Head and neck finding'),
('18328-VENOM', '18328', 'Iris abnormality - other', 'Ophthalmology/vision finding, Head and neck finding'),
('18334-VENOM', '18334', 'Lens abnormality - other', 'Ophthalmology/vision finding, Head and neck finding'),
('18332-VENOM', '18332', 'Lens opacity', 'Ophthalmology/vision finding, Head and neck finding'),
('2187-VENOM', '2187', 'Lethargy', 'Musculoskeletal system finding'),
('2178-VENOM', '2178', 'Loss of balance', 'Nervous system finding'),
('2189-VENOM', '2189', 'Lymphadenomegaly', 'Lymphatic system finding, Immune system finding, Haematopoietic system finding'),
('7639-VENOM', '7639', 'Male reproductive abnormality', 'Reproductive system finding'),
('2193-VENOM', '2193', 'Mass/swelling - abdominal', 'Mass lesion/swelling finding, Abdominal finding'),
('17363-VENOM', '17363', 'Mass/swelling - axilla', 'Mass lesion/swelling finding, Thoracic finding'),
('7532-VENOM', '7532', 'Mass/swelling - ear (aural)', 'Auditory system finding, Mass lesion/swelling finding, Head and neck finding'),
('7594-VENOM', '7594', 'Mass/swelling - eye (ophthalmic)', 'Ophthalmology/vision finding, Mass lesion/swelling finding, Head and neck finding'),
('17366-VENOM', '17366', 'Mass/swelling - eyelid', 'Ophthalmology/vision finding, Mass lesion/swelling finding, Head and neck finding'),
('17368-VENOM', '17368', 'Mass/swelling - facial (site unspecified)', 'Mass lesion/swelling finding, Head and neck finding'),
('17370-VENOM', '17370', 'Mass/swelling - facial, lip', 'Alimentary tract/Digestive system finding, Soft tissue finding, Mass lesion/swelling finding, Head and neck finding'),
('17372-VENOM', '17372', 'Mass/swelling - gingival (gum)', 'Alimentary tract/Digestive system finding, Soft tissue finding, Mass lesion/swelling finding, Head and neck finding'),
('17373-VENOM', '17373', 'Mass/swelling - jaw (site unspecified)', 'Musculoskeletal system finding, Mass lesion/swelling finding, Head and neck finding'),
('17375-VENOM', '17375', 'Mass/swelling - jaw, mandible', 'Musculoskeletal system finding, Mass lesion/swelling finding, Head and neck finding'),
('17376-VENOM', '17376', 'Mass/swelling - jaw, maxilla', 'Musculoskeletal system finding, Mass lesion/swelling finding, Head and neck finding'),
('17377-VENOM', '17377', 'Mass/swelling - joint', 'Musculoskeletal system finding, Mass lesion/swelling finding'),
('17378-VENOM', '17378', 'Mass/swelling - limb, lower', 'Limb finding, Mass lesion/swelling finding'),
('17380-VENOM', '17380', 'Mass/swelling - limb, upper', 'Limb finding, Mass lesion/swelling finding'),
('17382-VENOM', '17382', 'Mass/swelling - lymph node', 'Lymphatic system finding, Immune system finding, Haematopoietic system finding, Mass lesion/swelling finding'),
('17384-VENOM', '17384', 'Mass/swelling - mammary', 'Reproductive system finding, Mass lesion/swelling finding'),
('7562-VENOM', '7562', 'Mass/swelling - musculoskeletal', 'Musculoskeletal system finding, Mass lesion/swelling finding'),
('17385-VENOM', '17385', 'Mass/swelling - neck (cervical)', 'Mass lesion/swelling finding, Head and neck finding'),
('17390-VENOM', '17390', 'Mass/swelling - nose (nasal), nasal planum', 'Respiratory system finding, Mass lesion/swelling finding, Head and neck finding'),
('2198-VENOM', '2198', 'Mass/swelling - oral (mouth)', 'Alimentary tract/Digestive system finding, Mass lesion/swelling finding, Head and neck finding'),
('2202-VENOM', '2202', 'Mass/swelling - other', 'Mass lesion/swelling finding'),
('17391-VENOM', '17391', 'Mass/swelling - penile', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding'),
('7535-VENOM', '7535', 'Mass/swelling - perianal', 'Mass lesion/swelling finding, Perianal finding'),
('17392-VENOM', '17392', 'Mass/swelling - prepuce', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding'),
('17393-VENOM', '17393', 'Mass/swelling - scrotal', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding'),
('17394-VENOM', '17394', 'Mass/swelling - skin (cutaneous)/subcutaneous (site unspecified)', 'Haircoat/skin (+ appendages) finding, Soft tissue finding, Mass lesion/swelling finding'),
('17397-VENOM', '17397', 'Mass/swelling - skin (cutaneous)/subcutaneous, digit', 'Haircoat/skin (+ appendages) finding, Soft tissue finding, Mass lesion/swelling finding, Limb finding'),
('17399-VENOM', '17399', 'Mass/swelling - skin (cutaneous)/subcutaneous, perianal', 'Haircoat/skin (+ appendages) finding, Soft tissue finding, Mass lesion/swelling finding, Perineal finding'),
('17401-VENOM', '17401', 'Mass/swelling - testicular', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding'),
('17402-VENOM', '17402', 'Mass/swelling - tongue (lingual)', 'Alimentary tract/Digestive system finding, Soft tissue finding, Mass lesion/swelling finding, Head and neck finding'),
('17403-VENOM', '17403', 'Mass/swelling - vulval', 'Urogenital system finding, Reproductive system finding, Mass lesion/swelling finding'),
('72-VENOM', '72', 'Mentation altered - anxious/distressed', 'Nervous system finding'),
('2204-VENOM', '2204', 'Mentation altered - comatose', 'Nervous system finding'),
('2206-VENOM', '2206', 'Mentation altered - depressed', 'Nervous system finding'),
('7577-VENOM', '7577', 'Mentation altered - stuporous', 'Nervous system finding'),
('7542-VENOM', '7542', 'Moribund', 'Clinical finding'),
('2212-VENOM', '2212', 'Mucous membranes abnormal - cyanosed', 'Soft tissue finding, Haircoat/skin (+ appendages) finding'),
('2217-VENOM', '2217', 'Mucous membranes abnormal - hyperaemic', 'Soft tissue finding, Cardiovascular system finding'),
('75-VENOM', '75', 'Mucous membranes abnormal - other', 'Soft tissue finding'),
('74-VENOM', '74', 'Mucous membranes abnormal - pale', 'Soft tissue finding'),
('2221-VENOM', '2221', 'Muscular atrophy - head', 'Musculoskeletal system finding, Head and neck finding'),
('2224-VENOM', '2224', 'Muscular atrophy - limb', 'Musculoskeletal system finding, Limb finding'),
('2227-VENOM', '2227', 'Muscular atrophy - other', 'Musculoskeletal system finding'),
('7565-VENOM', '7565', 'Musculoskeletal injury', 'Musculoskeletal system finding, Traumatic episode finding'),
('4719-VENOM', '4719', 'NOT PRESENTED FOR A COMPLAINT', 'Clinical finding'),
('2228-VENOM', '2228', 'Nystagmus', 'Ophthalmology/vision finding, Nervous system finding, Head and neck finding'),
('2230-VENOM', '2230', 'Obesity', 'Weight/Body condition finding'),
('76-VENOM', '76', 'Ophthalmic (eye) abnormality', 'Ophthalmology/vision finding, Head and neck finding'),
('7583-VENOM', '7583', 'Ophthalmic (eye) injury', 'Ophthalmology/vision finding, Traumatic episode finding, Head and neck finding'),
('2236-VENOM', '2236', 'Oral (mouth) abnormality - other', 'Alimentary tract/Digestive system finding, Head and neck finding'),
('2233-VENOM', '2233', 'Oral (mouth) abnormality - ulceration', 'Alimentary tract/Digestive system finding, Inflammatory disorder finding, Head and neck finding'),
('7608-VENOM', '7608', 'Oral (mouth) injury', 'Alimentary tract/Digestive system finding, Traumatic episode finding, Head and neck finding'),
('2898-VENOM', '2898', 'Pain - abdominal', 'Nervous system finding, Abdominal finding'),
('7524-VENOM', '7524', 'Pain - aural (ear)', 'Nervous system finding, Auditory system finding, Head and neck finding'),
('7569-VENOM', '7569', 'Pain - musculoskeletal', 'Nervous system finding, Musculoskeletal system finding'),
('2899-VENOM', '2899', 'Pain - neck (cervical)', 'Nervous system finding, Head and neck finding'),
('2238-VENOM', '2238', 'Pain - on being handled', 'Nervous system finding'),
('2240-VENOM', '2240', 'Pain - on eating/chewing', 'Nervous system finding'),
('7587-VENOM', '7587', 'Pain - ophthalmic (eye)', 'Nervous system finding, Ophthalmology/vision finding, Head and neck finding'),
('7642-VENOM', '7642', 'Pain - oral (mouth)', 'Nervous system finding, Alimentary tract/Digestive system finding, Head and neck finding'),
('77-VENOM', '77', 'Pain - other', 'Nervous system finding'),
('2900-VENOM', '2900', 'Pain - spinal', 'Nervous system finding, Musculoskeletal system finding, Spine (vertebral column) finding'),
('79-VENOM', '79', 'Panting', 'Respiratory system finding'),
('2242-VENOM', '2242', 'Paresis/paralysis', 'Nervous system finding, Musculoskeletal system finding, Limb finding'),
('2245-VENOM', '2245', 'Paronychia', 'Haircoat/skin (+ appendages) finding, Inflammatory disorder finding, Limb finding'),
('2252-VENOM', '2252', 'Penile/preputial abnormality – other', 'Urogenital system finding, Reproductive system finding'),
('2248-VENOM', '2248', 'Penile/preputial abnormality – paraphimosis', 'Urogenital system finding, Reproductive system finding'),
('2253-VENOM', '2253', 'Polydipsia', 'Nervous system finding'),
('2255-VENOM', '2255', 'Polyphagia/ravenous appetite', 'Nervous system finding'),
('194-VENOM', '194', 'Polyuria', 'Urogenital system finding'),
('81-VENOM', '81', 'Polyuria/polydipsia', 'Urogenital system finding, Nervous system finding'),
('7505-VENOM', '7505', 'Poor quality of life', 'Clinical finding'),
('2262-VENOM', '2262', 'Post-operative complication - other', 'Complication of medical care finding'),
('2259-VENOM', '2259', 'Post-operative complication - wound related', 'Complication of medical care finding, Wound finding'),
('7558-VENOM', '7558', 'Posture abnormal - low head carriage', 'Musculoskeletal system finding, Head and neck finding'),
('7554-VENOM', '7554', 'Posture abnormal - other', 'Musculoskeletal system finding'),
('15541-VENOM', '15541', 'Presented for investigation of abnormal test result', 'Clinical finding'),
('15537-VENOM', '15537', 'Presented for radioactive treatment', 'Clinical finding'),
('15540-VENOM', '15540', 'Presented for specific therapeutic intervention', 'Clinical finding'),
('4670-VENOM', '4670', 'Presenting complaint not listed', 'Clinical finding'),
('84-VENOM', '84', 'Pruritus', 'Haircoat/skin (+ appendages) finding, Nervous system finding'),
('2263-VENOM', '2263', 'Pupillary abnormality - anisocoria', 'Ophthalmology/vision finding, Nervous system finding, Head and neck finding'),
('2267-VENOM', '2267', 'Pupillary abnormality - other', 'Ophthalmology/vision finding, Head and neck finding'),
('2294-VENOM', '2294', 'Pyrexia/hyperthermia', 'Body temperature finding'),
('2268-VENOM', '2268', 'Recumbency/reluctance to move', 'Musculoskeletal system finding, Nervous system finding'),
('2270-VENOM', '2270', 'Red eye', 'Ophthalmology/vision finding, Head and neck finding'),
('4671-VENOM', '4671', 'RE-EXAMINATION appointment', 'Clinical finding'),
('86-VENOM', '86', 'Regurgitation', 'Alimentary tract/Digestive system finding'),
('88-VENOM', '88', 'Respiratory noise increased', 'Respiratory system finding'),
('90-VENOM', '90', 'Salivation - increased/drooling', 'Alimentary tract/Digestive system finding, Head and neck finding'),
('93-VENOM', '93', 'Seizure(s)', 'Nervous system finding, Head and neck finding'),
('2275-VENOM', '2275', 'Skeletal abnormality - acquired', 'Musculoskeletal system finding'),
('2279-VENOM', '2279', 'Skeletal abnormality - congenital', 'Musculoskeletal system finding, Congenital finding'),
('2281-VENOM', '2281', 'Skin (cutaneous) abnormality – eruptions/hives/rash', 'Haircoat/skin (+ appendages) finding'),
('2283-VENOM', '2283', 'Skin (cutaneous) abnormality – other', 'Haircoat/skin (+ appendages) finding'),
('96-VENOM', '96', 'Sneezing', 'Respiratory system finding, Head and neck finding'),
('7624-VENOM', '7624', 'Stertor (snoring)', 'Respiratory system finding, Head and neck finding'),
('7573-VENOM', '7573', 'Stiffness', 'Musculoskeletal system finding'),
('7597-VENOM', '7597', 'Strabismus', 'Ophthalmology/vision finding, Nervous system finding, Head and neck finding'),
('7627-VENOM', '7627', 'Stridor', 'Respiratory system finding, Head and neck finding'),
('2284-VENOM', '2284', 'Tachypnoea', 'Respiratory system finding'),
('2286-VENOM', '2286', 'Testicular/scrotal abnormality', 'Urogenital system finding, Reproductive system finding'),
('7550-VENOM', '7550', 'Thin', 'Weight/Body condition finding'),
('7546-VENOM', '7546', 'Thirst decreased', 'Nervous system finding'),
('98-VENOM', '98', 'Traumatic episode', 'Clinical finding'),
('2288-VENOM', '2288', 'Tremors/shaking/trembling', 'Nervous system finding'),
('189-VENOM', '189', 'Urination abnormal - dysuria', 'Urogenital system finding'),
('2290-VENOM', '2290', 'Urination abnormal - inappropriate urination', 'Urogenital system finding'),
('191-VENOM', '191', 'Urination abnormal - oliguria', 'Urogenital system finding'),
('192-VENOM', '192', 'Urination abnormal - other', 'Urogenital system finding'),
('193-VENOM', '193', 'Urination abnormal - pollakiuria', 'Urogenital system finding'),
('195-VENOM', '195', 'Urine colour abnormal', 'Urogenital system finding'),
('7601-VENOM', '7601', 'Uveal abnormality', 'Ophthalmology/vision finding, Head and neck finding'),
('100-VENOM', '100', 'Vision impaired', 'Ophthalmology/vision finding, Head and neck finding'),
('2292-VENOM', '2292', 'Vocalisation', 'Nervous system finding'),
('5764-VENOM', '5764', 'Vomiting - haematemesis', 'Alimentary tract/Digestive system finding'),
('5766-VENOM', '5766', 'Vomiting - other', 'Alimentary tract/Digestive system finding'),
('103-VENOM', '103', 'Weakness', 'Musculoskeletal system finding'),
('196-VENOM', '196', 'Weight gain', 'Weight/Body condition finding'),
('197-VENOM', '197', 'Weight loss', 'Weight/Body condition finding'),
('105-VENOM', '105', 'Wound', 'Clinical finding');
INSERT INTO lookups (version, linkId, arch_short_name, active, arch_version, code, name, description, default_lookup)
  SELECT
    0,
    UUID(),
    'lookup.presentingComplaintVeNom',
    1,
    '1.0',
    d.code,
    d.term,
    d.modelling,
    0
  FROM venom_presenting_complaint d
  WHERE NOT exists(SELECT
                     *
                   FROM lookups e
                   WHERE e.arch_short_name = 'lookup.presentingComplaintVeNom' AND e.code = d.code);
INSERT INTO lookup_details (lookup_id, name, type, value)
  SELECT
    l.lookup_id,
    'dataDictionaryId',
    'string',
    d.dictionaryId
  FROM lookups l, venom_presenting_complaint d
  WHERE l.arch_short_name = 'lookup.presentingComplaintVeNom' AND l.code = d.code
        AND NOT exists(SELECT
                         *
                       FROM lookup_details d
                       WHERE d.lookup_id = l.lookup_id AND d.name = 'dataDictionaryId');
DROP TABLE venom_presenting_complaint;

#
#  Modify contact.faxNumber archetype to contact.phoneNumber for current users.for OVPMS-1172
#

# Add the FAX lookup.contactPurpose if it doesn't exist.

INSERT INTO lookups (version, linkId, arch_short_name, active, arch_version,
                     code, name, description, default_lookup)
  SELECT
    0,
    UUID(),
    "lookup.contactPurpose",
    1,
    "1.0",
    "FAX",
    "Fax",
    "Fax Contact Purpose",
    0
  FROM dual
  WHERE NOT exists(SELECT
                     *
                   FROM lookups
                   WHERE arch_short_name = "lookup.contactPurpose" AND code = "FAX");

#  Add new purpose to existing fax

INSERT INTO contact_classifications (contact_id, lookup_id)
  SELECT
    contact_id,
    lookup_id
  FROM contacts c, lookups l
  WHERE c.arch_short_name = "contact.faxNumber" AND l.code = "FAX" AND l.arch_short_name = "lookup.contactPurpose"
        AND NOT exists(SELECT
                         *
                       FROM contact_classifications cl
                       WHERE cl.contact_id = c.contact_id AND cl.lookup_id = l.lookup_id);

# Change the detail name to telephoneNumber

UPDATE contacts c, contact_details d
SET d.name = "telephoneNumber"
WHERE c.arch_short_name = "contact.faxNumber" AND c.contact_id = d.contact_id AND d.name = "faxNumber";

# Change the contact archetype

UPDATE contacts
SET arch_short_name = "contact.phoneNumber"
WHERE arch_short_name = "contact.faxNumber";

# Remove contact.faxNumber archetype from the database

DELETE d
FROM assertion_descriptors d, node_descriptors n, archetype_descriptors a
WHERE d.node_desc_id = n.node_desc_id AND n.archetype_desc_id = a.archetype_desc_id
      AND a.name = "contact.faxNumber.1.0";

DELETE a, n
FROM node_descriptors n, archetype_descriptors a
WHERE n.archetype_desc_id = a.archetype_desc_id AND a.name = "contact.faxNumber.1.0";

#
# Security authorities for OVPMS-1523 HL7 Pharmacy Orders, and OVPMS-1624 Add security authorities for 1.8 archetypes
#
DROP TABLE IF EXISTS new_authorities;
CREATE TABLE new_authorities (
  name        VARCHAR(255) PRIMARY KEY,
  description VARCHAR(255),
  method      VARCHAR(255),
  archetype   VARCHAR(255));

INSERT INTO new_authorities (name, description, method, archetype)
  VALUES ("Customer Orders Create", "Authority to Create Customer Orders", "create", "act.customerOrder*"),
  ("Customer Orders Save", "Authority to Save Customer Orders", "save", "act.customerOrder*"),
  ("Customer Orders Remove", "Authority to Remove Customer Orders", "remove", "act.customerOrder*"),
  ("Customer Returns Create", "Authority to Create Customer Returns", "create", "act.customerReturn*"),
  ("Customer Returns Save", "Authority to Save Customer Returns", "save", "act.customerReturn*"),
  ("Customer Returns Remove", "Authority to Remove Customer Returns", "remove", "act.customerReturn*"),
  ("HL7 Message Act Create", "Authority to Create HL7 Message Act", "create", "act.HL7Message"),
  ("HL7 Message Act Save", "Authority to Save HL7 Message Act", "save", "act.HL7Message"),
  ("HL7 Message Act Remove", "Authority to Remove HL7 Message Act", "remove", "act.HL7Message"),
  ("Entity Links Create", "Authority to Create Entity Links", "create", "entityLink.*"),
  ("Entity Links Save", "Authority to Save Entity Links", "save", "entityLink.*"),
  ("Entity Links Remove", "Authority to Remove Entity Links", "remove", "entityLink.*"),
  ("Patient Customer Note Create", "Authority to Create Patient Customer Notes", "create", "act.patientCustomerNote"),
  ("Patient Customer Note Save", "Authority to Save Patient Customer Notes", "save", "act.patientCustomerNote"),
  ("Patient Customer Note Remove", "Authority to Remove Patient Customer Notes", "remove", "act.patientCustomerNote"),
  ("Product Batch Create", "Authority to Create Product Batches", "create", "entity.productBatch"),
  ("Product Batch Save", "Authority to Save Product Batches", "save", "entity.productBatch"),
  ("Product Batch Remove", "Authority to Remove Product Batches", "remove", "entity.productBatch"),
  ("Appointment Series Create", "Authority to Create Appointment Series", "create", "act.customerAppointmentSeries"),
  ("Appointment Series Save", "Authority to Save Appointment Series", "save", "act.customerAppointmentSeries"),
  ("Appointment Series Remove", "Authority to Remove Appointment Series", "remove", "act.customerAppointmentSeries");

INSERT INTO granted_authorities (version, linkId, arch_short_name, arch_version, name, description, active, service_name, method, archetype)
  SELECT
    0,
    UUID(),
    'security.archetypeAuthority',
    "1.0",
    a.name,
    a.description,
    1,
    'archetypeService',
    a.method,
    a.archetype
  FROM new_authorities a
  WHERE NOT exists(
      SELECT
        *
      FROM granted_authorities g
      WHERE g.name = a.name);

INSERT INTO roles_authorities (security_role_id, authority_id)
  SELECT
    r.security_role_id,
    g.granted_authority_id
  FROM security_roles r
    JOIN granted_authorities g
    JOIN new_authorities a
      ON a.name = g.name
  WHERE r.name = "Base Role" AND NOT exists
  (SELECT
     *
   FROM roles_authorities x
   WHERE x.security_role_id = r.security_role_id AND x.authority_id = g.granted_authority_id);

DROP TABLE new_authorities;


# Migration for OVPMS-1555 Add support for multiple To, CC, and BCC addresses when emailing

# The following procedure is used to drop the foreign key from users -> entities.
# Largely from http://stackoverflow.com/a/16717776
DROP PROCEDURE IF EXISTS dropForeignKeysFromTable;

DELIMITER $$
CREATE PROCEDURE dropForeignKeysFromTable(IN param_table_schema VARCHAR(255), IN param_table_name VARCHAR(255))
  BEGIN
    DECLARE done INT DEFAULT FALSE;
    DECLARE dropCommand VARCHAR(255);
    DECLARE dropCur CURSOR FOR
      SELECT
        concat('ALTER TABLE `', table_schema, '`.', table_name, ' DROP FOREIGN KEY ', constraint_name,
               ', DROP KEY ', constraint_name, ';')
      FROM information_schema.table_constraints
      WHERE constraint_type = 'FOREIGN KEY'
            AND table_name = param_table_name
            AND table_schema = param_table_schema;

    DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

    OPEN dropCur;

      read_loop: LOOP
      FETCH dropCur
      INTO dropCommand;
    IF done
    THEN
      LEAVE read_loop;
    END IF;

    SET @sdropCommand = dropCommand;

    PREPARE dropClientUpdateKeyStmt FROM @sdropCommand;

      EXECUTE dropClientUpdateKeyStmt;

      DEALLOCATE PREPARE dropClientUpdateKeyStmt;
    END LOOP;

    CLOSE dropCur;
  END$$

DELIMITER ;

CALL dropForeignKeysFromTable(DATABASE(), 'users');

DROP PROCEDURE IF EXISTS dropForeignKeysFromTable;

INSERT INTO parties (party_id)
  SELECT
    entity_id
  FROM entities
  WHERE arch_short_name = "security.user"
        AND NOT EXISTS(SELECT
                         *
                       FROM parties
                       WHERE party_id = entity_id);

ALTER TABLE users
ADD KEY `FK6A68E0826A12449` (`user_id`),
ADD CONSTRAINT `FK6A68E0826A12449` FOREIGN KEY (`user_id`) REFERENCES `parties` (`party_id`)
  ON DELETE CASCADE;

#
# Replace entityRelationship.productIncludes with entityLink.productIncludes for OVPMS-1559 Nested product templates
#

INSERT INTO entity_links (version, linkId, arch_short_name, arch_version, name, description, active_start_time,
                          active_end_time, sequence, source_id, target_id)
  SELECT
    version,
    linkId,
    "entityLink.productIncludes",
    "1.0",
    name,
    description,
    active_start_time,
    active_end_time,
    sequence,
    source_id,
    target_id
  FROM entity_relationships r
  WHERE r.arch_short_name = "entityRelationship.productIncludes"
        AND NOT exists(
      SELECT
        *
      FROM entity_links l
      WHERE l.source_id = r.source_id
            AND l.target_id = r.target_id
            AND
            (l.active_start_time = r.active_start_time OR (l.active_start_time IS null AND l.active_start_time IS null))
            AND (l.active_end_time = r.active_end_time OR (l.active_end_time IS null AND l.active_end_time IS null))
            AND l.arch_short_name = "entityLink.productIncludes");

INSERT INTO entity_link_details (id, name, type, value)
  SELECT
    l.id,
    "lowQuantity",
    d.type,
    d.value
  FROM entity_relationships r
    JOIN entity_relationship_details d
      ON r.entity_relationship_id = d.entity_relationship_id
         AND d.name = "includeQty"
    JOIN entity_links l
      ON l.arch_short_name = "entityLink.productIncludes"
         AND l.source_id = r.source_id AND l.target_id = r.target_id
         AND l.linkId = r.linkId
  WHERE r.arch_short_name = "entityRelationship.productIncludes"
        AND NOT exists(
      SELECT
        *
      FROM entity_link_details ld
      WHERE ld.id = l.id);

# Copy the lowQuantity to the highQuantity
INSERT INTO entity_link_details (id, name, type, value)
  SELECT
    d.id,
    "highQuantity",
    d.type,
    d.value
  FROM entity_link_details d
    JOIN entity_links l
      ON l.arch_short_name = "entityLink.productIncludes"
         AND d.id = l.id AND d.name = "lowQuantity"
         AND NOT exists(
        SELECT
          *
        FROM entity_link_details e
        WHERE e.id = d.id AND e.name = "highQuantity");

# Remove the old relationships
DELETE d
FROM entity_relationship_details d
  JOIN entity_relationships r
    ON d.entity_relationship_id = r.entity_relationship_id
WHERE r.arch_short_name = "entityRelationship.productIncludes";

DELETE r
FROM entity_relationships r
WHERE r.arch_short_name = "entityRelationship.productIncludes";


#
# Copy discount to lowDiscount, rename discount to highDiscount for OVPMS-1564 Estimates: use separate discounts for
# low and high totals
#

# Copy the discount to the lowDiscount
INSERT INTO act_details (act_id, name, type, value)
  SELECT
    d.act_id,
    "lowDiscount",
    d.type,
    d.value
  FROM act_details d
    JOIN acts a
      ON a.arch_short_name = "act.customerEstimationItem"
         AND d.act_id = a.act_id AND d.name = "discount"
         AND NOT exists(
        SELECT
          *
        FROM act_details e
        WHERE e.act_id = d.act_id AND e.name = "lowDiscount");

# rename discount to highDiscount
UPDATE act_details d
  JOIN acts a
    ON a.arch_short_name = "act.customerEstimationItem" AND a.act_id = d.act_id AND d.name = "discount"
SET d.name = "highDiscount";

#
# OVPMS-1627 Migrate date details nodes to sql-timestamp
#
update act_details d
set d.value=substring(value, 1, 21),
    d.type="sql-timestamp"
where d.type = "date";

update entity_details d
set d.value=substring(value, 1, 21),
    d.type="sql-timestamp"
where d.type = "date";
